% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% FIS analysis, part 1: extract interfaces (sediment, water, air) and save
% Detect 3 interfaces: flume-sediment; sediment-water; water-air for setup image
% pre and post-experiment

clear all; close all; clc
set(0,'DefaultFigureColormap',hot)

% Choose experiment date for analysis and set paths
exp_date = '220901';
pfad = fct_pfad(exp_date);

% Colors for boundary detection visualization
cmap_bnd = flipud(0.5.*[ ...
    0 0 2; 0 0 1; ...
    0 2 0; 0 1 0; ...
    2 0 0; 1 0 0; ...
    0 2 2; 2 0 2; ...
    2 2 0; 1 1 2]);
        
%% Extracting metadata from labbook
% (1) Read in
metadata    = readcell('../labbook.xlsx','Sheet',exp_date);

% (2) Extract values
% (2a) Lengthscale, input time, input concentration
lengthScale = metadata{strcmp(metadata,'Length Scale FIS'),2};    % mm/Pix_Cam 
tZ          = metadata{strcmp(metadata,'Input Experiment'),2};    % Input MP = start
c0          = metadata{strcmp(metadata,'Zugabe MP conc'),2}*1000; % mg/L to g/L

% (2b) Global anchor for cos trafo
trafo_idx   = find(strcmp(metadata,'COS-Trafo FIS - regression'));
x_gl        = metadata{trafo_idx,2};  % m
y_gl        = metadata{trafo_idx,3};  % m
x_loc       = metadata{trafo_idx,4};  % Pix
y_loc       = metadata{trafo_idx,5};  % Pix

% (2c) Add day to input time
exp_datenum  = datenum( exp_date,'yymmdd');
tZ = datetime(tZ + exp_datenum,'ConvertFrom','datenum');

%% Extract Interfaces before and after experiment
%%%%%%%%%%%%%%%%%%%% Detection parameters %%%%%%%%%%%%%%%%%%%%%%%%
threshold_mult = 0.5;  % Intensity gradient to identify edges
strel_radius   = 5;    % Dilate identified edges    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Search for black images, before and after
data_form  = dir([pfad.FIS_Setup, '*form*mat']);
data_names = string({data_form.name});
data_pre   = contains(data_names,'pre');
data_pos   = contains(data_names,'pos');

for idx_load = 1 : numel(data_names)
    disp(data_names{idx_load})
    load(strcat(pfad.FIS_Setup,data_names{idx_load}),'A')
    
    % Convert image to BWs, identifying continuous regions
    % (1) Analyse for sharp edges = intensity gradient
    [~,threshold] = edge(A,'sobel');
    img_edge = edge(A,'sobel',threshold * threshold_mult);
    
    % (2) Dilate edges
    se90 = strel('line',strel_radius,90);
    se0 = strel('line',strel_radius,0);
    img_dilate = imdilate(img_edge,[se90 se0]);
    
    % (3) Fill holes in bw image
    img_fill = imfill(img_dilate,'holes');    
    
    % (4) Final mask
    img_final = img_fill;
    
    
    figure    
    subplot(2,2,1)   
    imshow(img_edge)
    title('Binary Gradient (edge)')
    set(gca,'YDir','normal')
    axis equal
    axis([0 size(A,2) 0 size(A,1)])
        
    subplot(2,2,2)
    imshow(img_dilate)
    title('Dilated Gradient (imdilate)')
    set(gca,'YDir','normal')
    axis equal
    axis([0 size(A,2) 0 size(A,1)])
    
    subplot(2,2,3)
    imshow(img_fill)
    title('Filled dilated gradient (imfill)')
    set(gca,'YDir','normal')
    axis equal
    axis([0 size(A,2) 0 size(A,1)])
    
    
    % Find interfaces = boundaries of filled image
    % (1) Find all boundaries, L is boundary loc with index of B
    [B,L] = bwboundaries(img_final,'noholes'); % B is vector, L is B-indexcoded matrix
    size_B = size(B,1);
    bnd_length = zeros(size_B,1);
    for j = 1 : size(B,1)
        bnd_length(j) = size(B{j},1);
    end
        
    % (2) Take the 10 longest boundaries and extract interfaces
    interfaces = zeros(1,10);
    interface_long = maxk(bnd_length,10);
    for i = 1 : 10
        interfaces(i) = find(bnd_length==interface_long(i));   
    end

    interfaces_name = string(1:10);
    xpix = 1:size(A,2);
    ypix = zeros(1,length(xpix));

    for i = 1 : length(interfaces)        
        % (3) Extract highest point of continuous boundary
        for j = xpix
            y_bnd    = find(L(:,j)==interfaces(i), 1, 'last');
            if isempty(y_bnd)
                y_bnd = nan; % If no value at all, set to nan
            end
            ypix(j)=y_bnd;
        end

        % (4) Exclude nans from boundary locations       
        ypix_li  = find(~isnan(ypix),1,'first');
        ypix_re  = find(~isnan(ypix),1,'last');
        bnd_nan  = [xpix; ypix]';
        bnd_data = bnd_nan(ypix_li:ypix_re,:);

        % (5a) Extrapolate borders if necessary (if nan is set at borders)
        
        no_fit = 10; % Number of points used for interpolation
        
        %  Left side
        xepol_li = (1:ypix_li-1);
        xipol_li = (ypix_li : ypix_li + no_fit);
        %
        fit_li   = polyfit(xipol_li, ypix(xipol_li),1);
        yepol_li = round(polyval(fit_li,xepol_li),0);
    
        % Right side
        xint_re  = (ypix_re - no_fit +1 : ypix_re);
        xepol_re = (ypix_re + 1 :2560);
        % 
        fit_re   = polyfit(xint_re, ypix(xint_re),1);
        yepol_re = round(polyval(fit_re,xepol_re),0);
        
        bnd_epol_i = cat(1,[xepol_li',yepol_li'],bnd_data,[xepol_re',yepol_re']);
        
        
        % (5b) Save individual interfaces (in pixel units)
        bnd_epol{i} = bnd_epol_i;
    end  
    
    
    % Visual check for correct interface identification  
    figure('units','normalized','outerposition',[0 0 1 1])
    hold all
    imagesc(A)
    xlim([0 2560])
    ylim([0 2160])
    colormap("gray")
    for i = 1 : 10
        p = plot(bnd_epol{i}(:,1), bnd_epol{i}(:,2),'DisplayName',interfaces_name(i));
        set(p,'Color',cmap_bnd(i,:),'LineWidth',2)
    end
    hold off
    
    colormap('gray')
    set(gca,'YDir','normal')
    set(gcf,'Visible','on')
    legend('Location','southeast')
    title(sprintf('Interfaces identification %s',data_names{idx_load}),'Interpreter','none')
    
    answer = questdlg({'Interfaces correctly assigned?', ...
                       ['Int 1 is flme->sed, Int 2 is sed->water, ' ...
                        'Int 3 is water->air']}, 'Interfaces', ...
                        'Continue', ...
                        'Reassign Interfaces', ...
                        'Exit: Bad interfaces', ...
                        'Reassign Interfaces');
    % Handle response
    switch answer
        case 'Continue'
            disp('Continuing')
            bnd_flume = bnd_epol{1};
            bnd_sed   = bnd_epol{2};
            bnd_water = bnd_epol{3};
            close(gcf)  
        case 'Exit: Bad interfaces'
            close(gcf)
            return
        case 'Reassign Interfaces'       
            answer1 = inputdlg('Enter space-separated interfaces for flume->sed:',...
                 'Flume-Sediment-Interface', [1 50]);
            interfaces_1 = str2num(answer1{1});
            
            answer2 = inputdlg('Enter space-separated interfaces for sed->water:',...
                 'Sediment-Water-Interface', [1 50]);
            interfaces_2 = str2num(answer2{1});
            
            answer3 = inputdlg('Enter space-separated interfaces for water->air:',...
                 'Water-Air-Interface', [1 50]);
            interfaces_3 = str2num(answer3{1})  ;      
            close(gcf)
            
            % If interface is split, concatenate and sort
            bnd_flume_notunique = sortrows(vertcat(bnd_epol{interfaces_1}));
            bnd_sed_notunique   = sortrows(vertcat(bnd_epol{interfaces_2}));
            bnd_water_notunique = sortrows(vertcat(bnd_epol{interfaces_3}));      
            
            % If duplicated values (x), take median
            [bnd_flume(:,1),~,idx1] = unique(bnd_flume_notunique(:,1),'stable');
            [bnd_sed(:,1),~,idx2]   = unique(bnd_sed_notunique(:,1)  ,'stable');
            [bnd_water(:,1),~,idx3] = unique(bnd_water_notunique(:,1),'stable');
            
            bnd_flume(:,2) =accumarray(idx1,bnd_flume_notunique(:,2),[],@median); 
            bnd_sed(:,2)   =accumarray(idx2,bnd_sed_notunique(:,2)  ,[],@median); 
            bnd_water(:,2) =accumarray(idx3,bnd_water_notunique(:,2),[],@median);              
        otherwise
            errordlg('No interfaces identified. Exiting','Interfaces error')
    end
    
    % Show identified boundaries and check if assignment was correct
    figure
    hold all
    imagesc(A)
    plot(bnd_water(:,1), bnd_water(:,2),'r','LineWidth',2,'DisplayName','Water->Air')
    plot(bnd_sed(:,1),     bnd_sed(:,2),'y','LineWidth',2,'DisplayName','Sed->Water')
    plot(bnd_flume(:,1), bnd_flume(:,2),'g','LineWidth',2,'DisplayName','Flume->Sed')
    hold off
    
    title(sprintf('Form Picture %s with interfaces',data_names{idx_load}),'Interpreter','none')
    colormap("gray")
    set(gca,'YDir','normal')
    set(gcf,'Visible','on')
    axis equal
    axis([0 2048 0 2048])
    legend
    set(gcf,'units','normalized','position',[0.1,0.1,0.5,0.5])
    
    answer = questdlg('Interfaces correctly assigned?', ...
        'Interfaces','Yes','No','Yes');
    % Handle response
    switch answer
        case 'Yes'
            disp('Continuing')
            close(gcf)  
        case 'No'
            disp('Aborting')
            close(gcf)
            return
    end
            
    % Save sediment elevation for comparison
    bnd_sed_comp{idx_load} = bnd_sed; 
    bnd_water_comp{idx_load} = bnd_water; 
    bnd_flume_comp{idx_load} = bnd_flume; 
    A_form_comp{idx_load}   = A;
    
    clear bnd_sed bnd_water bnd_flume A bnd_epol L bnd_epol_i    
end

% Save data into individual matrices
bnd_sed_pre = bnd_sed_comp{data_pre};
bnd_sed_pos = bnd_sed_comp{data_pos};

bnd_water_pre = bnd_water_comp{data_pre};
bnd_water_pos = bnd_water_comp{data_pos};

bnd_flume_pre = bnd_flume_comp{data_pre};
bnd_flume_pos = bnd_flume_comp{data_pos};

A_pre   = A_form_comp{data_pre};
A_pos   = A_form_comp{data_pos};

%% Bedform metrics from image pre/pos experiment
for i = 1 :2
    [crestVal(i,:),   crestLoc(i,:)] = findpeaks( bnd_sed_comp{i}(:,2),'MinPeakProminence',50,'MinPeakDistance',200);
    [troughVal(i,:), troughLoc(i,:)] = findpeaks(-bnd_sed_comp{i}(:,2),'MinPeakProminence',50,'MinPeakDistance',200);
end
troughVal = -troughVal;     % Because of -1 in findpeaks

ripple.lengthTrough = lengthScale * (troughLoc(:,2) - troughLoc(:,1));
ripple.lengthCrest  = lengthScale * (crestLoc( :,2) - crestLoc( :,1));
ripple.height = mean(lengthScale * (crestVal - troughVal),2);
ripple.ident = data_names;

%% Overlay identified SWI and form image
figure
hold all
imagesc(A_pre)
plot(bnd_sed_pre(:,1),bnd_sed_pre(:,2),'g-','LineWidth',2, 'DisplayName','Pre')
plot(bnd_sed_pos(:,1),bnd_sed_pos(:,2),'b-','LineWidth',2, 'DisplayName','Pos')
plot(troughLoc,troughVal,  'mx','LineWidth',2, 'DisplayName','Troughs','MarkerSize',20)
plot(crestLoc  ,crestVal,  'yx','LineWidth',2, 'DisplayName','Crests','MarkerSize',20)
hold off
%
setDiag
legend
title({'SWI before and after experiment, bg img = pre'; ...
    sprintf('Ripple metrics: length = %2.1f mm, height = %2.1f mm',...
    mean(ripple.lengthTrough), mean(ripple.height))})
%
filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Ripples');
print('-r300',filename,'-dpng');

figure
imagesc(A_pre./A_pos)
setDiag
title('Comparison pre:pos experiment (division A_{pre} ./A_{pos})')

%% Save and end
clear bnd_sed_comp bnd_water_comp bnd_flume_comp A_form_comp
close all

fullfilename = strcat(pfad.results,exp_date,'-FIS_interfaces.mat');
save(fullfilename)

msgbox('End of Script reached!','Yeay')