% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% FIS analysis (2a): analyse by averaging over (10 pixel)
clear all; close all; clc
set(0,'DefaultFigureColormap',hot)

% Set experiment date for analysis
exp_date = '220901';
pfad     = fct_pfad(exp_date);

% Define analysis values
dpix     = 10;  % Pixel binning
precal   = 1;   % Precalibration for slope scaling: 1 = consider, 0 = ignore
sigma_g  = 2;   % Gauss-filtering of images for homogenization
export_data = 1; % Export data to csv and gif at the end, 1 = yes, 0=no

%% Load first part of FIS analysis (interfaces file)
load(strcat(pfad.results,exp_date,'-FIS_interfaces.mat'))

% Position anchor in local coordinates (to be binned) 
x_locBin = x_loc / dpix; % Pix / (Pix/Pix_bin) = Pix_bin
y_locBin = y_loc / dpix;

% Lenghtscale in mm and m
lengthScaleBin = lengthScale * dpix; % mm/Pix * Pix/Pix_bin
lengthScaleBinM = lengthScaleBin / 1000; % m/Pix_bin

%% Find blank image = background illumination
% (1) Find info when blank shall be read out
datasets      = dir([pfad.FIS_Exp10,'*mat']);    % possibly badly sorted
datasetsTable = struct2table(datasets);             % convert to table
sortedT       = sortrows(datasetsTable, 'datenum'); % sort the table as for creation
datasets      = table2struct(sortedT);               % convert back to struct
qty_data      = size(datasets,1);                   % Amount of datasets availbale


% Retrieving Cam infos from first file
load(strcat(pfad.FIS_Exp10,datasets(1).name), ...
    'TimeStart','TimeStamp','fps','file_length')

% TimeStamps as 32 bit: avoid int overflow, relevant for hour-long experiments 
delta_t = str2double(TimeStamp); 
[pkvals, pklocs] = findpeaks(delta_t); % Find where wrapping takes place
for k = 1 : numel(pklocs)
    delta_t(pklocs(k)+1:end) = delta_t(pklocs(k)+1:end) + 2^32-1;
end

% Assign time steps
t_Cam0  = datetime(TimeStart, ...
    'InputFormat','eee MMM dd HH:mm:ss yyyy');    % Acquisition start
t_Cam0 = t_Cam0 - hours(round(hours(t_Cam0-tZ))); % Ensure same timezone
t_Cam  = t_Cam0 + seconds(delta_t/10^6);          % Re-create timesteps (stamps in s)
t_CamA = t_Cam;                                   % Save absolute time for comp sifx-file
if isdatetime(t_Cam)
    t_Cam = seconds(t_Cam - tZ);                  % Convert to durations
end


% (2) Define blank: last sub-dataset (last file_length frames) before tCam = 0 
[~,idx_CamBlank] = min(abs(t_Cam));
idx_CamBlank_file = ceil(idx_CamBlank/file_length)-1 ; % Achtung fr 201211
blankFileName = datasets(idx_CamBlank_file).name;
load(sprintf('%s%s',pfad.FIS_Exp10,blankFileName),'A')

A_blank = mean(A,3);
if dpix>1
    sprintf(['Blank image is from file %s, mean from %i images ' ...
        'and gauss filtered with sigma = %i'], ...
        blankFileName,file_length,sigma_g)
    A_blank = imgaussfilt(A_blank,sigma_g);
else
    sprintf(['Blank image is from file %s, mean from %i images, ' ...
        'NOT gauss filtered'], blankFileName,file_length)
end
clear A

% Check blank value: Compare to first datafiles (join together)
qty_check = 50; % Amount of datasets for blank
Amean        = zeros(file_length*qty_check,1);
%
for k = 1 : qty_check    
    if ~mod(k,10)
        fprintf('Current data is %i (%i)\n',k,qty_check)
    end
    load(sprintf('%s%s',pfad.FIS_Exp10,datasets(k).name),'A')  
    Amean((k-1)*file_length+1:k*file_length) = squeeze(mean(A,[1,2]));
    clear A
end

% Graphical check blank value
figure
hold all
plot(t_Cam(1:length(Amean)),Amean,'b')
plot(t_Cam(file_length*(idx_CamBlank_file-[1,0])+[1,0]), ...
    repmat(mean(A_blank,[1 2]),2,1),'r','LineWidth',2) % bnds as in datasets(idx_CamBlank_file).name
hold off
title('Average of complete frame over time with blank')
    legend('Average data','Blank data')


figure
imagesc(A_blank)
setDiag
title('Blank image')

%% Binning blank image and interface locations
% Prealloc, empty matrix for bg img, ascending vector (left-right) for interfaces
A_preBin      = zeros(size(A_pre)/dpix); 
bnd_sed_pix   = repmat((1:size(A_preBin,2))',1,2);
bnd_water_pix = repmat((1:size(A_preBin,2))',1,2);
bnd_flume_pix = repmat((1:size(A_preBin,2))',1,2);


if dpix>1    
    % Binning image
    for j=1:size(A_preBin,2)
        for i = 1:size(A_preBin,1)
            A_preBin(i,j) = mean(A_pre((i-1)*dpix+1:i*dpix, ...
                                       (j-1)*dpix+1:j*dpix,  :),'all');             
        end
    end
    
    % Binning interfaces
    for i = 1 : size(A_preBin,2)
        bnd_flume_pix(i,2) =  ceil(mean(bnd_flume_pre((i-1)*dpix+1 : i*dpix, 2))/dpix);
        bnd_sed_pix(i,2)   = round(mean(bnd_sed_pre(  (i-1)*dpix+1 : i*dpix, 2))/dpix);
        bnd_water_pix(i,2) = floor(mean(bnd_water_pre((i-1)*dpix+1 : i*dpix, 2))/dpix);
    end
else
    % No binning intended, so save some time
    A_preBin = A_pre;
    bnd_flume_pix = bnd_flume_pre;
    bnd_sed_pix   = bnd_sed_pre;
    bnd_water_pix = bnd_water_pre;
end

%% Load Cam Cal Relation, assign interfaces, additional precalibration
% (1) Load Camera calibration relation 
filename = strcat(pfad.results,exp_date, '-FIS_Cal_Relation.mat');
load(filename, ...
    'm_Cam_sed','m_Cam_liq', 'm_ROI', 'inEllipse',...
    'dataCuboid', 'x_boundary','y_boundary','dpix_reg','LOD','LOQ')

% (2) Additional scaling: pre/poscalibration 
if precal == 1
    m_comp = m_ROI(contains(dataCuboid,'comp'));
    m_pre  = m_ROI(contains(dataCuboid,'pre'));
    m_pos  = m_ROI(contains(dataCuboid,'pos'));

    % Scaling factor Precal
    f_precal = m_pre/m_comp;
    f_poscal = m_pos/m_comp;
        
    % Scaled calibration relation
    m_Cam_liq = m_Cam_liq * f_precal;
    m_Cam_sed = m_Cam_sed * f_precal;
end


% (3) Create mask for interfaces information
inSediment = false(size(A_preBin));
inLiquid   = false(size(A_preBin));
for i = 1 : size(A_preBin,2)
    inSediment(bnd_flume_pix(i,2) : bnd_sed_pix(i,2),i) = true;
    inLiquid(bnd_sed_pix(i,2)+1 : bnd_water_pix(i,2),i) = true;
end


% (4) Join validity (high R2, inEllipse) with medium info (Sediment, Liquid)
try
    inSedimentValid = inEllipse & inSediment;
    inLiquidValid   = inEllipse & inLiquid;
catch ME
    inSedimentValid = inSediment;
    inLiquidValid   = inLiquid;
end


% (5) Calibration Relation with medium information 
m_Cam_SedLiq = m_Cam_liq.*inLiquid + m_Cam_sed.*inSediment;


% (6) Output figures
% (6a) Individual masks
figure
subplot(2,2,1)
imagesc(inSediment)
set(gca,'YDir','normal');
title('In Sediment')
%
subplot(2,2,2)
imagesc(inSedimentValid)
set(gca,'YDir','normal');
title('In Sediment and Valid')
%
subplot(2,2,3)
imagesc(inLiquid)
set(gca,'YDir','normal');
title('In Liquid')
%
subplot(2,2,4)
imagesc(inLiquidValid)
set(gca,'YDir','normal');
title('In Liquid and Valid')

% (6b) Final slope for conversion
figure
imagesc(m_Cam_SedLiq)
title('Slope in valid area, sediment and liquid phase')
set(gca,'YDir','normal');
colorbar
set(gca,'ColorScale','log')
%
filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-final_valid');
print('-r300',filename,'-dpng');

%% Define slices for depth dependent bt analysis
% Define slice location at first crest
xloc_Slice = round(crestLoc(2,find(min(abs(crestLoc(2,:)-1000))))/dpix_reg);  % Horizontal loc slices (fix)

% Define slice dimensions
length_x = 1;                            % Breite in (gebinnte) Pixel
length_y = round(10/lengthScaleBin);     % Hhe in   (gebinnte) Pixel fr 10 mm
length_x_mm = length_x * lengthScaleBin;
length_y_mm = length_y * lengthScaleBin;
sprintf(['Slices binned over\nwidth  = %2.1f mm (%i bins)\n' ...
                             'height = %2.1f mm (%i bins)'], ...
    length_x_mm,length_x, length_y_mm, length_y)

% Retrieve all y positions for slices
% Differentiate between surface water (liq) and pore water (sed)
yloc_Slice_sed = find(inSedimentValid(:,xloc_Slice));
yloc_Slice_liq = find(inLiquidValid(  :,xloc_Slice));

% Quantity of slices, without top loc 
qty_slices_sed = floor((length(yloc_Slice_sed)-1)/length_y);
qty_slices_liq = floor((length(yloc_Slice_liq)-2)/length_y);

% Override values for comparison purposes
qty_slices_sed = 12;
qty_slices_liq = 2;

qty_slices = qty_slices_sed + qty_slices_liq;

% Define vectors for slices
start_x = repmat(xloc_Slice,1,qty_slices);
start_y_sed = yloc_Slice_sed(end) - length_y*(0: qty_slices_sed -1);
start_y_liq = yloc_Slice_liq(1)-1 + length_y*(qty_slices_liq:-1:1);
start_y = [start_y_liq, start_y_sed];

yloc_slice = [fliplr(1:qty_slices_liq), -(1:qty_slices_sed)]; % First sediment cut: "-1"

% Creating slice locations
for k = 1 : qty_slices
    slices_x(k,:) = start_x(k): 1: start_x(k) + length_x-1;
    slices_y(k,:) = start_y(k):-1: start_y(k) - length_y+1;
end

%% Load datasets and calculate slice-averaged values
disp('Reading in all FCS data and taking slice averages')

A_slices = zeros(length_y,length_x,file_length*qty_data,qty_slices);

% Define data that shall be read in
for k = 1 : qty_data    
    if ~mod(k,10)
        fprintf('Current data is %d (%d)\n',k,qty_data)
    end
    
    % Load illumination within one datafile
    load(sprintf('%s%s',pfad.FIS_Exp10,datasets(k).name),'A')  ;
    A_filt = imgaussfilt(A,sigma_g);   
    clear A
    
    % Extract illumination for each slice at each pixel 
    for ii = 1 : qty_slices
        A_slices(:,:,file_length*(k-1)+1:k*file_length,ii) = ...
         A_filt(slices_y(ii,:), slices_x(ii,:),:);
    end
end
size(A_slices); % Slice: (avg'd depth, avg'd width, timesteps, nr_slices)


% Calculate concentration pixel-wise and average over slice
I_slice = zeros(size(A_slices)); % Prealloc
c_slice = zeros(size(A_slices)); % Prealloc

for ii = 1 : qty_slices
    I_slice(:,:,:,ii) = A_slices(:,:,:,ii)       - A_blank(slices_y(ii,:),slices_x(ii,:));
    c_slice(:,:,:,ii) =  I_slice(:,:,:,ii) .* m_Cam_SedLiq(slices_y(ii,:),slices_x(ii,:)); 
    
    I_slicemean = squeeze(mean(I_slice,[1 2]));
    c_slicemean = squeeze(mean(c_slice,[1 2]));
end


% Finding peaks in breakthrough curves
for j = 1:qty_slices
    [pkVals, pkLocs] = findpeaks(c_slicemean(:,j),'MinPeakProminence',50);
    pkLocs_slices(j) = pkLocs(1);%end);
    pkVals_slices(j) = pkVals(1);%end);
end
pkVals_slices;
pkLocs_slices;

% Ignoring third slice for bad data, caused by overillumination at SWI and
% imperfect interface location
pkVals_slices(3) = nan;
c_slicemean(:,3) = nan;

figure
hold all
for i = 1 : qty_slices
    plot(t_Cam,c_slicemean(:,i),'DisplayName',num2str(i))
end
hold off
legend

%% Find disturbances (From labbook)
% If disturbances (light in room due to emergency -> problem in FIS data.
% Therefore slicemeans will be set to nan

idx_dist1 = find(strcmp(metadata,'Disturbances FIS-start'));
idx_dist2 = find(strcmp(metadata,'Disturbances FIS-end'));
qty_dist  = idx_dist2 - idx_dist1 - 1; % Quantity of disturbances
sprintf('Attention! %i disturbance(s) during experiment',sum(qty_dist))

for i = 1 : qty_dist
    t_bad          = [metadata{idx_dist1 + i , 2:3}]; 
    t_exclude(i,:) = datetime(t_bad + exp_datenum,'ConvertFrom','datenum');
    
    t_start = find(t_CamA >= t_exclude(i,1),1);
    t_end = find(t_CamA <= t_exclude(i,2),1,'last');  
    
    if isempty(t_start) || isempty(t_end)
        continue
    end
    
    if t_end<t_start
        warning('Disturbance time end lies before time start. Please check')
        continue
    end
    
    c_slicemean(t_start:t_end,:) = nan;
    sprintf('disturbance! data set to nan \nfrom: %s \nto:   %s',t_exclude(i,:))
end

%% Improve breakthrough curves in slices
% First peak is artefact from overillumination in surface water
% Idea: smooth the erroneous peak, scale into depths, and subtract. 
% ADE-fit worked fine to subtract background noise

% (1) Fit surface breakthroughs to ADE
for i = 1:2
    [c_slicefitADE(:,i),ADEFit{i}] = fit_ade(t_Cam(1:size(c_slicemean(:,i),1)), ...
        c_slicemean(:,i),true,[]);    
    ax = gca;
    ax.Title.String = sprintf('FIS slice %i %s', i,ax.Title.String);
    close(gcf)
end

figure
hold all
for i = 1 : 2
    plot(t_Cam(1:size(c_slicefitADE(:,i),1)),c_slicefitADE(:,i)./max(c_slicefitADE(:,i)))
end
hold off
legend('1','2')
set(gca,'XScale','log','YScale','log')
title('Comparison surface breakthroughs FIS')
ylim(10.^[-4,0])



% (2) Take mean of surface breakthrough, scale to pore water and subtract;
% set to zero before peak arrives
c_sliceArtefact = mean(c_slicefitADE(:,1:2),2);
c_sliceArtefact = c_sliceArtefact./max(c_sliceArtefact);
%
c_slicemeanArtefact = zeros(size(c_slicemean));
[~,idx_max] = max(c_sliceArtefact);
c_slicemeanArtefact(:,4:end) = c_slicemean(idx_max,4:end).*c_sliceArtefact;
c_slicemeanImproved = c_slicemean - c_slicemeanArtefact;
c_slicemeanImproved(1:idx_max,3:end) = 0;

figure
hold all
set(gca,'ColorOrder',jet(10),'ColorOrderIndex',1)
plot(c_slicemean(:,4:end),':')
set(gca,'ColorOrder',jet(10),'ColorOrderIndex',1)
plot(c_slicemeanArtefact(:,4:end)+2000,'-')
set(gca,'ColorOrder',jet(10),'ColorOrderIndex',1)
plot(c_slicemeanImproved(:,4:end)+4000,'--')
hold off
leg = legend('NumColumns',3);
leg.Title.String = 'Original    Artefact    Subtracted';


% Ignore slice with nan values
[~,nanrow] = find(isnan(c_slicemean));
nanrow = unique(nanrow);
slices_nonnan = setdiff(1:qty_slices,nanrow);


%% Smooth breakthrough curves
sprm = 10^-5;
clear c_slicefit c_sliceImprovedFit

for i = slices_nonnan % Ignore slice with nan values
    fprintf('Smoothing slice %d (%d)\n',i,size(c_slicemean,2))
    
    [slicefit,~,~] = fit(t_Cam(1:length(c_slicemean)),c_slicemean(:,i), ...
                          'smoothingspline','SmoothingParam',sprm);   
    c_slicefit(:,i) = feval(slicefit,t_Cam(1:length(c_slicemean)));
    
    [slicefit,~,~] = fit(t_Cam(1:length(c_slicemeanImproved)),c_slicemeanImproved(:,i), ...
                          'smoothingspline','SmoothingParam',sprm);   
    c_slicemeanImprovedFit(:,i) = feval(slicefit,t_Cam(1:length(c_slicemeanImproved)));
end
c_slicemeanImprovedFit(:,nanrow) = nan;

%% Plot depth dependent breakthrough curves at crest
% We've got 4 versions: 
% (1) "raw" values,     (2): fit from "raw" values, 
% (3) artefact removed, (4): fit from artifact removed

clear conc filename markers colors
conc(1,:,:) = c_slicemean;
conc(2,:,:) = c_slicefit;
conc(3,:,:) = c_slicemeanImproved;
conc(4,:,:) = c_slicemeanImprovedFit;

filename(1) = string(strcat(pfad.graphics_FIS,exp_date,'-FIS_slices'));
filename(2) = string(strcat(pfad.graphics_FIS,exp_date,'-FIS_slices_fit'));
filename(3) = string(strcat(pfad.graphics_FIS,exp_date,'-FIS_slices_improved'));
filename(4) = string(strcat(pfad.graphics_FIS,exp_date,'-FIS_slices_improved_fit'));

markers{1} = ["-"];
markers{2} = ["-","--","-.",":"];
markers{3} = markers{1};
markers{4} = markers{2};

colors{1} = colormap(jet(12));
colors{2} = [1 0 0; 0 0 1; 0 0.8 0];
colors{3} = colors{1};
colors{4} = colors{2};

% Repeat markers as 1-2-3-4, 1-2-3-4, 1-2-3-4 and
%         colors as 1-1-1-1, 2-2-2-2, 3-3-3-3
for i = 1 : length(colors)
    if length(markers{i})<12
        markers{i} = repmat(markers{i},1,12/size(markers{i},2))';
    end
    if length(colors{i})<12
        colors{i} = reshape(repmat(colors{i}',12/length(colors{i}),1),3,[])';
    end
end


% Define arrow for visualization of coordinatesystem
cos_xl = start_x(1)-20;         % x, left
cos_dx = 10;                    % x, length left to right
cos_xc = cos_xl+cos_dx/2;       % x, center 
cos_xr = cos_xl+cos_dx;         % x, right

cos_yb = start_y_sed(1) + 0.5;  % y, bottom
cos_dy = 15;                    % y, length bottom up
cos_yc = cos_yb + cos_dy/2;     % y, center
cos_yu = cos_yb + cos_dy;       % y, up
    
for i = 1 : length(colors)
    figure
    
    % Main plot (1): breakthrough curves at different depths
    hold all
    for j = 1 : 12%qty_slices
        p(j)=plot(t_Cam(1:size(conc,2)),conc(i,:,j), ...
                  markers{i}(j),'Color', colors{i}(j,:),...
                  'DisplayName',num2str(yloc_slice(j)),'MarkerSize',3);       
    end
    hold off
    %
    leg = legend(p,'NumColumns',3,'Location','east');
    leg.ItemTokenSize = [18,1];
    leg.Position(1:2) = [0.59, 0.42];
    %
    xlim([-0.2 1800])
    ylim(get(gca,'YLim').*[0 1])
    %
    xlabel('Time after MP injection [s]')
    ylabel('Concentration [g/L]')
        
    
    %%%%%%%%%%%%%%%%%%% Inset plot %%%%%%%%%%%%%%%%%%%%%
    % (I1) Image bedform with slice positioning
    axes('Position',[.445 .63 .55 .34]);
    hold all
    imagesc(A_preBin)
    for j = 1 : 12 %qty_slices
        plot(repmat(slices_x(j),1,length_y),slices_y(j,:), ...
            'LineWidth',3, 'Color', colors{i}(j,:));
    end
    
    % (I2) Arrow for coordinatesystem
    plot([cos_xl,cos_xr],repmat(cos_yb,1,2),'Color','w','LineWidth',1)
    plot(repmat(cos_xc,1,2), [cos_yb, cos_yu],'Color','w','LineWidth',1)
    plot([cos_xl,cos_xc,cos_xr],[cos_yc,cos_yu,cos_yc],'Color','w','LineWidth',1)
    text(cos_xl-1.5*cos_dx,cos_yc,'$z$','Color','w','FontWeight','bold','Interpreter','latex')

    % (I3) Add Annotation Surface Flow
    x = [0.86 0.76];
    y = [0.88 0.88];
    an = annotation('textarrow',x,y,'String','flow');
    set(an,'Color','c')  
        
    % (I0) settings
    set(gca,'XLim',[75,175], 'YLim', [45, 145])
    set(gca,'XLim',[60,180], 'YLim', [55, 145])
    set(gca,'YDir','normal','Visible', 'off')
    axis equal
    colormap(gray)
    set(gca,'ColorScale','log')
    
    set(gcf,'units','centimeters','position',[0,0,7.5,7])
    set(findall(gcf,'-property','FontSize'),'FontSize',11) 
    set(findall(gcf,'-property','fontname'),'FontName','helvetica')
    set(gcf, 'InvertHardcopy', 'off')
    set(gcf,'Color',[1,1,1])
    set(gcf, 'renderer', 'painters');
    print('-r300',filename(i),'-dpng');    
end

%% Visualization breakthrough: image concentrations at given times
clear pics c_pics

% Choose amount of pics and distance in images
pics_t0 = 0;                           % Zeitpunkt fr erstes Bild [s]
pics_dt = 120;                         % Abstand fr nchste Bilder [s]

pics_dtvec    = pics_dt*(0:5); % Time difference
pics_dtvec = [0,100,300,500,750,1000];
pics_dtvec = [0,100,300,500,1000,1800];
pics_tvec     = pics_t0    + pics_dtvec;
pics_t1 = pics_tvec(end);

[~,pics_t0idx] = min(abs(t_Cam-pics_t0)); % Index von t0 = start [frames]
[~,pics_t1idx] = min(abs(t_Cam-pics_t1)); % Index von t1 = end [frames]


pics_position = pics_t0idx + pics_dtvec*fps;

% Detect data in which pic is saved and relative position
pics_dataset = floor(pics_position/80)+1;
pics_index  = mod(pics_position-1,file_length)+1;

% See, if next pics position has to be loaded from new data
newLoad = [true, pics_dataset(2:end)-pics_dataset(1:end-1)];

% Load pics and convert to MP concentration
for i = 1 : length(pics_position)
    if newLoad(i)
        fullfilename = sprintf('%s%s',pfad.FIS_Exp10,datasets(pics_dataset(i)).name);
        expData = load(fullfilename,'A');
    end
    pics(:,:,i)   = expData.A(:,:,pics_index(i));
    c_pics(:,:,i) = m_Cam_SedLiq.*imgaussfilt(pics(:,:,i)-A_blank,sigma_g);
end
clear expData pics



% Show breakthrough with concentration colormapping
figure
set(gcf,'units','centimeters','position',[0,0,14,8])
for i = 1 : length(pics_tvec)
    subplot(2,ceil(length(pics_tvec)/2),i)
    if i<=3
        set(gca,'Position',get(gca,'Position')+[-0.1 0 0 0])
    else
        set(gca,'Position',get(gca,'Position')+[-0.1 0.10 0 0])
    end
    hold all
    imagesc(c_pics(1:186,:,i)/c0)
    plot(bnd_sed_pix(:,1),bnd_sed_pix(:,2),'g','LineWidth',2);
    plot(bnd_water_pix(:,1),bnd_water_pix(:,2),'c','LineWidth',2);
    plot(bnd_flume_pix(:,1),bnd_flume_pix(:,2),'k','LineWidth',2);   
    text(10,186,strcat(num2str(pics_tvec(i)),' s'), ...
        'HorizontalAlignment','left','VerticalAlignment','top', ...
        'Color','w');
    hold off
    
    xlim([0 248])
    clims = [0,0.001];
    caxis(clims)
    axis equal
    set(gca,'XColor','none','YColor','none')
    set(gca,'YDir','normal')
end

axes
set(gca,'Visible','off')
cb = colorbar;
cb.Title.String = 'c/c_0';
cb.Title.Units = 'normalized';
cb.Title.Position(2) = 1.04*cb.Title.Position(2);
%cb.Ticks = [0:250:750];
cb.Position = [0.85 0.24 0.04 0.66];
caxis(clims)

set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,14,8])
set(gcf, 'renderer', 'painters');
filename = strcat(pfad.graphics_FIS,exp_date,'-FIS_Snapshots');
print('-r300',filename,'-dpng');

%% Plot: Heatmap - Intensity in vertical slice over time
figure
imagesc(t_Cam,yloc_slice,c_slicemean'./c0);

set(gca,'YDir','normal')
set(gca,'XLim',[-10 900])
set(gca,'YLim',[-12 2])

cb = colorbar;
cb.Title.String = 'c [g/L]';
set(gca,'ColorScale','log')
caxis(10.^[-4 -1])

title('Breakthrough below crest over time')
xlabel('Time after MP input [s]')
ylabel('Depth below sediment-water-interface [cm]')
fullfilename = strcat(pfad.graphics_FIS,exp_date,'-FIS_Heatmap');
print('-r300',fullfilename,'-dpng');

%% All about exporting fields and slices
if export_data    
    %%%%%%% Calculate the fields first %%%% 
    % Fields get averaged over fps images = over 1 second
    qty_SecPerFile = file_length/fps;

    % Prealloc, concat dauert zu lange
    c_ttl = zeros(size(A_blank,1),size(A_blank,2),qty_data*qty_SecPerFile);
    A_ttl = zeros(size(A_blank,1),size(A_blank,2),qty_data*qty_SecPerFile);    

    fprintf('Now loading images for export\n')
    for k = 1 : qty_data    
        if ~mod(k,10)
            fprintf('Current data is %d (%d)\n',k,qty_data)
        end
        
        % Load illumination within one datafile
        load(sprintf('%s%s',pfad.FIS_Exp10,datasets(k).name),'A');
        
        % Workflow as usual: Gaussfiltered and converted to concentration
        % Use mean values over fps = 1 s
        A_new = imgaussfilt(A,sigma_g);   
        c_new = (A_new-A_blank).*m_Cam_SedLiq;
        
        for j = 1 : qty_SecPerFile
            c_new_mean = mean(c_new(:,:,1+fps*(j-1):fps*j),3); 
            A_new_mean = mean(A_new(:,:,1+fps*(j-1):fps*j),3);
            
            c_ttl(:,:,(k-1)*qty_SecPerFile+j) = c_new_mean;
            A_ttl(:,:,(k-1)*qty_SecPerFile+j) = A_new_mean;
        end
        clear A  
    end
    
    
    % Create timestamps every 1 s for export and restrict to analysis
    t_Cam_s = t_Cam(fps/2:fps:end);
    t_Cam_s = t_Cam_s(1:size(c_ttl,3));
    
    % Export only t>0
    idx   = t_Cam_s>0; 
    exp_t = t_Cam_s(idx);
    c     = c_ttl(:,:,idx);
    A     = A_ttl(:,:,idx);
    
    % Create vector for pixel locations, transfer from local to global KOS
    y = 1:size(A_blank,1);
    x = 1:size(A_blank,2);
    x_tf = -(x-x_locBin)*lengthScaleBinM + x_gl; 
    y_tf =  (y-y_locBin)*lengthScaleBinM + y_gl;
    
    % Create (x,y)-Grid for Export: (x1,y1,c11) etc; reshape into column format
    [centersIn,Y] = ndgrid( x_tf,y_tf);
    exp_x = reshape(centersIn,[],1); 
    exp_y = reshape(Y,[],1);


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Export (1): concentration fields
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %{
    answer_concfield = questdlg('Export concentration field?','Concentration?','Yes','No','No');
    switch answer_concfield
        case '' % No answer given
            error('User chose exit')
        case 'Yes'
            fprintf('Exporting concentration fields...')
    
            % Create folder to export to
            pfad.res_concfields  = strcat(pfad.results_10,'concentrationfields/');
            pfad.res_intfields   = strcat(pfad.results_10,'intensityfields/');
            fct_isFolder(pfad);
    
            for i = 1 : length(exp_t)           
                if ~mod(i,10)
                    fprintf('...at File %d (%d)\n',i,length(exp_t))
                end
                        
                exp_c = reshape(permute(c(:,:,i),[2 1]),[],1); % Permute: from (y,x) to (x,y) as in tbl
                tbl_c = [exp_x,exp_y,exp_c];
                
                filename = [pfad.res_concfields,exp_date,'-FIS-conc-field_',num2str(exp_t(i),'%.1f'),'.txt'];           
                fileID = fopen(filename,'w');
                fprintf(fileID,'%% Exp-ID %6s, Results from: Fluorescence-Imaging-System\n',exp_date);
                fprintf(fileID,['%% x = horizontal position, y = vertical position, ' ...
                    'c = MP concentration\n%% t = ',num2str(exp_t(i),'%.4f'),' s \n']); ...
                fprintf(fileID,'%5s\t%5s\t%7s\n','x [m]','y [m]', 'c [g/L]');
                fprintf(fileID,'%5.3f\t%5.3f\t%5.1f\n',tbl_c');
                fclose(fileID);
            end
    end


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Export (2): intensity fields
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    answer_intfield = questdlg('Export Intensity field?','Intensity?','Yes','No','No');
    switch answer_intfield
        case '' % No answer given
            error('User chose exit')
        case 'Yes'
            fprintf('Exporting intensity fields:...')
            for i = 1 : length(exp_t)
                if ~mod(i,10)
                    fprintf('...at file %d (%d)\n',i,length(exp_t))
                end
                exp_A = reshape(permute(A(:,:,i),[2 1]),[],1); % Permute: from (y,x) to (x,y) as in tbl
                tbl_A = [exp_x,exp_y,exp_A];
                
                filename = [pfad.res_intfields,exp_date,'-FIS-int-field_',num2str(exp_t(i),'%.1f'),'.txt'];
                
                fileID = fopen(filename,'w');
                fprintf(fileID,'%% Exp-ID %6s, Results from: Fluorescence-Imaging-System\n',exp_date);
                fprintf(fileID,['%% x = horizontal position, y = vertical position, ' ...
                    'I = Illumination\n%% t = ',num2str(exp_t(i),'%.4f'),' s \n']); ...
                fprintf(fileID,'%5s\t%5s\t%7s\n','x [m]','y [m]', 'I [counts]');
                fprintf(fileID,'%5.3f\t%5.3f\t%5.1f\n',tbl_A');
                fclose(fileID);
            end
    end
    %}
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Export (3): Blank and gradient
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    % Blank
    exp_Bl = reshape(permute(A_blank,[2 1]),[],1); % Permute: from (y,x) to (x,y) as in tbl
    tbl_Bl = [exp_x,exp_y,exp_Bl];
    
    filename = [pfad.results,exp_date,'-FIS_blank','.txt'];
    
    fileID = fopen(filename,'w');
    fprintf(fileID,'%% Exp-ID %6s, Results from: Fluorescence-Imaging-System\n',exp_date);
    fprintf(fileID,['%% x = horizontal position, y = vertical position, ' ...
        'I = Illumination blank\n']); ...
    fprintf(fileID,'%5s\t%5s\t%7s\n','x [m]','y [m]', 'I [counts]');
    fprintf(fileID,'%5.3f\t%5.3f\t%5.1f\n',tbl_Bl');
    fclose(fileID);
    
    % Gradient
    exp_m = reshape(permute(m_Cam_SedLiq,[2 1]),[],1); % Permute: from (y,x) to (x,y) as in tbl
    tbl_m = [exp_x,exp_y,exp_m];
    
    filename = [pfad.results,exp_date,'-FIS_conversion','.txt'];
    
    fileID = fopen(filename,'w');
    fprintf(fileID,'%% Exp-ID %6s, Results from: Fluorescence-Imaging-System\n',exp_date);
    fprintf(fileID,['%% x = horizontal position, y = vertical position, ' ...
        'm = gradient\n']); ...
    fprintf(fileID,'%5s\t%5s\t%7s\n','x [m]','y [m]', 'm [(g/L)/count]');
    fprintf(fileID,'%5.3f\t%5.3f\t%5.1f\n',tbl_m');
    fclose(fileID);
    


        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Export (4): Concentration gif
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%        
    % Export every export_time seconds
    export_time = 60;
    t_Cam_min = find(mod(t_Cam_s,export_time)<1)'; 
    
    filename = strcat(pfad.graphics_FIS,exp_date,'-FIS_Concentration_Fields.gif');
    createGif(filename)
            
    for i = t_Cam_min    
        h = figure;
        imagesc(c_ttl(:,:,i));
        colorbar
        colormap;
        caxis([0 1000]);
        title(sprintf('%.0f s after MP input',t_Cam_s(i)));
        set(gca,'YDir','normal')
                    
        createGif(filename,h,0.2)
    end


    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Export (5): Concentration slices
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    export_profile = [t_Cam(1:length(c_slicemean)), c_slicemean];
    
    filename = strcat(pfad.results,exp_date,'-FIS_slices.dat');    
    fileID = fopen(filename,'w');
    %
    fprintf(fileID,'%% Exp-ID %6s, Results from: Fluorescence-Imaging-System\n',exp_date);
    fprintf(fileID,'%% Uniform t-grid, slice depth = 10 mm, concentration is averaged over slice depth\n');
    fprintf(fileID,'%% Time in [s], concentration in [ug/L], #cm above crest\n');
    fprintf(fileID,['t [s]\t','+2 cm\t+1 cm\t0 cm\t-1 cm\t-2 cm\t-3 cm\t-4 cm\t-5 cm\t-6 cm\t-7 cm\t-8 cm\t-9 cm\t-10 cm\t-11 cm\t-12 cm','\n']);
    fprintf(fileID,['%4.3f\t',repmat('%4.2f\t',1,size(c_slicemean,2)),'\n'],export_profile');
    %
    fclose(fileID);
end 

%% Save
close all
clear A_ttl c_ttl A 

filename = strcat(pfad.results,exp_date,'-FIS_10pix');
save(filename)

msgbox('Script finished','Yeay')
%EOF