% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Comparison experiments (Cam data, averaging (10 pix))
clear all; close all; clc; tic
set(0,'DefaultFigureColormap',hot)

%% Set experiment dates, and informative text for graphics
exp_date = {'220727','220803','210812','220818','220901'}';
exp_text = {'1 m', '3 m', '10 m','fine sand', 'high flow'};
exp_text_long = {
    'a: 1 m, coarse sand, low flow',...
    'b: 3 m, coarse sand, low flow',...
    'c: 10 m, coarse sand, low flow',...
    'd: 1 m, fine sand, low flow',...
    'e: 1 m, coarse sand, high flow'};

% Set times for snapshots infiltration [s]
for i = 1 : length(exp_date)
    pics_tvec{i} = [300,600,1200,1800];
end

% Create movie at the end?
create_movie = 1; %1 = yes, 0 = no

% Set colors to compare the experiments
colors = [255,0,127; 54,53,55; 12,206,107; 0,255,255; 0,0,255]./255;

% Limits colorbar for normalized mass transfer rate
clims = 10.^[-4,-2];

% Choose times for snapshots of infiltration for comparison
t_snap = [300,500,1000,1800]; % Seconds

% Read images for video export video every n seconds & smooth (movmean)
export_time = 10; % Seconds

%% Preparation
% script: extract names, set paths, quantity of experiments
exp_name = string(strcat('exp',exp_date));
pfad     = fct_pfad(char(exp_date));
qty_Exp  = length(exp_date); 

% Preallocate structures for result load
FISdata = struct();

%% Read dataset without Cam 
fprintf('Reading dataset without cam\n')
for i = 1:qty_Exp
    expFile = strcat(pfad.results(i,:),exp_date{i},'-exp-wo-cam.mat');
    exp_i = ['exp',exp_date{i}];
    temp = load(expFile,'-mat','m_zug','Qfit');
    m0(i) = temp.m_zug;    
    Q(i) = mean(temp.Qfit);
end
clear temp

%% Read FIS limits of quantification
fprintf('Reading FIS limits of quantification\n')
for i = 1 : qty_Exp
    expFile = strcat(pfad.results(i,:),exp_date{i},'-FIS_Cal_Relation.mat');
    exp_i = ['exp',exp_date{i}];
    temp = load(expFile,'-mat', 'LOD','LOQ','dataAll');
    
    LOQ(:,i)   = temp.LOQ;
    LOQ_Aq(i)  = temp.LOQ(end);
    LOQ_sed(i) = temp.LOQ(end-1);
    
    % Save centrally in FISdata structure
    f = fieldnames(temp);
    for j = 1:length(f)
        FISdata.(exp_i).(f{j}) = temp.(f{j});
    end    
end
clear temp

%% Read FIS concentration fields
fprintf('Reading FIS concentration fields\n')
for i = 1:qty_Exp
    expFile = strcat(pfad.results(i,:),exp_date{i},'-FIS_10pix.mat');
    exp_i = ['exp',exp_date{i}];
    temp = load(expFile,'-mat','c','exp_t');
    
    % Export only every # seconds
    idx = find(mod(temp.exp_t,export_time)<1)';    % Find timepoints  
    
    % Calculate moving mean
    temp.c_Moving = movmean(temp.c,export_time,3);
     
    % Save data from temp
    FISdata.(exp_i).t        = temp.exp_t(idx);
    FISdata.(exp_i).c        = temp.c(:,:,idx);
    FISdata.(exp_i).c_Moving = temp.c_Moving(:,:,idx);
   
    % Restrict to concentrations above LOQ from Aquarium
    idx = FISdata.(exp_i).c_Moving<LOQ_Aq(i);
    FISdata.(exp_i).c(idx)        = nan;
    FISdata.(exp_i).c_Moving(idx) = nan;
    
    % Normalize concentration
    FISdata.(exp_i).rtd        = FISdata.(exp_i).c        .*Q(i) ./m0(i);
    FISdata.(exp_i).rtd_Moving = FISdata.(exp_i).c_Moving .*Q(i) ./m0(i);   
    clear temp
end

%% Einlesen Cam Data
fprintf('Reading FIS boundaries\n')
for i = 1 : qty_Exp
    expFile = strcat(pfad.results(i,:),exp_date{i},'-FIS_10pix.mat');
    exp_i = ['exp',exp_date{i}];
    temp = load(expFile,'-mat', ...
        'c0','c_slicemeanImproved','t_Cam','yloc_slice', ...
        'bnd_sed_pix', 'bnd_water_pix');
  
    c0{i}            = temp.c0;
    c_mean{i}        = temp.c_slicemeanImproved';
    t{i}             = temp.t_Cam;
    y{i}             = temp.yloc_slice; 
    bnd_sed_pix{i}   = temp.bnd_sed_pix;
    bnd_water_pix{i} = temp.bnd_water_pix;
    
    % Save centrally in FISdata structure
    f = fieldnames(temp);
    for j = 1:length(f)
        FISdata.(exp_i).(f{j}) = temp.(f{j});
    end  
end 
clear temp

% Make sure all slices have same ylocs
y_all = reshape(cell2mat(y'),qty_Exp,[]);
y0 = mean(y_all,1);
if nnz(var(y_all,0,1))
    y_all;
    error('y locations of slices are non uniform')
end

% Rename slice 0: "0" = 0-1 cm, "1" = 1-2 cm etc
y0 = 2:-1:-11;

%% Show infiltration depths (heatmap)
fprintf('Infiltration depth over time (heatmap)\n')
% Extract plume front arrival and plume center arrival
prcStart  =  5;
prcMiddle = 50;
prcEnd    = 95;

% Prealloc
plumeStart  = nan(size(y{i},2),qty_Exp);
plumeMiddle = nan(size(y{i},2),qty_Exp);
plumeEnd    = nan(size(y{i},2),qty_Exp);

for i = 1:qty_Exp  
    % MP is detected if over limit of quantification; delete lower values
    hasMP{i} = c_mean{i} > LOQ_Aq(i);
    
    c_meanHigh = c_mean{i};
    c_meanHigh(~hasMP{i}) = nan;
    
    % Find all indices when MP was detected
    [row{i},col{i}] = ind2sub(size(hasMP{i}),find(hasMP{i}));
        
    % For all depths, take average infiltration time
    for j = 4:14
        % Find all times at given depth j
        idx = col{i}(row{i}==j); 
        t_MP  = t{i}(idx);
        
        qty_hasMPInDepth(j,i) = length(idx);
        
        % Remove outliers
        [t_MP,idx_rm] = rmoutliers(t_MP,'ThresholdFactor',1);
  
        % Find start, middle, and end of MP detection = plume in depth
        plumeStart(j,i)  = prctile(t_MP, prcStart);
        plumeMiddle(j,i) = prctile(t_MP, prcMiddle);
        plumeEnd(j,i)    = prctile(t_MP, prcEnd);
    end
    
    % Overlay plume arrival percentiles with heatmap
    figure
    hold all
    imagesc(t{i},y0,c_meanHigh);
    plot(plumeStart(:,i),y0,'c','LineWidth',2)
    plot(plumeMiddle(:,i),y0,'w','LineWidth',2)
    plot(plumeEnd(:,i),y0,'c','LineWidth',2)
    hold off
    %
    set(gca,'YDir','normal')
    set(gca,'XLim',[-10 1800])
    set(gca,'YLim',[-11 2])
    %
    cb = colorbar;
    cb.Title.String = 'c [g/L]';
    set(gca,'ColorScale','log')
    caxis(10.^[1,4])
    %
    %title(strcat(exp_text_MP{i},' Breakthrough below crest over time'))
    xlabel('Time after MP input [s]')
    ylabel('Depth below ripple crest [cm]')
    %
    set(findall(gcf,'-property','FontSize'),'FontSize',11) 
    set(findall(gcf,'-property','fontname'),'FontName','helvetica')
    set(gcf,'units','centimeters','position',[0,0,7.5,8])
    filename = char(strcat(pfad.graphics_FIS(i,:),exp_date(i,:),'-FIS_infiltrationDepth'));
    print('-r300',filename,'-dpng');
end

%% Comparison infiltration depths (heatmap)
% Show at chosen depths: Not all, since extremely noisy in deeper layers
% for some experiments
%showDepths = {4:14,4:14,4:14,4:14,4:14}
showDepths = {4:14,4:13,4:10,[4,6,7,8],4:13};

% Maximum infiltration depth (ignoring outliers)
idxDepth = cellfun(@max,showDepths);
maxDepth = y0(idxDepth);
for i = 1 : 5
    maxDepthArrival(1,i) = plumeStart(idxDepth(i),i);
end

% Print some results
res{1} = sprintf('Experiments: %s, %s, %s, %s, %s',exp_text{:});
res{2} = sprintf('Onset of infiltration [s] at %.0f, %.0f, %.0f, %.0f, %.0f',plumeStart(4,:));
res{3} = sprintf('Delay of infiltration [s] at %.0f, %.0f, %.0f, %.0f, %.0f',plumeStart(4,:)-plumeStart(4,1));
res{4} = sprintf('Max infiltration depth [cm] at %i, %i, %i, %i, %i',maxDepth);
res{5} = sprintf('Arrival at max depth [s] %.0f, %.0f, %.0f, %.0f, %.0f',maxDepthArrival);
msgbox(res,'Infiltration front')


%%%%%%%%%%%%%% Figure 1: Varying diameter: 1 m vs 3 m and 10 m
showExps = [1,2,3];
lineStyle = {'-','--',':','--',':'};
clear p
figure

hold all
for j = 1:length(showExps)
    i = showExps(j);
    
    fill([plumeStart(showDepths{i},i); flipud(plumeEnd(showDepths{i},i))], ...
         [y0(showDepths{i}),       fliplr(y0(showDepths{i}))]', ...
         colors(i,:), 'FaceColor', colors(i,:),'EdgeColor', colors(i,:), ...
         'LineStyle',lineStyle{i},'LineWidth',1,...
        'FaceAlpha', 0.1,'EdgeAlpha',0.5);
    
    p(j) = plot(plumeStart(showDepths{i},i), y0(showDepths{i}), ...
        'LineStyle',lineStyle{i}, 'Color',colors(i,:),'LineWidth',1,'DisplayName',exp_text{i});
end
hold off
leg = legend(p);
leg.Position = [0.38,0.88,0.4,0.1];
leg.ItemTokenSize = [11,2];
leg.NumColumns = 3;

set(gca,'XLim',[-10 1800],'YLim',[-11.5 1.5])
xlabel('Time after MP input [s]')
ylabel('Depth below ripple crest [cm]')
set(gca,'Position',[0.165,0.165,0.83,0.83])
box on
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,7.5,7])
filename = strcat(pfad.comparison,'FIS_infiltrationDepths_diamMP.png');
print('-r300',filename,'-dpng');


%%%%%%%%%%%% Figure 2: 1 m vs. fine sand and high-flow
showExps = [1,4,5];
clear p

figure
hold all
for j = 1:length(showExps)
    i = showExps(j);
    
    fill([plumeStart(showDepths{i},i); flipud(plumeEnd(showDepths{i},i))], ...
         [y0(showDepths{i}),       fliplr(y0(showDepths{i}))]', ...
         colors(i,:), 'FaceColor', colors(i,:),'EdgeColor', colors(i,:), ...
         'LineStyle',lineStyle{i},'LineWidth',1,...
        'FaceAlpha', 0.1,'EdgeAlpha',0.5);
    
    p(j) = plot(plumeStart(showDepths{i},i), y0(showDepths{i}), ...
        'LineStyle',lineStyle{i}, 'Color',colors(i,:),'LineWidth',1,'DisplayName',exp_text{i});
end
hold off
leg = legend(p);
leg.Position = [0.38,0.88,0.4,0.1];
leg.ItemTokenSize = [11,2];
leg.NumColumns = 3;

set(gca,'XLim',[-10 1800],'YLim',[-11.5 1.5])
xlabel('Time after MP input [s]')
ylabel('Depth below ripple crest [cm]')

set(gca,'Position',[0.165,0.165,0.83,0.83])
box on
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,7.5,7])
filename = strcat(pfad.comparison,'FIS_infiltrationDepths_HD.png');
print('-r300',filename,'-dpng');
  
%% Calculate vertical propagation velocities + figure
fprintf('Vertical propagation velocities\n')
% Propagation velocity
vel_prop = 10 ./ diff(plumeStart); % mm / deltaSeconds

% Figure 1: Varying diameter
showExps = [1,2,3];
clear p
markers = {'x','^','o','s','d'};

figure
hold all
for j = 1:length(showExps)
    i = showExps(j);
    
    % Fitting exponentially: --> log-transform, then linear model, and exp
    xdata = vel_prop(showDepths{i}(1:end-1),i);
    ydata = y0(showDepths{i}(1:end-1));
    %
    exp_model = fitlm(ydata,log(xdata));
    
    p(j) = scatter(xdata, ydata, 'Marker',markers{i},...
        'MarkerEdgeColor',colors(i,:),'LineWidth',0.5,'DisplayName',...
         sprintf('%s',exp_text{i}));
    plot(exp(exp_model.Fitted), ydata, ...
        'LineStyle',lineStyle{i},'Color',colors(i,:),'LineWidth',1,'DisplayName',exp_text{i})
end
hold off

leg = legend(p);
leg.Position = [0.38,0.88,0.4,0.1];
leg.ItemTokenSize = [8,2];
leg.NumColumns = 3;

set(gca,'XLim',[-0.05 1.15])
set(gca,'YLim',[-11.5 1.5])
set(gca,'XLim',10.^[-2,0.1],'XScale','log')
xticks(10.^(-2:0))
xlabel('Propagation velocity [mm/s]')
ylabel('Depth below ripple crest [cm]')
yticks(-12:2:2)
set(gca,'Position',[0.165,0.165,0.83,0.83])
box on
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,7.5,8])
filename = strcat(pfad.comparison,'FIS_propagationVelocity1.png');
print('-r300',filename,'-dpng');


% Figure 2: Transport conditions
showExps = [1,4,5];
clear p

figure
hold all
for j = 1:length(showExps)
    i = showExps(j);
    
    % Fitting exponentially: --> log-transform, then linear model, and exp
    xdata = vel_prop(showDepths{i}(1:end-1),i);
    ydata = y0(showDepths{i}(1:end-1));    
    %
    exp_model = fitlm(ydata,log(xdata));
    
    p(j) = scatter(xdata, ydata, 'Marker',markers{i},...
        'MarkerEdgeColor',colors(i,:),'LineWidth',0.5,'DisplayName',...
         sprintf('%s',exp_text{i}));
    plot(exp(exp_model.Fitted), ydata, ...
        'LineStyle',lineStyle{i},'Color',colors(i,:),'LineWidth',1,'DisplayName',exp_text{i});
end
hold off
%
leg = legend(p);
leg.Position = [0.38,0.88,0.4,0.1];
leg.ItemTokenSize = [8,2];
leg.NumColumns = 3;
%
set(gca,'XLim',[-0.05 1.15])
set(gca,'YLim',[-11.5 1.5])
set(gca,'XLim',10.^[-2,0.1],'XScale','log')
%
xticks(10.^(-2:0))
xlabel('Propagation velocity [mm/s]')
ylabel('Depth below ripple crest [cm]')
yticks(-12:2:2)
%
set(gca,'Position',[0.165,0.165,0.83,0.83])
box on
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,7.5,8])
%
filename = strcat(pfad.comparison,'FIS_propagationVelocity2.png');
print('-r300',filename,'-dpng');

%% Comparison snapshots of infiltration
fprintf('Compare infiltration snapshots - figure\n')
% Show breakthrough with concentration colormapping
idx_snap = t_snap/export_time+1;
qty_snap = length(t_snap);

figure('Name','cpics')
set(gcf,'units','centimeters','position',[0,0,14,8])
for i = 1 : qty_Exp
    exp_i = ['exp',exp_date{i}];
    for j = 1:qty_snap
        subplot(qty_Exp,qty_snap,(i-1)*qty_snap+j)

        set(gca,'Position',get(gca,'Position')+[-0.02 0 -0.02 0])
        hold all
        imagesc(FISdata.(exp_i).rtd(:,:,idx_snap(j)))
        plot(bnd_sed_pix{i}(:,1),  bnd_sed_pix{i}(:,2)  ,'c-', 'LineWidth',1.5);
        plot(bnd_water_pix{i}(:,1),bnd_water_pix{i}(:,2),'c--','LineWidth',1.5);
       
        % Annotation texts
        % (Text1) first row: info of timestamps of snapshots
        if i==1
            text(135,200,strcat(num2str(pics_tvec{i}(j)),' s'), ...
               'HorizontalAlignment','center','VerticalAlignment','top', ...
               'Color','r','FontWeight','bold');
        end
        
        % (Text2) left column: Experiment identification
        if j==1
            text(-125,108,exp_text{i}, ...
               'HorizontalAlignment','left','VerticalAlignment','middle', ...
               'Color','r','FontWeight','bold');
        end
       
        % Add surface flow direction
        if j == 1
            q = quiver(210,125,-70,0,'Color',[0 0.5 1],'LineWidth',1,'MaxHeadSize',0.8);
        end       
        hold off
       
        xlim([120,220])
        ylim([50,150])
        colormap([flipud(fct_cmap(128,0,1,[])); fct_cmap(128,0,[],1)])
        colormap(hot(256))
        caxis(clims)
        
        axis equal      
        set(gca,'XColor','none','YColor','none')
        set(gca,'YDir','normal')
        set(gca,'ColorScale','log')
        box on
    end
    
end

% Add colorbar to the right end of the graphic
axes
set(gca,'Visible','off')
cb = colorbar;
cb.Title.String = '$\displaystyle\frac{c\cdot Q}{m_0 }[s^{-1}]$';
cb.Title.Interpreter = 'latex';
cb.Title.Units = 'normalized';
cb.Title.Position = cb.Title.Position + [0.4,0.04,0];

xticklocks = 10.^(-5:0);
cb.XTick = xticklocks;
cb.TickLabels = compose('10^{%i}',log10(xticklocks));

cb.Position = [0.90 0.24 0.04 0.50];
set(gca,'ColorScale','log')
caxis(clims)

set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf,'units','centimeters','position',[0,0,16,12])
set(gcf, 'renderer', 'painters');

filename = strcat(pfad.comparison,'FIS_Snapshots_Compare.png');
print('-r300',filename,'-dpng');

%% Create Movie
if create_movie 
    fprintf('Creating movie\n')
    showImages = 1: (1800 / export_time)+1;    
    
    videoStart     = showImages(1);
    videoEnd       = showImages(end);
    videoStep      = showImages(2)-showImages(1);
    numberOfFrames = length(showImages);
    
    
    % Set up the movie structure
    % Create cell array with all frames.
    allTheFrames = cell(numberOfFrames,1);
    vidHeight = 2160;
    vidWidth  = 2160;
    allTheFrames(:) = {zeros(vidHeight, vidWidth, 3, 'uint8')};
    
    % Create cell array with all colormaps.
    allTheColorMaps = cell(numberOfFrames,1);
    allTheColorMaps(:) = {zeros(256, 3)};
    
    % Preallocate movie as array of structures.
    myMovie = struct('cdata', allTheFrames, 'colormap', allTheColorMaps);
    
    % Set up figure for frames
    figure('units','pixels','position',[100,100,1280,720])
    set(gcf, 'renderer', 'painters');
    set(gcf,'Visible','off')
    
    
    % Create the movie.
    for k = 1 : numberOfFrames
        j = showImages(k);
        frameIndex = videoStart + (k-1) * videoStep;
        if k==1 || ~mod(k,5)
            fprintf('Frame %i = Img %i (%i)\n',frameIndex,k, numberOfFrames)
        end
           
        cla reset
        
        for i = 1 : qty_Exp
            exp_i = ['exp',exp_date{i}];
            
            subplot(2,3,i)
            imagesc(FISdata.(exp_i).rtd_Moving(:,:,j));
            %
            caxis(clims);
            set(gca,'ColorScale','log')
            %
            xlim([1,216])
            ylim([1,216])
            xticks([])
            yticks([])
            %
            title(sprintf('%s',exp_text_long{i}));
            set(gca,'YDir','normal')
            axis equal
            %
            ax = gca;
            ax.Position(2) = ax.Position(2) - 0.08;               
        end
    
        
        % Add colorbar to lower right corner of image     
        subplot(2,3,6)
        set(gca,'Visible','off')
        cb = colorbar;
        
        caxis(clims)
        set(gca,'ColorScale','log')
        
        xticklocks = 10.^(-5:0);
        cb.XTick = xticklocks;
        cb.TickLabels = compose('10^{%i}',log10(xticklocks));
                
        cb.Title.String = '$\displaystyle\frac{c\cdot Q}{m_0 }[s^{-1}]$';
        cb.Title.Interpreter = 'latex';
        
        cb.Title.Position = [0,180,0];
        cb.Position = [0.75 0.04 0.02 0.30];
    
        % Add title and time
        timeStringHeader = {'Time after','MPs input:'};
        timeString = sprintf('t = %.0f s',FISdata.(exp_i).t(j));
        titleString = {['Microplastic transfer between surface water and the ' ...
            'streambed sediments, varying'], ...
            '(a-c) particle size, (d) streambed grain size, and (e) discharge'};
        cbString = {'Normalized mass','transfer rate'};
        jpbString = 'Boos et al., 2023';
        %
        text(280,-40,timeString, 'HorizontalAlignment','right','Color','r','Units','pixels');           
        text(230,10,timeStringHeader, 'HorizontalAlignment','center','Color','r','Units','pixels');           
        text_jpb = text(-800,-50,jpbString, 'HorizontalAlignment','center','Units','pixels');           
       
        text_cbString = text(220,220,cbString, 'HorizontalAlignment','center','Units','pixels');           
    
        text_title = text(-250,610,titleString,'HorizontalAlignment','center','Units','pixels');
    
        set(findall(gcf,'-property','FontSize'),'FontSize',16) ;
        set(findall(gcf,'-property','FontWeight'),'FontWeight','bold') ;
        set(findall(gcf,'-property','fontname'),'FontName','helvetica');
        
        text_jpb.FontSize = 12;
        text_title.FontSize = 20;
        text_cbString.FontWeight = 'normal';
        drawnow;
        thisFrame = getframe(gcf);
        myMovie(k) = thisFrame;
        %set(gcf,'Visible','on')
    end
    
    figure
    imshow(thisFrame.cdata)
end

%% Write Movie to disk
if create_movie    
    profile = 'MPEG-4';
    VideoFileName = strcat(pfad.comparison,'All_Experiments-Video.mp4');
    delete(VideoFileName)
    writerObj = VideoWriter(VideoFileName, profile);
    writerObj.FrameRate = 5; % 5 fps
    writerObj.Quality = 100;
    open(writerObj);
    
    % Repeat first frame for some time
    frame1 = myMovie(1);
    for v = 1 : 5*writerObj.FrameRate % For the first 5 seconds
        writeVideo(writerObj, frame1);
    end
    
    % Write out all other frames.
    for frameNumber = 1 : numberOfFrames 
        writeVideo(writerObj, myMovie(frameNumber));
    end
    
    % Stay by last frame
    for v = 1 : 5*writerObj.FrameRate % For 5 seconds
        writeVideo(writerObj, myMovie(numberOfFrames));
    end
    close(writerObj);
end

msgbox('Script finished','Yeay')
%EOF