% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% FIS analysis (2b): Individual partikel tracking (no binning)
%%%%%%%%%%%%%%%%%%
% Needs additional functions from File Exchange:
% hatchfill2 (Kesh Ikuma) and legendflex (Kelly Kearney)
%%%%%%%%%%%%%%%%%%

%% Define funcions of script
clear all; close all; clc

% Experiment date: String, yyMMdd
exp_date = '210812';

% Create gif with raw images highlighting light sources (-> particles)
gif_rawimages = 1; % 1 = yes, 0 = no

% Set experiment duration
experiment_duration = 30; % Minutes

% Averaging particle numbers in sediment over # seconds
movmean_sec = 10;
movmean_cog = 60; % Visualization is every minute = 60 seconds

% Minimum of detected particles to visualize infiltration boundaries
n_min = 20;

% Visualization of particles, boundaries, and centers of gravity
img_no = 11; % Background image: at 10 min = 11th image

%% Load interfaces
pfad = fct_pfad(exp_date);
filename = strcat(pfad.results,exp_date,'-FIS_interfaces.mat');
load(filename)

%% Find blank image
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (1) Find info when blank shall be read out
datasets      = dir([pfad.FIS_Exp01,'*mat']);    % possibly badly sorted
datasetsTable = struct2table(datasets);             % convert to table
sortedT       = sortrows(datasetsTable, 'datenum'); % sort the table as for creation
datasets      = table2struct(sortedT);               % convert back to struct
qty_data      = size(datasets,1);                   % Amount of datasets availbale


% Retrieving Cam infos from first file
% time + file_length = amount of images per file
load(strcat(pfad.FIS_Exp01,datasets(1).name), ...
    'TimeStart','TimeStamp','fps','file_length')


% TimeStamps as 32 bit: avoid int overflow, relevant for hour-long experiments 
delta_t = str2double(TimeStamp); 
[pkvals, pklocs] = findpeaks(delta_t); % Find where wrapping takes place
for k = 1 : numel(pklocs)
    delta_t(pklocs(k)+1:end) = delta_t(pklocs(k)+1:end) + 2^32-1;
end

% Assign time steps
t_Cam0  = datetime(TimeStart, ...
    'InputFormat','eee MMM dd HH:mm:ss yyyy');    % Acquisition start
t_Cam0 = t_Cam0 - hours(round(hours(t_Cam0-tZ))); % Ensure same timezone
t_Cam  = t_Cam0 + seconds(delta_t/10^6);         % Re-create timesteps (stamps in s)
t_CamA = t_Cam;                                  % Save absolute time for comp sifx-file
if isdatetime(t_Cam)
    t_Cam = seconds(t_Cam - tZ);                 % Convert to durations
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% (2) Define blank image = background illumination
% Blank image: last sub-dataset (last file_length frames) before tCam = 0 
[~,idx_Cam0] = min(abs(t_Cam));
idx_CamBlank_file = ceil(idx_Cam0/file_length)-1 ; % Achtung fr 201211
blankFileName = datasets(idx_CamBlank_file).name;

load(sprintf('%s%s',pfad.FIS_Exp01,blankFileName),'A')
A_blank = mean(A,3);
clear A

% Check blank value: Compare to first datafiles (join together)
qty_check = 50; % Amount of datasets for blank
Amean        = nan(file_length*qty_check,1);
fprintf('\nChecking blank value:\nreading in first values\n')
%
for k = 1:5:qty_check
    fprintf('Current data is %i (%i)\n',k, qty_check)
    load(sprintf('%s%s',pfad.FIS_Exp01,datasets(k).name),'A')  
    Amean((k-1)*file_length+1:k*file_length) = squeeze(mean(A,[1,2]));
    clear A
end

% Graphical check blank value
figure
hold all
plot(t_Cam(1:length(Amean)),Amean,'b')
plot(t_Cam(file_length*(idx_CamBlank_file-[1,0])+[1,0]), ...
    repmat(mean(A_blank,[1 2]),2,1),'r','LineWidth',2) % bnds as in datasets(idx_CamBlank_file).name
hold off
title('Average of complete frame over time with blank')
    legend('Average data','Blank data')


figure
imagesc(A_blank)
setDiag
title('Blank image')

%% Read in image data
% Basis at t = 0, take one image every minute
idx_read = idx_Cam0 : 60*fps: length(t_Cam);
idx_file = ceil(idx_read/file_length);
idx_data = mod( idx_read,file_length);
qty_dataread = length(idx_read);

A_part = zeros(size(A_blank,1),size(A_blank,2),qty_dataread);
fprintf('\nReading one image every minute\n')

for i = 1:qty_dataread
    fprintf('Current data is %d (%d)\n',i,qty_dataread)
    
    % Load illumination within one datafile at one timestep
    filename = sprintf('%s%s',pfad.FIS_Exp01,datasets(idx_file(i)).name);
    load(filename,'A')

    A_part(:,:,i) = A(:,:,idx_data(i));
    clear A
end

min_end = 1 + experiment_duration; % 0 min, and all other minutes

%% Gif: Raw data over time
if gif_rawimages
    filename = strcat(pfad.graphics_FIS,exp_date,'-FIS_IndParticles_RawOverTime.gif');
    createGif(filename)

    for i = 1:qty_dataread
        h = figure('Units','normalized','OuterPosition',[0 0 1 1]);
        hold all
        imagesc(A_part(:,:,i)./A_blank);
        plot(bnd_sed_pre(:,1), bnd_sed_pre(:,2),'r','LineWidth',2)
        hold off
        
        setDiag
        cb = colorbar;
        cb.Title.String = 'Intensity';
        caxis([1 1.1])
        set(gcf,'Visible','on')
        title(sprintf('%.0f s after MP input',t_Cam(idx_read(i))))
        
        createGif(filename,h)
    end
end

%% Improve sediment-water-interface: Analyse at max illumination
%%%%%%%%%%%% (1) Obtain new mask %%%%%%%%%%%%
% (1a) Find image with peak intensity
[~, maxIdx] = max(squeeze(mean(A_part,[1 2])));
img = A_part(:,:,maxIdx);

% (1b) Analyse for highest intensities, binarize and erode
W = graydiffweight(img, 120, 'GrayDifferenceCutoff', 100);
mask = W < 10;
mask = imerode(mask,strel('disk',2)); % Erode a bit

% (1c) Display new mask
figure
subplot(1,2,1)
imagesc(W)
setDiag
set(gca,'ColorScale','log')
caxis(10.^[0,5])
title('Weights')
%
subplot(1,2,2)
imagesc(mask)
setDiag
title('New mask for SWI')


%%%%%%%%%%%% (2) Extract SWI boundary %%%%%%%%%%%%
% (2a) Extract all boundary locations, take longest bnd
B = bwboundaries(mask,'noholes');
B_len = cellfun(@(x) size(x,1), B);
[~,idx] = max(B_len);
bnd = fliplr(B{idx}); % For (x,y) coordinates

% (2b) Reduce to lower interface (remove water-air-interface)
idxBndTooHigh = bnd(:,2) > max(crestVal(1,:));
bnd(idxBndTooHigh,:)=[];

% (2c) Take mean heights for each (x) position
clear bnd_unique
[bnd_unique(:,1),~,idx1]  = unique(bnd(:,1),'stable');
bnd_unique( :,2) = accumarray(idx1,bnd(:,2),[],@mean); 

% (2d) Extrapolate to full image 
bnd_epol(:,1) = 1:size(A_pre,2);
bnd_epol(:,2) = interp1(bnd_unique(:,1),bnd_unique(:,2), ...
                        bnd_epol(:,1),'nearest','extrap');    

% (2e) Smooth boundary 
bnd_smooth = bnd_epol;
bndFit = fit(bnd_smooth(:,1),bnd_smooth(:,2), ...
              'smoothingspline','SmoothingParam',10^-4);   
bnd_smooth(:,2) = round(feval(bndFit,bnd_smooth(:,1)));


%%%%%%%%%%%% (3) Create new mask for inSediment %%%%%%%%%%%%
% (3a) Reduce x spread (left to right)
%      (vec1): trough2trough as from troughLocs 
%      (vec2): good position: lower boundary from illumination
vec1 = false(size(A_pre,2),1);
vec1(troughLoc(1,1):troughLoc(1,2)) = true;

vec2 = false(size(A_pre,2),1);
vec2 = bnd_epol(:,2) < bnd_sed_pre(:,2);

vec = ( find(vec1 & vec2) )';
vec = 850:1850; % Manual overwrite

% (3b) Combine with SWI
inSedIllum = false(size(A_pre));
for i = vec
    inSedIllum(bnd_flume_pre(i,2) : bnd_smooth(i,2),i) = true;
end

% (3c) Erode SWI, since surface-near particle detection is erroneous
inSedIllumEroded = imerode(inSedIllum,ones(5));



%%%%%%%%%%%%%%%%%%% Figure Outputs %%%%%%%%%%%%%%%%%%%
figure
imagesc(inSedIllum + inSedIllumEroded)
setDiag
title('mask in Sediment from max illumination + eroded')

figure
hold all
imagesc(img)
plot(bnd_sed_pre( :,1), bnd_sed_pre( :,2),'g','LineWidth',2)
plot(bnd_epol(  vec,1), bnd_epol(  vec,2),'b','LineWidth',2)
plot(bnd_smooth(vec,1), bnd_smooth(vec,2),'r','LineWidth',2)
hold off
%
setDiag
title('Image with boundaries')
legend('Orig bnd','New bnd max illum','New bnd, smoothed')

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS_IndParticles_Boundaries_Improved');
print('-r300',filename,'-dpng')

%% Show mean intensities over time (every minute)
meanIntensity_All  = squeeze(mean(A_part./A_blank,[1 2]));  % All image
meanIntensity_Mask = zeros(size(A_part,3),1);              % Only interrogation area

for i = 1 : size(A_part,3)
    A = A_part(:,:,i);
    meanIntensity_Mask(i) = squeeze(mean(A(inSedIllumEroded)./A_blank(inSedIllumEroded),[1 2]));
end
clear A

figure
hold all
plot(meanIntensity_All,'b')
plot(meanIntensity_Mask,'r')
hold off
xlabel('Time [min]')
ylabel('Intensity relative to blank image [1]')
legend('All image','Mask inSed + @High Illumination + Eroded')
title('Mean intensities of images acquired every 1 minute')

%% Load dataset with detected particles
try 
    filename = strcat(pfad.results,exp_date,'-FIS_Centers.mat');
    load(filename,'centers','centers_len');
    fprintf('Individual particle positions were loaded')
catch
    % No dataset available. Save to run external function 
    filename = strcat(pfad.results,exp_date,'-FIS_datasetForCounting.mat');
    save(filename, ...
        'A_blank','datasets','delta_t','inSedIllumEroded', ...
        'pfad','file_length', ...
        'bnd_smooth', 't_Cam','idx_read','exp_date')
    open cam_count_particles.m
    error('No dataset with detected particles available')
end

%% Particle discrimination: infiltration / deposition
% Deposition at lee-side "destroys" outer infiltration boundaries.
% Therefore mask this area to focus on infiltration
centersIn =  cell(numel(centers),1);    % Infiltration = lee-side
centersOut = cell(numel(centers),1);    % Deposition   = luv-side


% Manual definition deposition area
% Drawing needs a figure, open and close afterwards
figure 
%roi = images.roi.Rectangle('Rotatable',1); draw(roi)
%roi_lee  = drawellipse('Center',[947 1087],'SemiAxes',[432,211], ...
%                       'RotationAngle',62,'AspectRatio',0.2);
roi_lee = drawrectangle('Position',[798 1058 347 57],'RotationAngle',330, ...
    'AspectRatio',0.17);

% For further use, create mask and polyline from boundary
mask_lee = createMask(roi_lee,size(img,1),size(img,2));
bnd_lee  = fliplr(cell2mat(bwboundaries(mask_lee)));

% Apply mask to discriminate particles; add (0,0) alibi particles for algos
for i = 1 : numel(centers)
    isInROI = inROI(roi_lee,centers{i}(:,1),centers{i}(:,2));
    centersOut{i} = centers{i}( isInROI,:);
    centersIn{ i} = centers{i}(~isInROI,:);
    
    if size(centersOut{i},1)<2
        centersOut{i} = zeros(2);
    end
    if size(centersIn{i},1)<2
        centersIn{i} = zeros(2);
    end
end
close(gcf) % inROI needs open figure


% Improve info of infiltrated particles
% (1) Calculate amount of infiltrated particles
centersIn_len = cellfun(@(x) size(x,1), centersIn);

% (2) Remove "background" = mean of t < 0. Should be ~2 for dummy "particles"
part_bg = round(mean(centersIn_len(1:idx_Cam0)));
part_inf = centersIn_len - part_bg;

% (3) Additionally calculate moving mean
part_mmean = movmean(part_inf,fps*movmean_sec);

% (4)Calculate metrics: 1st/2nd/3rd order moments
M1 = cell2mat(cellfun(@(x) mean(x,1),centersIn,'UniformOutput',false));


%%%%%%%% Some results %%%%%%%%
% (1) Individual particles (but noisy) 
[max_len,max_idx] = max(part_inf);
max_t = t_Cam(max_idx);

end_len = part_inf(idx_read(min_end));
end_t = t_Cam(idx_read(min_end));

% (2) Moving mean (but underestimating peak)
end_lenMM = part_mmean(idx_read(min_end));

% (3) Print to screen
clear res
res{1} = sprintf('Max of infiltrated particles: n = %i, at t = %.0f s', max_len, max_t);
res{2} = sprintf('End of experiment, particles: n = %i, at t = %.0f s', end_len, end_t);
res{3} = sprintf('End of experiment, moving mean: n = %.0f',end_lenMM);
msgbox(res,'Infiltration results')

%% Visualize particle discrimination
% Show image (every minute) when most particles detected (not absolute max)
[~,idx_max] = max(centersIn_len(idx_read));
i = idx_read(idx_max);

figure
hold all
imagesc(A_part(:,:,idx_max))
plot(centersIn{i}( :,1),centersIn{ i}(:,2),'gx')
plot(centersOut{i}(:,1),centersOut{i}(:,2),'rx')

plot(bnd_lee(:,1), bnd_lee(:,2),'b','LineWidth',2)
plot(bnd_smooth(vec,1), bnd_smooth(vec,2),'c','LineWidth',2)
legend('Infiltration - luv', 'Deposition - lee', 'Mask lee','SWI')
title('Particle identification - used mask')
setDiag

%% Graphic: amount of infiltrated particles over time
figure('units','centimeters','position',[10,10,9,7])
hold all
scatter(t_Cam, part_inf,20,'rx','DisplayName', 'Measured data','MarkerEdgeAlpha',1)
plot(t_Cam, part_mmean, 'b-','LineWidth',1,'DisplayName',sprintf('Smoothed (%i s)',movmean_sec))
xlim([0,1800])
ylim([0,1000])
xticks(300*(0:6))
xlabel('Time [s]')
ylabel('Particles in streambed [1]')
%
leg = legend;
leg.Position = [0.6,0.7,0.2,0.2];
leg.ItemTokenSize = [10,5];
%
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf, 'InvertHardcopy', 'off')
set(gcf,'Color',[1,1,1])
set(gcf, 'renderer', 'painters')
%
filename = strcat(pfad.graphics_FIS,exp_date,'-InfiltratedParticles_Time');
print('-r300',filename,'-dpng');

%% Retrieve outer bound of particles
fprintf('\nRetrieving outer boundary of particle infiltration\n')
area = zeros(size(centers));
for i = 1 : numel(centersIn)
    if mod(i,1000)==0
        fprintf('Now at boundary %i of %i\n',i,numel(centersIn))
    end
    [k,area(i,1)] = boundary(centersIn{i}(:,1),centersIn{i}(:,2),0);
    bnd_particles{i,1} = [centersIn{i}(k,1), centersIn{i}(k,2)];
end

area_cm2 = area * (lengthScale/10)^2; % (Pix^2) * ([mm/Pix]*[1cm/10mm])^2

figure
% (1) Particle abundance
yyaxis left
plot(t_Cam,centersIn_len,'r','DisplayName','Particle abundance')
set(gca,'YColor','r')
ylim([0 1000])
ylabel('Particles abundance in sediment [#]')
%
% (2) Particle density
yyaxis right
plot(t_Cam,centersIn_len./area_cm2,'b','DisplayName','Areal density')
set(gca,'YColor','b')
ylim([0 50])
ylabel('Particle areal density [#/cm^2]')
%
leg = legend;
leg.ItemTokenSize = [15,2];
xlim([0,1800])
title('Infiltration of Hyporheic Zone with 10 m particles')
xlabel('Time after MP input [s]')

%% Improve boundary for graphic
% Interpolate lower half of lee mask to restrict boundary position
bnd_lee_Low = bnd_lee(1:ceil(end/2),:);

% Ensure no doubled points for interpolant
[xUnique,~,idx1] = unique(bnd_lee_Low(:,1),'stable');            
yUnique = accumarray(idx1,bnd_lee_Low(:,2),[],@median); 
bnd_lee_Low = [xUnique, yUnique];

bnd_lee_LowInt(:,1) = 1:size(A_pre,2);
bnd_lee_LowInt(:,2) = interp1(bnd_lee_Low(:,1), bnd_lee_Low(:,2), ...
                         bnd_lee_LowInt(:,1));
      

% Although all particles are below SWI, the compact boundary as outer particle
% connections can cross and be above SWI. Same for mask in lee discriminating 
% infiltration from deposition. If this happens, take the SWI/mask as upper bnd
fprintf('\nImproving particles boundaries: restrict to sed-water-interface\n')
bnd_particlesSWI = cell(numel(centersIn),1); % Create new bnd cell

for i = 1 :numel(centersIn)     
    % Progress feedback
    if mod(i,1000)==0
        fprintf('Now at boundary %i of %i\n',i,numel(centersIn))
    end
    
    % Ignore if only few particles are detected, 0-padding for functionality
    if centersIn_len(i)<10
        bnd_particlesSWI{i} = zeros(2); 
        continue
    end
    
    % Retrieve current boundary dataset 
    x = round(bnd_particles{i}(:,1));
    y = round(bnd_particles{i}(:,2));
    
    % Resort: Start from farmost right point
    [~,idx] = max(x);
    xs = [x(idx:end);x(1:idx-1)];
    ys = [y(idx:end);y(1:idx-1)];
    
 
    % Cut dataset: Top from max to min, low from min to max, counterclockwise
    [~,idx] = min(xs);
    xTop = xs(1:idx);
    yTop = ys(1:idx);
    xLow = xs(idx+1:end);
    yLow = ys(idx+1:end);

    
    % Enhance upper boundary: interpolate and replace if out of bounds
    % (a) Ensure no doubled points for interpolant
    [xTop,~,idx1] = unique(xTop,'stable');            
    yTop = accumarray(idx1,yTop,[],@median);     
    %
    % (b) Interpolate boundary
    xupI = (xTop(1):-1:xTop(end))';
    yupI = interp1(xTop,yTop,xupI,'linear');
    %
    % (c) Restrict to sediment-water-interface
    idx_SWI       = bnd_smooth(xupI,2) < yupI;
    yupI(idx_SWI) = bnd_smooth(xupI(idx_SWI),2);
    %
    % (d) Restrict to deposition boundary in lee
    idx_lee       = bnd_lee_LowInt(xupI,2) < yupI;
    yupI(idx_lee) = bnd_lee_LowInt(xupI(idx_lee),2);
    
    
    % Close the loop
    xr = [xLow;xupI;xLow(1)];
    yr = [yLow;yupI;yLow(1)];    
       
    bnd_particlesSWI{i} = [xr,yr];
end

% Show boundary for test image
figure
hold all
imagesc(img)
plot(bnd_particlesSWI{i}(:,1),bnd_particlesSWI{i}(:,2),'g-','DisplayName','New boundary' )
scatter(xs,ys,'rx','LineWidth',2)
plot(bnd_lee(:,1),bnd_lee(:,2),'c','LineWidth',2);
hold off
setDiag

%% Analyse center of mass
% Visualize above defined threshold, otherwise too noisy
n_Particles_start = find(centersIn_len>n_min,1,'first');
fprintf('More than %i particles detected from %.2f s', n_min,t_Cam(n_Particles_start))

% Center of Gravity
% (1) copy data
cog_t =  t_Cam(n_Particles_start:end);
cog_local = M1(n_Particles_start:end,:);
%
% (2) Convert to global coordinates
cog_global = cog_local;
cog_global(:,1) = -(cog_local(:,1)-x_loc)*lengthScale + x_gl*1000; % mm
cog_global(:,2) =  (cog_local(:,2)-y_loc)*lengthScale + y_gl*1000;    
%
% (3) Calculate moving mean 
cog_globalMM   = movmean(cog_global,fps*movmean_cog);
cog_velocity   = vecnorm(diff(cog_globalMM),2,2) *fps; % mm/frame * frame/second
cog_velocityMM = movmean(cog_velocity,fps*movmean_cog);


% Boundary in global coordinates
bnd_smooth_global = bnd_smooth;
bnd_smooth_global(:,1) = -(bnd_smooth(:,1)-x_loc)*lengthScale + x_gl*1000; % mm
bnd_smooth_global(:,2) =  (bnd_smooth(:,2)-y_loc)*lengthScale + y_gl*1000;    



%%%%%%%% Figure 1: Position of center of mass over time %%%%%%%%%%
% Restrict image to crest region
xl = [3720, 3760];
yl = [110, 150];

figure
hold all
scatter(cog_globalMM(:,1), cog_globalMM(:,2), 20,cog_t,'x','MarkerEdgeAlpha',1)
plot(bnd_smooth_global(vec,1), bnd_smooth_global(vec,2),'k','LineWidth',1,'DisplayName','water-HZ');
hold off

colormap(jet)
cb = colorbar;
cb.Position = [.835 .17 .03 .61];
cb.Title.String = 't [s]';
cb.Title.Position =  [10,128,0];
cb.Ticks = 0:300:1800;
caxis([0,1800])

leg = legend('Center of mass','streambed');
leg.NumColumns = 2;
leg.ItemTokenSize = [4,2];
leg.Position = [0.2,0.85,0.5,0.05];

xlim(xl)
ylim(yl)
xlabel('Longitudinal position [mm]')
ylabel('Vertical position [mm]')

set(gca,'DataAspectRatio',[1 1 1], 'XDir','reverse')
ax = gca;
ax.Position = [0.17,0.17,0.6,0.6];

%%% Inset plot: positioning of detailed area 
axes('Position',[0.61,0.6,0.2,0.2])
hold all
scatter(cog_globalMM(:,1), cog_globalMM(:,2), 3,cog_t,'x','MarkerEdgeAlpha',1)
plot(bnd_smooth_global(vec,1), bnd_smooth_global(vec,2),'k','LineWidth',1,'DisplayName','water-HZ');
rectangle('Position',[xl(1), yl(1), xl(2)-xl(1), yl(2)-yl(1)])
hold off
box on
axis equal
set(gca,'XDir','reverse')
xticks([])
yticks([])

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf, 'InvertHardcopy', 'off')
set(gcf,'Color',[1,1,1])
set(gcf, 'renderer', 'painters')

filename = strcat(pfad.graphics_FIS,exp_date,'-CoG_position');
print('-r300',filename,'-dpng');


%%%%%%%%% Figure 2: Velocity of center of mass %%%%%%%%%%%%
figure
hold all
scatter(cog_t(1:end-1),cog_velocity,10,'bx')
plot(cog_t(1:end-1), cog_velocityMM,'r-','LineWidth',1.25)
hold off
leg = legend('Measurements','Moving average');
leg.ItemTokenSize = [8,2];
leg.Location = 'northeast';

xlim([0,1800])
xlabel('Time [s]')
ylabel('Velocity center of mass [mm/s]')

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf, 'InvertHardcopy', 'off')
set(gcf,'Color',[1,1,1])
set(gcf, 'renderer', 'painters')

filename = strcat(pfad.graphics_FIS,exp_date,'-CoG_velocity');
print('-r300',filename,'-dpng');

%% Graphic: Superposition all detected particles
climTime = [0,experiment_duration*60];

clear p object_h
cmap_Overlay = jet(qty_dataread);
color_highlight = cmap_Overlay(img_no,:);
bnd_color = [0,0,0];

figure
hold all
% (1) Detected particles
for k = 1:qty_dataread
    t_k = idx_read(k);
    scatter(centersIn{t_k}(:,1), ...
            centersIn{t_k}(:,2), 50, ...
            repmat(t_Cam(t_k),1,length(centersIn{t_k})), ...
            'x','MarkerEdgeAlpha',0.5);    
end
%
% (2) Boundaries: water-air-interface, sediment-water-interface
p(3) = plot(bnd_water_pre(vec,1), bnd_water_pre(vec,2),'--', 'Color',bnd_color,...
    'LineWidth',2,'DisplayName','air-water');
p(4) = plot(bnd_smooth(vec,1), bnd_smooth(vec,2),'Color',bnd_color, ...
    'LineWidth',2,'DisplayName','water-HZ');
%
% (3) Boundarites: deposition area, as patch + hatchfill
p(2) = patch(bnd_lee(:,1), bnd_lee(:,2),'r','FaceColor','none','EdgeColor','k');
hatchfill2(p(2),'HatchAngle',-45,'HatchDensity',50,'EdgeColor',bnd_color);
%
% (4) Alibi particle for legend
p(1) = scatter(0,0,1,0,'x');
hold off

set(gca,'Visible','off')
colormap(cmap_Overlay)
cb = colorbar;
cb.Position = [.83 .25 .03 .61];
cb.Title.String = 't [s]';
cb.Title.Position =  [10,130,0];
cb.Ticks = 0:300:1800;
caxis(climTime)

xlim([850 1850])
ylim([650 1425])
set(gca,'Position',[0.005,0.08,0.8,1]);
axis equal

% legend with legendflex, to enable hatching in legend
legendData = {'Particles','Deposition',...
    'Air-water interface','streambed interface'};
[legend_h, object_h, plot_h, text_str] = legendflex(p, legendData, ...
    'FontSize', 11, 'anchor', [7 7],'buffer',[0 -20],'ncol',2,'xscale',0.6);
hatchfill2(object_h(6), 'HatchAngle',-70,'HatchDensity',30, ...
    'FaceColor','none','EdgeColor',bnd_color);

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf, 'InvertHardcopy', 'off')
set(gcf,'Color',[1,1,1])
set(gcf, 'renderer', 'painters')

filename = strcat(pfad.graphics_FIS,exp_date,'-InfiltratedParticlesOverlay');
print('-r300',filename,'-dpng');

%% Visualize infiltration areas
clear p object_h

climImg  = [1, 1.2]; % limits for colorbar: scaled intensity (bg img)
bnd_color = [0,0.6,0]; % Color of deposition boundary

% Colormap outer particle bnds; highlight timestep of img
cmapBnd = (autumn(qty_dataread));
cmapBnd(img_no,:) = color_highlight;

% Colormap centers of gravity; highlight timestep of img
cmapCOG = (autumn(qty_dataread));
idx_cmap = floor( size(cmapCOG,1)/(climTime(2)-climTime(1)+1)* ...
                  t_Cam(idx_read(img_no)) ) + 1;
cmapCOG(idx_cmap,:) = color_highlight;   


figname = sprintf('Particles, CGs and infiltration boundaries at t = %.0f s',...
              t_Cam(idx_read(img_no)));
         

figure('name',figname)
%%%%%% (A) Backbround image for visualization %%%%%%
ax1 = axes;
A_show = A_part(:,:,img_no)./A_blank;
imagesc(ax1,A_show);
setDiag

%%%%%% (B) MP particles, fronts, centers of gravity, interfaces %%%%%%
ax2 = axes; 
colororder(cmapBnd)
hold all

% (1) Boundary for various timesteps
for i = 1:qty_dataread
    plot(bnd_particlesSWI{idx_read(i)}(:,1), ...
         bnd_particlesSWI{idx_read(i)}(:,2),'Color',cmapBnd(i,:),'LineWidth',0.5 )
end

% (2) Boundary for chosen timestep (of image)
p(3) = plot(bnd_particlesSWI{idx_read(img_no)}(:,1), ...
            bnd_particlesSWI{idx_read(img_no)}(:,2), ...
            'Color',color_highlight,'LineWidth',2,'DisplayName','Boundary');
    
% (3) Particle locations for timestep of image
p(1) = scatter(ax2,centersIn{idx_read(img_no)}(:,1), ...
               centersIn{idx_read(img_no)}(:,2), 0.5, ...
               'filled','MarkerEdgeColor','none', ...
               'MarkerFaceColor',color_highlight);
           
% (4) Centers of gravity for various timesteps
scatter(ax2, M1(n_Particles_start:480:end,1), ...
             M1(n_Particles_start:480:end,2), 20,...
          t_Cam(n_Particles_start:480:end,1), ...
          'x')
   
% (5) Centers of gravity for timestep of image
p(2) = scatter(ax2, M1(idx_read(img_no),1), ...
                    M1(idx_read(img_no),2), 40,...
         'x','MarkerEdgeColor',color_highlight,'LineWidth',1);
     
% (6) Boundaries (a): water-air-interface, sediment-water-interface
p(4) = plot(bnd_water_pre(vec,1), bnd_water_pre(vec,2),'--', 'Color',bnd_color,...
    'LineWidth',2,'DisplayName','air-water');
p(5) = plot(bnd_smooth(vec,1), bnd_smooth(vec,2),'Color',bnd_color, ...
    'LineWidth',2,'DisplayName','water-HZ');

% (7) Boundaries (b): deposition area, as patch + hatchfill
p(6) = patch(bnd_lee(:,1), bnd_lee(:,2),'r','FaceColor','none','EdgeColor','k');
hatchfill2(p(6),'HatchAngle',-45,'HatchDensity',50,'EdgeColor',bnd_color)
hold off


%%%%%%%%%% Optimizing visualization %%%%%%%%%%
% legend (v2): With legendflex, to enable hatching in legend
legendData = {'Particles','Center of mass','Plume front','Air-water','streambed','Deposition'};
[legend_h, object_h, plot_h, text_str] = legendflex(p, legendData, ...
    'FontSize', 11, 'anchor', [7 7],'buffer',[0,-20],'ncol',2,'xscale',0.645);
hatchfill2(object_h(15), 'HatchAngle',-70,'HatchDensity',30,'FaceColor','none','EdgeColor',bnd_color);

% Restrict image to crest region
ax1.XLim = [850 1850];
ax1.YLim = [650 1425];

% Improve axis positioning
set(ax1,'Position',[0.02,0.08,0.8,1]);
set(ax2,'Position',ax1.Position);

% Perfectly align both axes (axis equal)
set(ax2,'XLim',xlim(ax1),'YLim',ylim(ax1));
set(ax1,'DataAspectRatio',ones(1,3))
set(ax2,'DataAspectRatio',ones(1,3))
    
% Link both axes, hide the top axes, hide ticks
linkaxes([ax1,ax2],'xy')
set(ax1,'Visible','off')
set(ax2,'Visible','off')

% Define colormaps, limits, colorbars
colormap(ax1,'gray')
colormap(ax2,cmapCOG)
caxis(ax1,climImg)
caxis(ax2,climTime)
cb1 = colorbar(ax1,'Visible','off');
cb2 = colorbar(ax2,'Position',[.83 .25 .03 .66],'Limits',climTime);
cb2.Title.String = 't [s]';
cb2.Title.Position =  [10,137,0];
cb2.Ticks = 0:300:1800;

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
set(gcf, 'InvertHardcopy', 'off')
set(gcf,'Color',[1,1,1])
set(gcf, 'renderer', 'painters')

filename = strcat(pfad.graphics_FIS,exp_date,'-InfiltratedParticles');
print('-r300',filename,'-dpng');

msgbox('Done','Yeay')
% EOF