% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

clear all; close all; clc;

% Set experiment date: yyMMdd
exp_date = '220901'
US.amount = 6; % Amount of employed ultrasonics
hoboFiles = 2; % Amount of Hobo files for ultrasonics

%% Set paths and read labbook
pfad = fct_pfad(exp_date);
labbook       = '../labbook.xlsx';
sheets = sheetnames(labbook);
metadata = readcell(labbook,'Sheet',exp_date);

%% Read out values from labbook
m_zug = metadata{strcmp(metadata,'Zugabe MP'),2} * 1000; % in g

% Experiment times
t0 = metadata{strcmp(metadata,'Blank Experiment'),2}; 
tZ = metadata{strcmp(metadata,'Input Experiment'),2}; 
t1 = metadata{strcmp(metadata,'Ende Experiment'),2}; 
t_Baro0 = metadata{strcmp(metadata,'Baro 0 Start'),2}; 
t_Baro1 = metadata{strcmp(metadata,'Baro 0 Ende'),2}; 

% Fluo Serial numbers
Fluo_in  = metadata{strcmp(metadata,'SN Fluo in'),2};
Fluo_out = metadata{strcmp(metadata,'SN Fluo out'),2};
devices = [Fluo_in, Fluo_out];

% Time lapse of fluos caused by tubings and pumping
dt_In  = seconds(round(metadata{strcmp(metadata,['Time Lapse ',num2str(devices(1))]),2},0));
dt_Out = seconds(round(metadata{strcmp(metadata,['Time Lapse ',num2str(devices(2))]),2},0));

% Convert times to datetimes with experiment date
exp_datetime = datetime(exp_date,'Format','yyMMdd');
t0 = datetime(t0+datenum(exp_datetime),'ConvertFrom','datenum');
tZ = datetime(tZ+datenum(exp_datetime),'ConvertFrom','datenum');
t1 = datetime(t1+datenum(exp_datetime),'ConvertFrom','datenum');
t_Baro0 = datetime(t_Baro0+datenum(exp_datetime),'ConvertFrom','datenum');
t_Baro1 = datetime(t_Baro1+datenum(exp_datetime),'ConvertFrom','datenum');

% Auslesen ultrasonic information: SN, longitudinal and vertical position
if nnz(strcmp(metadata,'US1')) > 0 % If US data exists
    for i = 1:US.amount
        idx_US = find(strcmp(metadata,sprintf('US%i',i')));
        US.(sprintf('dev%i',i)).SN = sprintf('US%i',metadata{idx_US+1});
        US.(sprintf('dev%i',i)).xpos = metadata{idx_US+2};
        US.(sprintf('dev%i',i)).zpos = metadata{idx_US+3};
    end
end

%% Reading and first processing Fluo Data
% Loading calibration data
fluocaldata = strcat(pfad.results, exp_date,'-Fluo_Calibration.mat')

if isfile(fluocaldata)
    load(fluocaldata,'-mat','fluoData','cmin','cmax','c_fix','y_fix');
    
    % Saving Fluorometer calibration slope easy-to-access
    % (a) Inflow 
    device = sprintf('Fluo%i',devices(1)); % Inflow    
    m_FluoIn  = fluoData.(device).('m0');
    m_FluoIn1 = fluoData.(device).('m1');
    m_FluoIn2 = fluoData.(device).('m2');
    LOQ_FluoIn = fluoData.(device).('LOQ');
    %
    % (b) Outflow
    device = sprintf('Fluo%i',devices(2)); % Outflow    
    m_FluoOut  = fluoData.(device).('m0');
    m_FluoOut1 = fluoData.(device).('m1');
    m_FluoOut2 = fluoData.(device).('m2');
    LOQ_FluoOut = fluoData.(device).('LOQ');
    
else
    error('No Calibration for Fluos found. Please check')
end

% Loading experiment data from Fluos, s = signal (raw), t = time
data = strcat(pfad.Fluo, exp_date, '-f',num2str(devices(1)),'.mv');
messwerte = readmatrix(data,"FileType","text","OutputType","string");
t_FluoIn = datetime(messwerte(:,2),'InputFormat', 'dd/MM/yy-HH:mm:ss');
s_FluoIn = str2double(messwerte(:,4));
%
data = strcat(pfad.Fluo, exp_date, '-f',num2str(devices(2)),'.mv');
messwerte = readmatrix(data,"FileType","text","OutputType","string");
t_FluoOut = datetime(messwerte(:,2),'InputFormat', 'dd/MM/yy-HH:mm:ss');
s_FluoOut = str2double(messwerte(:,4));
clear data messwerte

% Cut dataset to experiment date
t_FluoInMin =  find(t_FluoIn >exp_datetime,1);
t_FluoOutMin = find(t_FluoOut>exp_datetime,1);
% Inflow
t_FluoIn = t_FluoIn(t_FluoInMin:end);
s_FluoIn = s_FluoIn(t_FluoInMin:end);
% Outflow
t_FluoOut = t_FluoOut(t_FluoOutMin:end);
s_FluoOut = s_FluoOut(t_FluoOutMin:end);

% Plot Raw data
figure
hold all
plot(t_FluoIn, s_FluoIn, 'r','DisplayName','Fluo Inflow')
plot(t_FluoOut,s_FluoOut,'b','DisplayName','Fluo Outflow')
hold off
title('Raw signal Fluorometers')
ylabel('Signal [mV]')
legend


% Adjusting time for time lapse caused by pumping
t_FluoIn  = t_FluoIn  - dt_In;
t_FluoOut = t_FluoOut - dt_Out;

% Find Start of Experiment, Input MP, and End
n0_FluoIn  = find(t_FluoIn >=t0,1);
nZ_FluoIn  = find(t_FluoIn >=tZ,1);
n1_FluoIn  = find(t_FluoIn >=t1,1);
%
n0_FluoOut = find(t_FluoOut>=t0,1);
nZ_FluoOut = find(t_FluoOut>=tZ,1);
n1_FluoOut = find(t_FluoOut>=t1,1);

% Read out blank value for subtraction
s_FluoInb  = mean(s_FluoIn( n0_FluoIn :nZ_FluoIn ));
s_FluoOutb = mean(s_FluoOut(n0_FluoOut:nZ_FluoOut));

% Plot for visual inspection blank time interval
figure
hold all
plot(t_FluoIn(n0_FluoIn+(-50:150)),s_FluoIn(n0_FluoIn+(-50:150)), ...
     'r-','DisplayName',sprintf('raw %i',devices(1)))
plot(t_FluoIn([n0_FluoIn,nZ_FluoIn]),[s_FluoInb,s_FluoInb], ...
    'b','LineWidth',2,'DisplayName',sprintf('blank %i',devices(1)))
hold off
title('Check for blank definition: Raw signal Fluo Input')
ylabel('Signal [mV]')
legend
%
figure
hold all
plot(t_FluoOut(n0_FluoOut+(-50:150)),s_FluoOut(n0_FluoOut+(-50:150)), ...
     'r-', 'DisplayName',sprintf('raw %i',devices(2)))
plot(t_FluoOut([n0_FluoOut,nZ_FluoOut]),[s_FluoOutb,s_FluoOutb], ...
    'b','LineWidth',2,'DisplayName',sprintf('blank %i',devices(2)))
hold off
title('Check for blank definition: Raw signal Fluo Output')
ylabel('Signal [mV]')
legend

% Cut dataset for interval: exp start to end
t_FluoIn = t_FluoIn(n0_FluoIn:n1_FluoIn);
s_FluoIn = s_FluoIn(n0_FluoIn:n1_FluoIn);
%
t_FluoOut = t_FluoOut(n0_FluoOut:n1_FluoOut);
s_FluoOut = s_FluoOut(n0_FluoOut:n1_FluoOut);

% Convert time to relative duration after input MP
t_FluoIn  = round(seconds(t_FluoIn -tZ),0);
t_FluoOut = round(seconds(t_FluoOut-tZ),0);


% Convert to concentration, using complete and partial regression lines
% (a) Inflow Fluorometer
c_FluoIn  = 1/m_FluoIn *  (s_FluoIn - s_FluoInb);
c_FluoIn1 = 1/m_FluoIn1 * (s_FluoIn - s_FluoInb - y_fix) + c_fix; % small conc 
c_FluoIn2 = 1/m_FluoIn2 * (s_FluoIn - s_FluoInb - y_fix) + c_fix; % high  conc
%
c_FluoIn1(c_FluoIn1 >  c_fix) = nan;
c_FluoIn2(c_FluoIn2 <= c_fix) = nan;
c_FluoIn12 = nansum([c_FluoIn1, c_FluoIn2],2);
%
% (b) Outflow Fluorometer
c_FluoOut  = 1/m_FluoOut  * (s_FluoOut-s_FluoOutb);
c_FluoOut1 = 1/m_FluoOut1 * (s_FluoOut-s_FluoOutb - y_fix) + c_fix;
c_FluoOut2 = 1/m_FluoOut2 * (s_FluoOut-s_FluoOutb - y_fix) + c_fix;
%
c_FluoOut1(c_FluoOut1 >  c_fix) = nan;
c_FluoOut2(c_FluoOut2 <= c_fix) = nan;
c_FluoOut12 = nansum([c_FluoOut1, c_FluoOut2],2);


% Apply Quantification Limit: set to nan if too low
c_FluoIn12(  c_FluoIn12  < LOQ_FluoIn)  = nan;
c_FluoOut12( c_FluoOut12 < LOQ_FluoOut) = nan;

% Account for undesired recirculation possible after ~ 20min in 220901 experiment
if strcmp(exp_date,'220901')
   idx_nan = t_FluoIn > 20*60;
   c_FluoIn12(idx_nan)  = nan;
   c_FluoOut12(idx_nan) = nan;
end


% Plot concentrations during experiment
figure
hold all
plot(t_FluoIn, c_FluoIn12,'b',  'DisplayName', 'Inlet');
plot(t_FluoOut,c_FluoOut12,'r', 'DisplayName', 'Outlet');
yline(c_fix,'k--',      'DisplayName', 'switch cal slopes');
yline(cmin,'g-',        'DisplayName', 'Min cal conc');
yline(cmax,'g-',        'DisplayName', 'Max cal conc');
yline(LOQ_FluoIn, 'b--','DisplayName','LOQ inlet');
yline(LOQ_FluoOut,'r--','DisplayName','LOQ outlet');
hold off
legend('Location','best')
title(sprintf('Fluorometer Experiment %s',exp_date))
xlabel('Time after input [s]')
ylabel('MP concentration [g/L]')
set(gca,'XScale','log','YScale','log')
xlim([1 1000])
fullfilename = strcat(pfad.graphics,exp_date,'-Fluo_Experiment');
print('-r300',fullfilename,'-dpng')

%% Processing Hobo Data (ultrasonics + discharge)
% Check if dataset is available, case-insensitive .txt or .csv file
vorhandene_dateien  = dir([pfad.HS,exp_date, '*hobo*']);
dateinamen          = string({vorhandene_dateien.name});
hoboData = dateinamen(contains(dateinamen,[".csv",".txt"],'IgnoreCase',true));

switch length(hoboData)
    case hoboFiles % As planned, continue
    case 0
        error('No Hobo Data found!')
    otherwise
        error('Not sufficient Data found, please check')
end

hoboDataH = hoboData(contains(hoboData,'1234'))
dataH = readcell(strcat(pfad.HS, hoboDataH),'DateTimeType', 'text');
t_HoboH = cell2mat(dataH(2:end,1));
        
hoboDataQ = hoboData(contains(hoboData,'q','IgnoreCase',1))
dataQ = readcell(strcat(pfad.HS, hoboDataQ),'DateTimeType', 'text');
t_HoboQ = cell2mat(dataQ(2:end,1));
s_Q     = cell2mat(dataQ(2:end,4)); % Discharge data

% Convert to datetime, check if format is correct
timeFormat = 'yy.MM.dd HH:mm:ss';
try
    t_HoboH = datetime(t_HoboH,'InputFormat', timeFormat);
    if year(t_HoboH(1))<2000
        warning('Year is not correct (before 2000). Revise format')
    end
catch ME
    prompt = {sprintf('Identify %s',t_HoboH(1,:))};
    dlgtitle = 'Identify Date Format';
    dims = [1 35];
    definput = {timeFormat};
    timeFormat = string(inputdlg(prompt,dlgtitle,dims,definput));   
    t_HoboH = datetime(t_HoboH,'InputFormat', timeFormat);
end
try
    t_HoboQ = datetime(t_HoboQ,'InputFormat', timeFormat);
    if year(t_HoboQ(1))<2000
        warning('Year is not correct (before 2000). Revise format')
    end
catch ME
    prompt = {sprintf('Identify %s',t_HoboQ(1,:))};
    dlgtitle = 'Identify Date Format';
    dims = [1 35];
    definput = {timeFormat};
    timeFormat = string(inputdlg(prompt,dlgtitle,dims,definput));   
    t_HoboQ = datetime(t_HoboQ,'InputFormat', timeFormat);
end

for i = 1:6
    device = sprintf('dev%i',i);
    if i<=4
        US.(device).sig = cell2mat(dataH(2:end,i+1));
        US.(device).t   = t_HoboH;
    else
        US.(device).sig  = cell2mat(dataQ(2:end,(i-4)+1));
        US.(device).t  = t_HoboQ;
    end
end

% Graphic raw data
figure
plot(t_HoboQ, s_Q,'r')
title('Raw data discharge')
ylabel('Signal [mA]')

figure
hold all
for i = 1:6
    device = sprintf('dev%i',i);
    plot(US.(device).t,US.(device).sig,'DisplayName',US.(device).SN)
end
hold off
title('Raw data ultrasonics')
ylabel('Signal [V]')
legend


% Find Start and end of Experiment
n0_hoboH  = find(t_HoboH>=t0,1);
n1_hoboH  = find(t_HoboH>=t1,1);

n0_hoboQ  = find(t_HoboQ>=t0,1);
n1_hoboQ  = find(t_HoboQ>=t1,1);

% Cut dataset to experiment
t_HoboQ = t_HoboQ(n0_hoboQ:n1_hoboQ);
t_HoboH = t_HoboH(n0_hoboH:n1_hoboH);

for i = 1:6
    device = sprintf('dev%i',i);
    if i<=4
        US.(device).t   = US.(device).t(  n0_hoboH:n1_hoboH);
        US.(device).sig = US.(device).sig(n0_hoboH:n1_hoboH); 
    else
        US.(device).t   = US.(device).t(  n0_hoboQ:n1_hoboQ);
        US.(device).sig = US.(device).sig(n0_hoboQ:n1_hoboQ);   
    end
end
s_Q  =  s_Q(n0_hoboQ:n1_hoboQ);

% Convert to mm: signal to distance from device, then to height over bottom
for i=1:6
    device = sprintf('dev%i',i);   
    US.(device).z = fct_convert(US.(device).sig,US.(device).SN);
    US.(device).h = (US.(device).zpos - US.(device).z) / 1000;
end

% Convert discharge
Q   = fct_convert(s_Q,'MID')/3.6;  % Calibration for m/h -> L/s

% Convert time to relative duration after input MP, round to full seconds
t_HoboH  = round(seconds(t_HoboH-tZ),0);
t_HoboQ  = round(seconds(t_HoboQ-tZ),0);
for i=1:6
    device = sprintf('dev%i',i);
    if i<=4
        US.(device).t   = t_HoboH;
    else
        US.(device).t   = t_HoboQ; 
    end
end

% Find temporal tendency for discharge (polyline 2nd order)
polyfitOrder = 2;
[p, errorEstimate] = polyfit(t_HoboQ,Q,polyfitOrder);
[Qfit, delta] = polyval(p,t_HoboQ,errorEstimate);
Qmean = mean(Qfit);

% Plot discharge and tendency
figure('units','centimeters','Position',[10,10,16,5])
hold on
scatter(t_HoboQ,Q,'bx','MarkerEdgeAlpha',0.2);
p1 = scatter(nan,nan,'bx');
p2 = plot(t_HoboQ,Qfit,'r','LineWidth',2);
p3 = plot(t_HoboQ,Qfit-2*delta,'r--','LineWidth',2);
plot(t_HoboQ,Qfit+2*delta,'r--','LineWidth',2);
hold off
%
leg = legend([p1,p2,p3],'Data MID','Polynomial Fit','95 % Prediction Interval');
leg.ItemTokenSize = [20,2];
leg.NumColumns = 3;
leg.Position = [0.2,0.8,0.7,0.1]
xlabel('Time [s]')
ylabel('Discharge [L/s]')
xlim([0,1800])
%title(sprintf('Discharge data and polynomial fit, n = %i, Q_{mean} = %1.3f L/s',polyfitOrder,mean(Qfit)))
set(findall(gcf,'-property','FontSize'),'FontSize',11) % 11
print('-r300',strcat(pfad.graphics,exp_date,'-Discharge'),'-dpng')


% Plot waterdepth data
figure
colors = parula(8);
colors = colors(2:7,:);
colororder(colors)
hold on

for i=1:6
    device = sprintf('dev%i',i);   
    plot(US.(device).t, US.(device).h,'DisplayName',US.(device).SN);
end
hold off
xlabel('Time [s]')
ylabel('Water level elevation [m]')
title('Water level above flume bottom over time')
leg = legend;
leg.Title.String = 'long dist [cm]';
leg.Location = 'best';

% Save positions and elevations over time in extra matrix
X = [];
H = [];
for i = 1:6
    device = sprintf('dev%i',i);
    X = horzcat(X,US.(device).xpos);
    H = horzcat(H,US.(device).h); 
end

% Show elevation every minute
vec_minute = find(mod(t_HoboH,60)==0);

figure('units','centimeters','Position',[10,10,16,5])
colors = jet(length(vec_minute));
colororder(colors)
hold on
p = plot(X,H(vec_minute,:),'x-');
arrayfun(@(a)xline(a,'--',num2str(a)),X);
hold off
title('Water surface elevation')
xlabel('Hor. Distance [m]')
ylabel('Water level elevation [m]')
%ylim([0.15 0.2])

legendCell = cellstr(num2str((t_HoboH(vec_minute)), '%.0f'));
leg = legend(p,legendCell,'NumColumns',4,'Location','eastoutside');
leg.ItemTokenSize = [10,4];
leg.Title.String = 'Time after MP input [s]';
print('-r300',strcat(pfad.graphics,exp_date,'-Water_Surface_Elevation'),'-dpng')

clear data dataH dataQ 

%% Levellogger
% Check if dataset is available, case-insensitive .txt or .csv file
vorhandene_dateien  = dir([pfad.HS,exp_date, '*level*']);
dateinamen          = string({vorhandene_dateien.name});
levelData = dateinamen(contains(dateinamen,[".csv",".txt"],'IgnoreCase',true));

if isempty(levelData)
    error('No Level Data found!')
elseif length(levelData) > 1
    [indx,tf] = listdlg('ListString',cellstr(levelData), ...
        'PromptString', 'Various Data found, please choose', ...
        'SelectionMode','single','ListSize',[300,300]);
    levelData = levelData(indx);
end

% Read in data, find start of values, check if delimiter is '.' and not ','
data = readcell(strcat(pfad.HS, levelData),'DateTimeType', 'text');
idx_start  =  find(strcmp(data,'Date'))+1;
types = cellfun(@class, data(idx_start,:), 'UniformOutput', false);

if strcmp(types{4},'char')
    % Delimiter was ',' -> replace to '.' and convert to number
    level = str2num(cell2mat(strrep(data(idx_start:end,4),',','.')));
    temp  = str2num(cell2mat(strrep(data(idx_start:end,5),',','.')));
else
    % Delimiter was '.' -> everything ok, read in
    level = cell2mat(data(idx_start:end,4));
    temp  = cell2mat(data(idx_start:end,5));
end

% Time info is in 2 columns. Check dayFormat. Time Format as duration
dayFormat = 'dd.MM.yyyy';
try
    ll_day = datetime(data(idx_start:end,1),'InputFormat', dayFormat);
    if year(ll_day(1))<2000
        error('Wrong date format. Please check')
    end
catch ME
    dlgtitle = 'Wrong Date Format Levellogger';
    prompt = {sprintf('Identify %s',data(idx_start,1))};
    dims = [1 50];
    definput = {dayFormat};
    dayFormat = string(inputdlg(prompt,dlgtitle,dims,definput));   
    
    ll_day = datetime(data(idx_start:end,1),'InputFormat', dayFormat);
end

ll_hour = vertcat(data{idx_start:end,2});
t_ll = ll_hour + ll_day; 

% Find start and end of experiment 
nBaro0 = find(t_ll>=t_Baro0,1);
nBaro1 = find(t_ll>=t_Baro1,1);
n0_ll  = find(t_ll>=t0,1);
n1_ll  = find(t_ll>=t1,1);

level_baro0 = mean(level(nBaro0:nBaro1));


% Plot complete raw dataset
figure
subplot(2,1,1)
hold all
plot(t_ll,level,'DisplayName','h [m]');
plot([t_ll(nBaro0),t_ll(nBaro1)],repmat(level_baro0,1,2),'m', ...
    'DisplayName','Baro 0');
%xline(t_ll(nBaro1),'m','Baro 0','LabelVerticalAlignment','bottom');
hold off
%
ylabel('Water level [m]')
title('Measurement in water reservoir over time, raw data');
legend
box on
%
subplot(2,1,2)
plot(t_ll,temp,'r');
ylabel('Temperature [C]')
xlabel('Time [s]')
legend('T[C]')


% Convert time to relative duration after input MP, convert water height
t_ll = seconds(t_ll - tZ);
V = fct_convert(level-level_baro0,'LL');

% Abschneiden Infos vor/nach Experiment
V    =    V(n0_ll:n1_ll);
temp = temp(n0_ll:n1_ll);
t_ll = t_ll(n0_ll:n1_ll);

% Maximalvolumen Wasser
V_ttl = max(V);
c_calc = m_zug/V_ttl;

% Plot converted data for experiment
figure
subplot(2,1,1)
plot(t_ll,V);
%ylim([200 250])
ylabel('Volume in reservoir [L]')
legend('V [L]')
title(sprintf('Monitoring of Water Reservoir over time, converted, V max = %s L',num2str(V_ttl,'%.1f')));

subplot(2,1,2)
plot(t_ll,temp,'r');
xlabel('Time [s]')
ylabel('Temperature [C]')
legend('T[C]')
print('-r300',strcat(pfad.graphics,exp_date,'-Levellogger'),'-dpng')

clear data ll_hour ll_day level

%% Homogenizing dataset: one reference timeset, interpolation
% Switch to reference time and 1s MI, fill gaps (1d-Interpol)
t_hom = (max([t_FluoIn(1),t_FluoOut(1),t_HoboH(1), t_HoboQ(1)]) :...
         min([t_FluoIn(end),t_FluoOut(end),t_HoboH(end), t_HoboQ(end)]))';

c_FluoInHom = interp1(t_FluoIn, c_FluoIn12, t_hom);
c_FluoOutHom = interp1(t_FluoOut, c_FluoOut12, t_hom);
Qhom    = interp1(t_HoboQ, Q,   t_hom);
Vhom    = interp1(t_ll, V,      t_hom);
Thom    = interp1(t_ll, temp,   t_hom);

% Exclude negative or nan concentrations
c_FluoInHom(  c_FluoInHom  < 0 | isnan(c_FluoInHom )) =0;
c_FluoOutHom( c_FluoOutHom < 0 | isnan(c_FluoOutHom)) =0;

% Convert to mm: signal to distance from device, then to height over bottom
for i=1:6
    device = sprintf('dev%i',i);   
    US.(device).hhom = interp1(US.(device).t,US.(device).h,t_hom);
end

qty_jumpsFluoIn   =   nnz((t_FluoIn( 2:end) - t_FluoIn( 1:end-1))-2);
qty_jumpsFluoOut  =   nnz((t_FluoOut(2:end) - t_FluoOut(1:end-1))-2);
qty_jumpsHoboQ = nnz((t_HoboQ(2:end) - t_HoboQ(1:end-1))-1);
qty_jumpsHoboH = nnz((t_HoboH(2:end) - t_HoboH(1:end-1))-1);

fprintf('Interpolated datajumps:\n')
fprintf('Fluo In:  %i/%i\n',[qty_jumpsFluoIn, length(t_FluoIn)])
fprintf('Fluo Out: %i/%i\n',[qty_jumpsFluoOut, length(t_FluoOut)])
fprintf('Hobo Q: %i/%i\n',[qty_jumpsHoboQ, length(t_HoboQ)])
fprintf('Hobo h: %i/%i\n',[qty_jumpsHoboH, length(t_HoboH)])

%% Some metrics of Flkuorometer data (FHWM, 1/e2)
for i = 1:2  
    if i == 1
        c = c_FluoInHom;
        ident = 'Fluo-Inlet';
    else
        c = c_FluoOutHom;
        ident = 'Fluo-Outlet';
    end
   
    [c_max, idx_max] = max(c); % Peak concentration
    t_max = t_hom(idx_max);    % Peak time
    
    % Half concentration times (FWHM)
    fwhm_c   = 0.5*c_max;
    fwhm_int = find(c>fwhm_c);
    
    % Extract Interval where c exceeds/falls below 50% c
    fwhm_times_left  = t_hom(fwhm_int(1) + (-1:0));
    fwhm_values_left =     c(fwhm_int(1) + (-1:0));
    
    fwhm_times_right  = t_hom(fwhm_int(end) + (0:1));
    fwhm_values_right =     c(fwhm_int(end) + (0:1));
    
    fwhm_t1 = interp1(fwhm_values_left,  fwhm_times_left, fwhm_c);
    fwhm_t2 = interp1(fwhm_values_right, fwhm_times_right, fwhm_c);
    
    fwhm_t = fwhm_t2-fwhm_t1;
    
    
    % 1/e^2 concentration times
    e2_c   = 1/exp(1)^2 * c_max;
    e2_int = find(c>e2_c);
    
    % Extract Interval where c exceeds/falls below 50% c
    e2_times_left  = t_hom(e2_int(1) + (-1:0));
    e2_values_left =     c(e2_int(1) + (-1:0));
    
    e2_times_right  = t_hom(e2_int(end) + (0:1));
    e2_values_right =     c(e2_int(end) + (0:1));
    
    e2_t1 = interp1(e2_values_left,  e2_times_left, e2_c);
    e2_t2 = interp1(e2_values_right, e2_times_right, e2_c);
    e2_t = e2_t2-e2_t1;
    
    metrics.t_e2(i) = e2_t;
    metrics.t_fwhm(i) = fwhm_t;
    metrics.t_max(i) = t_max;
    metrics.c_max(i) = c_max;
    
    figure
    hold all
    plot(t_hom,c,'r')
    plot([fwhm_t1, fwhm_t2],[fwhm_c, fwhm_c],'k-')
    plot([e2_t1, e2_t2],[e2_c, e2_c],'b-')
    hold off
    title({ident,...
        sprintf('FWHM: c(1/2)  = %2.2f ug/L, t(1/2)  = %2.2f s',fwhm_c,fwhm_t), ...
        sprintf('1/e2: c(1/e2) = %2.2f ug/L, t(1/e2) = %2.2f s', e2_c, e2_t)})
    fullfilename = strcat(pfad.graphics,exp_date,'-',ident,'-timeMetrics.png');
    print('-r300',fullfilename,'-dpng')  
end

%% Data smoothing
% Remove outliers
outlier_FluoIn  = isoutlier(c_FluoInHom,'movmedian',10);
outlier_FluoOut = isoutlier(c_FluoOutHom,'movmedian',10);
nnz(outlier_FluoIn);
nnz(outlier_FluoOut);

% Moving median filter with 10 (seconds, or longer if outlier)
cf_FluoIn  = movmedian(c_FluoInHom(~outlier_FluoIn), 10);
cf_FluoOut = movmedian(c_FluoOutHom(~outlier_FluoOut),10);
Qf         = movmedian(Q,10)

% Plotting smoothed data Fluo input
figure
hold all
plot(t_hom,c_FluoInHom,'r')
plot(t_hom(outlier_FluoIn),c_FluoInHom(outlier_FluoIn),'mx','MarkerSize',10)
plot(t_hom(~outlier_FluoIn),c_FluoInHom(~outlier_FluoIn),'b')
plot(t_hom(~outlier_FluoIn),cf_FluoIn,'g','LineWidth',2)
hold off
legend('Alldata', 'Outlier','Outlier removed','Outlier removed and smoothed')
title('Spline-smoothed data of input fluorometer')
xlabel('Duration after MP input')
ylabel('Concentration MP [g/L]')


% Plotting smoothed data Fluo output
figure
hold all
plot(t_hom,c_FluoOutHom,'r')
plot(t_hom(outlier_FluoOut),c_FluoOutHom(outlier_FluoOut),'mx','MarkerSize',10)
plot(t_hom(~outlier_FluoOut),c_FluoOutHom(~outlier_FluoOut),'b')
plot(t_hom(~outlier_FluoOut),cf_FluoOut,'g','LineWidth',2)
hold off
legend('Alldata', 'Outlier','Outlier removed','Outlier removed and smoothed')
title('Spline-smoothed data of output fluorometer')
xlabel('Duration after MP input')
ylabel('Concentration MP [g/L]')


figure
hold all
scatter(t_hom,Qhom,'rx','MarkerEdgeAlpha',0.5)
plot(t_hom, Qf, 'g','LineWidth',2)
hold off
title('Spline-smoothed data of discharge')
xlabel('Duration after MP input')
ylabel('Discharge [L/s]')

%% Recovery Rate
% Extract start, peak and end of breakthrough ("bt") for recovery analysis: 
% (1) Start: 20 values before first value > 100 ugL (because of outliers)
btFirstWaveBeg_FluoIn = find(c_FluoInHom  > 100, 1) - 20 ;
btFirstWaveBeg_FluoOut = find(c_FluoOutHom > 100, 1) - 20 ;

% (2) Peak in breakthrough
% Inflow
[~, crLocs] = findpeaks(c_FluoInHom,'MinPeakProminence',50); % 500
btFirstWavePeak_FluoIn = crLocs(1);
% Outflow
[~, crLocs] = findpeaks(c_FluoOutHom,'MinPeakProminence',50); % 500
btFirstWavePeak_FluoOut = crLocs(1); %end);

% (3) End: deepest (first) valley from findpeaks
% Change only if recirculating flow
btFirstWaveEnd_FluoIn = length(t_hom);
btFirstWaveEnd_FluoOut = length(t_hom);
%{
% Inflow
[crVals, crLocs] = findpeaks(-c_28hom,'MinPeakProminence',200);
[~, crLocsmin] = min(abs(crVals));
btFirstWaveEnd_28 = crLocs(crLocsmin)
% Outflow
[crVals, crLocs] = findpeaks(-c_29hom,'MinPeakProminence',50);
[~, crLocsmin] = min(abs(crVals));
btFirstWaveEnd_29 = crLocs(crLocsmin)
%}

% (4) Define Intervals for analysis
% Breakthrough first wave
btFirstWaveInt_FluoIn  = btFirstWaveBeg_FluoIn:btFirstWaveEnd_FluoIn;
btFirstWaveInt_FluoOut = btFirstWaveBeg_FluoOut:btFirstWaveEnd_FluoOut;
%
% Breakthrough complete dataset
btCompleteInt_FluoIn  = btFirstWaveBeg_FluoIn:length(t_hom);
btCompleteInt_FluoOut = btFirstWaveBeg_FluoOut:length(t_hom);
%
% Breakthrough: Fitting the tail, peak and trough
btFirstWaveTailInt_FluoIn  = btFirstWavePeak_FluoIn:btFirstWaveEnd_FluoIn;
btFirstWaveTailInt_FluoOut = btFirstWavePeak_FluoOut:btFirstWaveEnd_FluoOut;

% (5) Residence Time Distribution c*Q/m [1/s]:, c [g/L], Q [L/s], m [g]
rtd_in(btCompleteInt_FluoIn,1) = c_FluoInHom(btCompleteInt_FluoIn) .* Qhom(btCompleteInt_FluoIn) / m_zug;
rtd_out(btCompleteInt_FluoOut,1) = c_FluoOutHom(btCompleteInt_FluoOut) .* Qhom(btCompleteInt_FluoOut) / m_zug;
rtdCut_in = rtd_in(btFirstWavePeak_FluoIn:btFirstWaveEnd_FluoIn);
rtdCut_out = rtd_out(btFirstWavePeak_FluoOut:btFirstWaveEnd_FluoOut);
rtd_cmin = cmin .* Qhom(btCompleteInt_FluoIn) / m_zug;
rtd_cmax = cmax .* Qhom(btCompleteInt_FluoIn) / m_zug;

% (6) Recovery during breakthrough: integrate rtd over time [s] --> [-] dimless number
% Start at begin breakthrough, before set to 0
rec_in_cum(btCompleteInt_FluoIn,1) = cumtrapz(t_hom(btCompleteInt_FluoIn),rtd_in(btCompleteInt_FluoIn));
rec_out_cum(btCompleteInt_FluoOut,1) = cumtrapz(t_hom(btCompleteInt_FluoOut),rtd_out(btCompleteInt_FluoOut));

% (7) Value for MP recovery during first wave
rec_in_bt  = rec_in_cum(btFirstWaveEnd_FluoIn) * 100;
rec_out_bt  = rec_out_cum(btFirstWaveEnd_FluoOut) * 100;
metrics.recFluo = [rec_in_bt, rec_out_bt];

% Plot: Breakthrough Interval with Data
figure
hold all
plot(t_hom(btFirstWaveBeg_FluoIn-30:btFirstWaveEnd_FluoIn) ,c_FluoInHom(btFirstWaveBeg_FluoIn-30:btFirstWaveEnd_FluoIn),'b')
plot(t_hom(btFirstWaveBeg_FluoOut-30:btFirstWaveEnd_FluoOut) ,c_FluoOutHom(btFirstWaveBeg_FluoOut-30:btFirstWaveEnd_FluoOut),'r')
%
xline(t_hom(btFirstWaveBeg_FluoIn),'-b','Inflow Start','LabelHorizontalAlignment','left');
xline(t_hom(btFirstWaveEnd_FluoIn),'-b','Inflow End'  ,'LabelHorizontalAlignment','left');
%
xline(t_hom(btFirstWaveBeg_FluoOut),'--r','Outflow Start','LabelHorizontalAlignment','right');
xline(t_hom(btFirstWaveEnd_FluoOut),'--r','Outflow End'  ,'LabelHorizontalAlignment','right');
hold off

title(sprintf('Intervals for analysis of breakthrough curves, Exp %s',exp_date));
xlabel('Time [s]')
ylabel('Concentration [g/L]')


% Plot (2): MP transport / residence time distribution 
clear p
figure
hold all
yyaxis left
p(1) = plot(t_hom(btCompleteInt_FluoIn) ,rtd_in(btCompleteInt_FluoIn),'b-','DisplayName','Inflow');
p(2) = plot(t_hom(btCompleteInt_FluoOut) ,rtd_out(btCompleteInt_FluoOut),'r-','DisplayName','Outflow');
%
plot(t_hom(btFirstWaveTailInt_FluoIn([1,end])),rtd_in(btFirstWaveTailInt_FluoIn([1,end])),'kx', 'LineWidth',2)
plot(t_hom(btFirstWaveTailInt_FluoOut([1,end])),rtd_out(btFirstWaveTailInt_FluoOut([1,end])),'kx', 'LineWidth',2)
%
xline(t_hom(btFirstWaveEnd_FluoIn),'-b',{'Inflow End'}, ...
    'LabelHorizontalAlignment','left','LabelVerticalAlignment','top');
xline(t_hom(btFirstWaveEnd_FluoOut),'-r',{'Outflow End'}, ...
    'LabelHorizontalAlignment','right','LabelVerticalAlignment','top');
hold off

xlabel('Time [s]')
ylabel('$\displaystyle\frac{c(t)\cdot Q(t)}{m_{input}} [s^{-1}]$', 'Interpreter','latex')
set(gca,'XScale','log','YScale','log')
set(gca,'YColor','k')

yyaxis right
color_dg = [0 150 0]./255;
p(3) = plot(t_hom(btCompleteInt_FluoIn),Qhom(btCompleteInt_FluoIn),...
    'Color',color_dg,'DisplayName','Discharge');
ylim([0 5])
set(gca,'YColor',color_dg)
ylabel('Q [L/s]')

title(sprintf('Masstransport, without sediment Exp %s',exp_date));
leg = legend(p);
leg.Location = 'best';
leg.ItemTokenSize = [10,4];

% Plot (3): Cumulative MP Transport 
figure
hold all
plot(t_hom,rec_in_cum,'b')
plot(t_hom,rec_out_cum,'r')
xline(t_hom(btFirstWaveEnd_FluoIn),'-b',{'Inflow End'},'LabelHorizontalAlignment','left');
xline(t_hom(btFirstWaveEnd_FluoOut),'-r',{'Outflow End'},'LabelHorizontalAlignment','right');
hold off
%
title(sprintf('Cumulative MP mass transport, Exp %s',exp_date));
legend('Inflow 28', 'Outflow 29','Location','east')
xlabel('Time [s]')
ylabel('$\int c\cdot Q d$t$ /m_{input}$ ','Interpreter','latex')
set(gca,'XScale','log','YScale','log')

%% Save for comparison
close all
save([pfad.results,exp_date,'-exp-wo-cam.mat'])

%% Export for Franzl
c_FluoInHom(c_FluoInHom==0) = nan;
c_FluoOutHom(c_FluoOutHom==0) = nan;
export_wocam = [t_hom'; c_FluoInHom'; c_FluoOutHom'; Qhom'; ...
    US.dev1.hhom'; US.dev2.hhom'; US.dev3.hhom'; US.dev4.hhom'; 
    US.dev5.hhom'; US.dev6.hhom'; Thom'; Vhom'];

fileID = fopen([pfad.results,exp_date,'-results_wo_cam.txt'],'w');
fprintf(fileID,'Experiment %s\n',exp_date);
fprintf(fileID,'Results of: Fluorometer, MID, Ultrasonics, Levelogger\n');
fprintf(fileID,'Input MP at t = 0 at x = 0\n');
fprintf(fileID,['t = time [s], c = concentration [ug/L],' ...
    'Q = discharge [L/s], h = waterheight [m], T = Temperature [C]' ...
    'V = Volume in Reservoir [L], 3digit number = x-Position measurement'],exp_date);
fprintf(fileID,['\n%s\t%s\t%s\t%s\t' ...
    'h%i\th%i\th%i\th%i\th%i\th%i\t%s\t%s\t\n'], ...
    't', 'c240', 'c489', 'Q', ...
    US.dev1.xpos, US.dev2.xpos, US.dev3.xpos, ...
    US.dev4.xpos, US.dev5.xpos, US.dev6.xpos, ...
    'T', 'V');
fprintf(fileID,['%3d\t%4.2f\t%4.2f\t%4.3f\t' ...
    '%4.4f\t%4.4f\t%4.4f\t%4.4f\t%4.4f\t%4.4f\t' ...
    '%4.1f\t%4.1f\t\n'],export_wocam);
fclose(fileID);

msgbox('Script finished','Yeay')
% EOF