% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Comparison of experiments (without Cam)
clear all; close all; clc

exp_date = {'220727','220803','210812','220818','220901'}';
exp_text = {'1 m', '3 m', '10 m','fine sand', 'high flow'};

exp_name = string(strcat('exp',exp_date));
pfad     = fct_pfad(char(exp_date));
qty_Exp  =   length(exp_date); 

%% Compare PIVs
for i = 1 : qty_Exp
    pivFile = strcat(pfad.results(i,:),exp_date{i},'-pivresults_all.mat');
    piv_i = ['exp',exp_date{i}];
    pivData.(piv_i) = load(pivFile,'-mat'); 
    
    u_mean(i)      = pivData.(piv_i).u_mean;
    u_meanQuick(i) = pivData.(piv_i).u_meanQuick;
    u_meanSlow(i)  = pivData.(piv_i).u_meanSlow;
    
    TKE_mean(i)    = nanmean(pivData.(piv_i).TKE_all,'all');
    TKE_max(i)     = nanmax(pivData.(piv_i).TKE_all,[],'all');
end
%clear pivData

%% Read Fluo data
for i = 1:qty_Exp
    expFile = strcat(pfad.results(i,:),exp_date{i},'-exp-wo-cam.mat');
    exp_i = ['exp',exp_date{i}];
    expData.(exp_i) = load(expFile,'-mat');
    
    t_hom{i}  = expData.(exp_i).t_hom;
    idx = t_hom{i} > 0;
    t_hom{i} = t_hom{i}(idx);
      
    rtd_FluoIn{i}  = expData.(exp_i).rtd_in(idx);
    rtd_FluoOut{i} = expData.(exp_i).rtd_out(idx);
    
    c_FluoIn{i}  = expData.(exp_i).c_FluoInHom(idx);
    c_FluoOut{i} = expData.(exp_i).c_FluoOutHom(idx);
    
    m_zug{i} = expData.(exp_i).m_zug;
    LOQ_FluoIn{i}  = expData.(exp_i).LOQ_FluoIn;
    LOQ_FluoOut{i} = expData.(exp_i).LOQ_FluoOut;
    
    Q_c{i}    = expData.(exp_i).Qhom(idx);  
    rec(i,:)  = expData.(exp_name(i)).metrics.recFluo;
end

%% Metric Fluo breakthrough
[rtdmax,rtdtim] = cellfun(@max, rtd_FluoOut');
rtdmax_rel = 100 * rtdmax ./ rtdmax(1);
rtdtim_rel = 100 * rtdtim ./ rtdtim(1);

% Print some results
clear res
res{1} = sprintf('Comparison of maximum of normalized mass transfer rates, compared to 1 m');
res{2} = sprintf('Experiments: %s, %s, %s, %s, %s\n',exp_text{:});
res{3} = sprintf('Absolute arrival time [s]  at %.0f, %.0f, %.0f, %.0f, %.0f',rtdtim);
res{4} = sprintf('Absolute maximum NMT [1/s] at %.3f, %.3f, %.3f, %.3f, %.3f\n',rtdmax);
res{5} = sprintf('Absolute arrival time [%%] at %.1f, %.1f, %.1f, %.1f, %.1f',rtdtim_rel);
res{6} = sprintf('Relative maximum NMT [%%]  at %.0f, %.0f, %.0f, %.0f, %.0f\n',rtdmax_rel);
res{7} = sprintf('Input masses Microplastics [mg]: %.0f, %.0f, %.0f, %.0f, %.0f',[m_zug{:}]/1000);
res{8} = sprintf('Recovery rate Fluo inlet [%%]:  %.1f, %.1f, %.1f, %.1f, %.1f',rec(:,1));
res{9} = sprintf('Recovery rate Fluo outlet [%%]: %.1f, %.1f, %.1f, %.1f, %.1f',rec(:,2));
msgbox(res,'Fluo Breakthrough results')

%% Smoothing
for i = 1:qty_Exp
    % Improve smoothing by 
    % (1) Finding outliers
    outlier_FluoIn  = isoutlier(rtd_FluoIn{i} ,'movmedian',10);
    outlier_FluoOut = isoutlier(rtd_FluoOut{i},'movmedian',10);
    %
    % (2) Finding zero values
    iszero_FluoIn =  (rtd_FluoIn{ i} == 0);
    iszero_FluoOut = (rtd_FluoOut{i} == 0);
    %
    % (3) and removing them
    remove_FluoIn  = outlier_FluoIn  | iszero_FluoIn;
    remove_FluoOut = outlier_FluoOut | iszero_FluoOut;
    %
    % (4) see how much was removed 
    nnz(remove_FluoIn);
    nnz(remove_FluoOut);
    
    
    % Smoothing parameters for spline interpolation
    smoothPrm.In = 0.999;
    smoothPrm.Out = 0.999;
    smoothPrm.Q  = 0.9;

    % Smoothing data (in log-transform)
    [fit_FluoIn, gofIn] = fit(log10(t_hom{i}(     ~remove_FluoIn)), ...
                              log10(rtd_FluoIn{i}(~remove_FluoIn)), ...
                              'smoothingspline', ...
                              'SmoothingParam', smoothPrm.In);
    [fit_FluoOut, gofOut] = fit(log10(t_hom{i}(      ~remove_FluoOut)), ...
                                log10(rtd_FluoOut{i}(~remove_FluoOut)), ...
                                'smoothingspline', ...
                                'SmoothingParam', smoothPrm.Out);
    [fit_Q, gofQ,~] = fit( log10(t_hom{i}), log10(Q_c{i}), ...
                                'smoothingspline','SmoothingParam', smoothPrm.Q);
    
    rtdFit_FluoIn{i}  = 10.^(feval(fit_FluoIn, log10(t_hom{i})));
    rtdFit_FluoOut{i} = 10.^(feval(fit_FluoOut,log10(t_hom{i})));
    Q_Fit{i}           = 10.^(feval(fit_Q,      log10(t_hom{i})));    
    
    
    % Fit of rtd must be above Quantification Limit
    LOQ_rtdIn{i} = LOQ_FluoIn{i}.*Q_Fit{i}./m_zug{i};
    LOQ_rtdOut{i} = LOQ_FluoOut{i}.*Q_Fit{i}./m_zug{i};
    
    rtdFit_FluoIn{i}(rtdFit_FluoIn{i}<LOQ_rtdIn{i}) = 0;
    rtdFit_FluoOut{i}(rtdFit_FluoOut{i}<LOQ_rtdOut{i}) = 0;
    
end

% Calculate scaled limit of quantification (cQ/m0)
Q_mean = cellfun(@mean, Q_Fit);
LOQ_rtd = [LOQ_FluoOut{:}] .* Q_mean ./ [m_zug{:}];

% Grafik
clear p
colors = [255,0,127; 54,53,55; 12,206,107; 0,255,255; 0,0,255]./255;


% Figure: compare Fluo outlet 
exp_show = 1:5; % Choose which experiments to compare
figure
hold all
for i = exp_show
    scatter(t_hom{i},rtd_FluoOut{i},'x','MarkerEdgeAlpha',0.01, 'MarkerEdgeColor',colors(i,:));
    p(i) = plot(t_hom{i},rtdFit_FluoOut{i},'Color',colors(i,:),'LineWidth',1); 
    plot(t_hom{i}, LOQ_rtdOut{i},'Color',colors(i,:),'LineStyle','--')
end
hold off
set(gca,'XScale','log','XLim',[30,2000],'XTick',10.^(1:3))
set(gca,'YScale','log','YLim',10.^-[6,1.9])

xlabel('Time [s]')
ylabel('$\displaystyle\frac{c\cdot Q}{m_0 }[s^{-1}]$', 'Interpreter','latex')
leg = legend(p(exp_show),exp_text(exp_show));
leg.Position = [0.72, 0.65, 0.25, 0.25];
leg.ItemTokenSize = [8,2];

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')

filename = strcat(pfad.comparison,'rtd_MP.png');
print('-r300',filename,'-dpng');

clear p

%% Calculate stored MP in study area (in-out)
% Prealloc
rtdDifference = cell(1,5);
t_threshold1 = nan(5,1);
t_threshold2 = nan(5,1);
c_Storage_TH1 = 5; % Threshold 5 %
c_Storage_TH2 = 40; % Threshold 40 %

for i = 1 : 5
    % Calculate from cumulative sums of inflow and outflow
    rtdDifference{i} = 100*(1-cumsum(rtd_FluoOut{i})./cumsum(rtd_FluoIn{i}));
    
    % Find mass stored at the end of experiment
    mass_end(i,1) = rtdDifference{i}(end);
    
    % Find index when less than threshold mass stored     
    idx1 = find(rtdDifference{i}<c_Storage_TH1,1,'first');
    idx2 = find(rtdDifference{i}<c_Storage_TH2,1,'first');     
    try  t_threshold1(i,1) = t_hom{i}(idx1); end
    try  t_threshold2(i,1) = t_hom{i}(idx2); end
end

% Print some results
clear res
res{1} = sprintf('Mass storage and release in interrogation area\n');
res{2} = sprintf('Experiments: %s, %s, %s, %s, %s',exp_text{:});
res{3} = sprintf('Time [s] until stored mass < %i %%; %.0f, %.0f, %.0f, %.0f, %.0f',c_Storage_TH2,t_threshold2);
res{4} = sprintf('Time [s] until stored mass < %i %%: %.0f, %.0f, %.0f, %.0f, %.0f',c_Storage_TH1,t_threshold1);
res{5} = sprintf('Mass [%%] stored at experiment end;  %.1f, %.1f, %.1f, %.1f, %.1f',mass_end);
msgbox(res,'Fluo Breakthrough metrics')

figure; 
hold all; 
for i = 1 : 5
    plot(t_hom{i},rtdDifference{i},'Color',colors(i,:),'LineWidth',1); 
end
xlim([30,1800])
ylim([0,103])
set(gca,'XScale','log','XTick',10.^(1:3))
leg = legend(exp_text,'Location','northeast');
leg.ItemTokenSize = [8,2];
leg.Position = [0.72, 0.65, 0.25, 0.25];
xlabel('Time [s]')
ylabel('$\displaystyle\frac{m_{out}-m_{in}}{m_{in}}[\%]$', 'Interpreter','latex')

set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')

filename = strcat(pfad.comparison,'Fluo_total_MP_mass_out_vs_in.png');
print('-r300',filename,'-dpng');

%% Export all mass transfers as csv
t_export = t_hom{1};
exportMatrix = fct_export(t_export, ...
    rtdFit_FluoIn{ 1},rtdFit_FluoIn{ 2},rtdFit_FluoIn{ 3},rtdFit_FluoIn{ 4},rtdFit_FluoIn{ 5},...
    rtdFit_FluoOut{1},rtdFit_FluoOut{2},rtdFit_FluoOut{3},rtdFit_FluoOut{4},rtdFit_FluoOut{5});

filename = strcat(pfad.comparison,'concFIT_export.txt');
fileID = fopen(filename,'w');
fprintf(fileID,'Exported at %s\n\n', datestr(datetime));
fprintf(fileID,'Results of Fluorometer for 5 experiments:\n');
fprintf(fileID,['Experiment1: 1 um (Reference) \nExperiment2: 3 um \n' ...
    'Experiment3: 10 um \nExperiment4: ds/3 (fine sediment) \n' ...
    'Experiment5: 2Q (high discharge)\n\n']);
fprintf(fileID,'Fluo inflow at x = 240 cm, Fluo outflow at x = 489 cm\n');
fprintf(fileID,'Input MP at t = 0 at x = 0\n');
fprintf(fileID,'t = time [s], c = normalised mass transfer rate [1/s] (c*Q/m0) \n\n');
fprintf(fileID,['Flagging of concentrations: \n' ...
    'c = 0 if below detection limit, \n' ...
    'c = nan if not measured (lack of DI water)\n\n']);
fprintf(fileID,'t \t c_in,1 \t c_in,2 \t c_in,3 \t c_in,4 \t c_in,5 \t ');
fprintf(fileID,'c_out,1 \t c_out,2 \t c_out,3 \t c_out,4 \t c_out,5 \n');

fprintf(fileID, '%i \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \t %.2e \n',exportMatrix);
fclose(fileID);

%% Analysing water surface elevations
clear H X
for i = 1 : length(exp_name)
    X(i,:)   = expData.(exp_name(i)).X; % Measurement positions
    H(i,:)   = mean(expData.(exp_name(i)).H); % Water heights, mean over time
    Q_MID(i,:) = mean(expData.(exp_name(i)).Qfit); % Mean discharge over time
end
X = X./100;

% Surface slope
for i = 1 : 5
    lm = fitlm(X(i,1:4), H(i,1:4));
    slopes(i) = lm.Coefficients.Estimate(2);
end


% Print some hydrodynamical results
clear res
res{1} = sprintf('Metrics of hydrodynamics');

% Mean slope
res{2} = sprintf('Mean slope of %1.1f \x00B1 %1.1f mm/m', ...
    1000*mean(slopes), 1000*std(slopes));

% Overflow heights
h_overflow = max(H,[],2)-0.15;           % max - 15 cm height weir height
res{3} = sprintf('Mean overflow height: %.0f mm low-flow, %.0f mm high-flow\n', ...
    1000*mean(h_overflow(1:4)), 1000*h_overflow(5));

% Discharge from overflow height (Thompson)
mu = 0.667; % berfallbeiwert as mean from GUNT manual

Q_thompson = 8/15*mu*tand(60/2)*(2*9.81)^0.5*h_overflow.^2.5*1000; % L/s

Q_diff = (Q_thompson-Q_MID)./Q_MID*100; % deviation in percent for paper
res{4} = sprintf('Discharge from Thompson %.0f %% higher than MID',mean(Q_diff));
res{5} = sprintf('Discharge from MID [L/s]: low-flow %.2f, high-flow: %.2f\n', mean(Q_MID(1:4)), Q_MID(5));

res{6} = sprintf('Experiments: %s, %s, %s, %s, %s',exp_text{:});
res{7} = sprintf('Discharge from MID [m/h]: %.2f, %.2f, %.2f, %.2f, %.2f',3.6*Q_MID');
res{8} = sprintf('Complete zone mean velocity [m/s]: %.3f, %.3f, %.3f, %.3f, %.3f, (%.2f)',u_mean', mean(u_mean(1:4)));
res{9} = sprintf('Upper zone mean velocity [m/s]: %.3f, %.3f, %.3f, %.3f, %.3f, (%.2f)',u_meanQuick',mean(u_meanQuick(1:4)'));
res{10} = sprintf('Lower zone mean velocity [m/s]: %.3f, %.3f, %.3f, %.3f, %.3f, (%.2f)',u_meanSlow',mean(u_meanSlow(1:4)'));
msgbox(res,'Hydrodynamical metrics');


%% Water surface: graphics
figure
colors = [255,0,127; 54,53,55; 12,206,107; 0,255,255; 0,0,255]./255;
colororder(colors)
%
plot(X',H','LineWidth',1)
%
leg = legend(exp_text);
leg.ItemTokenSize = [10,4];
leg.NumColumns = 2;
leg.Location = 'northeast';
%
xlabel('Longitudinal position [m]')
ylabel('Water level [m]')
ylim([0.18,0.22])
yticks(0.15:0.01:0.22)
xtickformat('%.2f')
ytickformat('%.2f')
%
ax = gca;
ax.Position = [0.20, 0.20, 0.75, 0.75];
ax.XDir = 'reverse';
%
set(gcf,'units','centimeters','position',[10,10,7.5,7])
set(findall(gcf,'-property','FontSize'),'FontSize',11) 
set(findall(gcf,'-property','fontname'),'FontName','helvetica')
%
fullfilename = strcat(pfad.comparison,'waterLevel');
print('-r300',fullfilename,'-dpng');

% EOF