% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Evaluate individual FIS calibration

%% Define settings
clear all; close all; clc
set(0,'DefaultFigureColormap',hot)

% Choose experiment date
exp_date = '220901';
pfad = fct_pfad(exp_date);

% Choose calibrations which shall be evaluated (center ROI + 
% bigger ROI for Aquarium only)
cal_name = ["1_Flume_liq"; "2_Flume_sed"; "3_Aquarium";
            "4_cal_comp";  "5_cal_pre";   "6_cal_pos"; 
            "3_Aquarium_Flume"];
cal_2do = listdlg('PromptString','Which cals shall be evaluated?', ...
    'ListString',cal_name);

% Gauss-filtering of calibration results
sigma_g = 2;

% Suppress figure output for calibration
silence = false; % if true: don't show individual calibration graphics


%% Evaluate
for k = 1 : length(cal_2do)
    exp_name = char(cal_name(cal_2do(k)));    
    analysis = ["cuboid", "flume"];
    
    if cal_2do(k) < 7        
        analysis = char(analysis( 1 ));
    else
        analysis = char(analysis( 2 ));
        exp_name = exp_name(1:end-6); % Jump to normal folder
    end
       
    exp_folder = strcat('1_Calibration/',exp_name,'/');
    dpix_reg = 10;          % Spatial averaging over dpix^2 
    CalcIntercept = false;  % false: y = mx + 0; true: y = mx + b

    %% Check for available Data
    pfad.MatlabCal = strcat(pfad.FIS,'matlab/',exp_folder);
    pfad.FISCalRes = strcat(pfad.FIS,'matlab/4_Cal-results/',num2str(dpix_reg),'pix/');
    
    vorhandene_dateien  = dir([pfad.MatlabCal, '*.mat']);
    dateinamen          = string({vorhandene_dateien.name});
    nr_cal              = numel(dateinamen);
    
    if nr_cal == 0
        warning('No data found in %s',pfad.MatlabCal)
        continue
    end
    fct_isFolder(pfad)
    
    %% Read out labbook: concentration, volume, ROIs
    labbook = readcell('../labbook.xlsx','Sheet',exp_date);
    
    % Data from labbook: 
    % (1) concentration of MP calibration stock solution  [mg/L] 
    c_Stl = labbook{strcmpi(labbook,'Stl conc'),2};
    %
    % (2) volume threshold for linear regression [uL]    
    V_lin = labbook{strcmp(labbook,'Volumen linear'),2};
    %
    % (3) flask volume [mL]
    if contains(exp_folder,'Aq','IgnoreCase',true)
        V_ttl = labbook{strcmpi(labbook,'Vol Aq'),2};
    else 
        V_ttl = labbook{strcmp(labbook,'Vol total'),2};
    end
    %
    % (4a) ROI ranges for center cuboid
    x_range_c = labbook{strcmpi(labbook,'Range Cuboid'),2}; %
    y_range_c = labbook{strcmpi(labbook,'Range Cuboid'),3}; %
    x_cuboid = str2double(strsplit(x_range_c,'-'));
    y_cuboid = str2double(strsplit(y_range_c,'-'));
    %
    % (4b) Extract Ranges for ROI aquarium
    x_range_a = labbook{strcmpi(labbook,'Range Aquarium'),2}; % 
    y_range_a = labbook{strcmpi(labbook,'Range Aquarium'),3}; % 
    x_aquarium = str2double(strsplit(x_range_a,'-'));
    y_aquarium = str2double(strsplit(y_range_a,'-'));
    
    %% Read in datasets
    for m = 1 : nr_cal
        load(strcat(pfad.MatlabCal,dateinamen(m)), ...
            'A_fovmean','A_timemean','A_CI','V_Stl','time_mean', ...
            'NumInt', 'ExpTime','int_x', 'int_y');
        A(:,:,m)   = A_timemean / str2double(NumInt);
        A_fov_m(:,m)  = A_fovmean / str2double(NumInt);
        A_CI_m(:,:,m) = A_CI;
        V_Stl_m(m) = V_Stl;
        clear A_timemean A_fovmean V_Stl A_CI
    end

    % Rename
    V_Stl = V_Stl_m';
    A_fov = A_fov_m;
    A_CI = A_CI_m;
   
    % Calculate concentration from added volume of stl
    c = c_Stl*V_Stl/V_ttl; % [c] =  mg/L * L / mL = g/L
     
    clear V_Stl_m A_fov_m A_CI_m
    
    %% Blank subtraction
    blank = (c == 0);         % Find Index when c == 0   
    if nnz(blank) > 0
        A_blank      = A(:,:,blank);  % Save Blank separately
        A            = A - A_blank;   % Subtract Blank from images
        A(:,:,blank) = [];            % Delete blank from images
        c(blank)     = [];            % delete blank from concentrations
        V_Stl(blank) = [];
        A_fov_blank = A_fov(:,blank);
        A_fov(:,blank) = [];
    else 
        warning('No blank found')
    end
    
    idx_lin = find(V_Stl==V_lin);
     
    % Ensure ascending order
    [c, idx] = sort(c);
    A = A(:,:,idx);
    dateinamen = dateinamen(idx);
    V_Stl = V_Stl(idx);
    A_fov = A_fov(:,idx);
    
    %% Define Calculation Area
    % Print interrogation areas as from labbook
    fprintf('Interrog. Area Aquarium. x: %i-%i, y: %i-%i', ...
        x_aquarium(1), x_aquarium(2), y_aquarium(1), y_aquarium(2))
    fprintf('Interrog. Area Cuboid. x: %i-%i, y: %i-%i', ...
        x_cuboid(1), x_cuboid(2), y_cuboid(1), y_cuboid(2))
    
    % Rectangle Complete Field of View
    ROI_fov = [1 1 size(A_blank,2) size(A_blank,1)];
    
    % Rectangle ROI Cuboid
    w_cuboid   = x_cuboid(2) - x_cuboid(1) + 1;
    h_cuboid   = y_cuboid(2) - y_cuboid(1) + 1;
    ROI_cuboid = [x_cuboid(1) y_cuboid(1) w_cuboid h_cuboid];
    
    % Rectangle ROI Aquarium
    w_flume  = x_aquarium(2) - x_aquarium(1) + 1;
    h2  = y_aquarium(2) - y_aquarium(1) + 1;
    ROI_aquarium = [x_aquarium(1) y_aquarium(1) w_flume h2];
    
    %%%% In relative coordsys Aquarium
    % Distance between ROIs: from Aquarium to Cuboid
    dx_Aq2Cub = x_cuboid - x_aquarium(1) + 1;
    dy_Aq2Cub = y_cuboid - y_aquarium(1) + 1;
    
    w_Aq2Cub   = dx_Aq2Cub(2) - dx_Aq2Cub(1) + 1;
    h_Aq2Cub   = dy_Aq2Cub(2) - dy_Aq2Cub(1) + 1;
    ROI_Aq2Cub = [dx_Aq2Cub(1) dy_Aq2Cub(1) w_Aq2Cub h_Aq2Cub];
    
    % Visualization Matlab: Pixel Center 0.5 for value at pix 1
    ROI_fov = ROI_fov - 0.5*[1 1 0 0];
    ROI_cuboid = ROI_cuboid - 0.5*[1 1 0 0];
    ROI_aquarium = ROI_aquarium - 0.5*[1 1 0 0];
    ROI_Aq2Cub = ROI_Aq2Cub - 0.5*[1 1 0 0];
    
    
    
    %%%%%%%%%%%%%%%%%% Graphics: Choice of ROIs %%%%%%%%%%%%%%%%%%
    if ~silence
        figure
        hold all
        
        % (1) Show background/blank
        imagesc(A_blank)
        
        % (2) Add ROIS
        rectangle('Position', ROI_fov,'EdgeColor','m','LineWidth',2)
        rectangle('Position', ROI_cuboid,'EdgeColor','b','LineWidth',2)
        rectangle('Position', ROI_aquarium,'EdgeColor','g','LineWidth',2)
        rectangle('Position', ROI_Aq2Cub,'EdgeColor','c','LineWidth',2)
        
        % (3) Add text for readability
        text(ROI_fov(3)    + 100, ROI_fov(4)    + 100,'Complete FOV','Color','m','FontWeight','bold')
        text(x_cuboid(2)   + 100, y_cuboid(2)   + 100,'ROI Cuboid','Color','b','FontWeight','bold')
        text(x_aquarium(2) + 100, y_aquarium(2) + 100,'ROI Aquarium','Color','g','FontWeight','bold')
        text(dx_Aq2Cub(2)  + 100, dy_Aq2Cub(2)  + 100,'ROI Difference Aq2Cube','Color','c','FontWeight','bold')
        
        hold off
        title('Blank image with FOV and all ROIs')    
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %% Choose Calculation Area
    if strcmp(analysis,'flume')
        loc_x = x_aquarium;
        loc_y = y_aquarium;
    else  
        loc_x = x_cuboid;
        loc_y = y_cuboid;
    end
    ROI_x = loc_x(1):loc_x(2);
    ROI_y = loc_y(1):loc_y(2);
    A_ROI0 = A_blank(ROI_y,ROI_x);
    A_ROI  = A(ROI_y, ROI_x,:);
    A_CI_ROI = A_CI(ROI_y,ROI_x,:);
    
    %% Grafikausgabe All Image
    if ~silence
        figure('name','Image of All Area / Field of View')
        hold all
        imagesc(A_blank)
        rectangle('Position',ROI_fov,'EdgeColor','m','LineWidth',2)
        rectangle('Position', ROI_cuboid,'EdgeColor','b','LineWidth',2)
        rectangle('Position', ROI_aquarium,'EdgeColor','g','LineWidth',2)
        hold off
        
        set(gca,'YDir','normal')
        axis equal
        axis([0 size(A_blank,2) 0 size(A_blank,1)])
        title('Blank Image with cuboid and aquarium ROI')
        colorbar        

    
        figure('name','All calib points with mean intensity increase ROI')
        for i = 1:size(A,3)
            subplot(ceil(size(A,3)/2),2,i)
            
            hold all    
            imagesc(A(:,:,i))
            rectangle('Position',ROI_fov,'EdgeColor','m','LineWidth',2)
            rectangle('Position', ROI_cuboid,'EdgeColor','b','LineWidth',2)
            rectangle('Position', ROI_aquarium,'EdgeColor','g','LineWidth',2)
            hold off
            
            set(gca,'YDir','normal');
            axis equal
            axis([0 size(A,2) 0 size(A,1)])
            title(sprintf('c = %.1f g/L, mean %i f, mean ROI %.1f',...
                  c(i), time_mean, mean(A_ROI(:,:,i),'all')));
            colorbar
        end
    end
    
    %% Grafikausgabe Only Interrogation Area
    if ~silence
        figure('name','Images Only Interrogation Area')
        hold all
        imagesc(A_ROI0)
        rectangle('Position', ROI_Aq2Cub,'EdgeColor','c','LineWidth',2)
        hold off
        %
        set(gca,'YDir','normal')
        axis equal
        axis([0 size(A_ROI0,2) 0 size(A_ROI0,1)])
        title('Blank Image')
        colorbar
    
        figure
        for i = 1:size(A_ROI,3)
            subplot(ceil(size(A_ROI,3)/2),2,i)
            hold all
            imagesc(A_ROI(:,:,i))
            rectangle('Position', ROI_Aq2Cub,'EdgeColor','c','LineWidth',2)
            hold off
            set(gca,'YDir','normal');
            axis equal
            axis([0 size(A_ROI,2) 0 size(A_ROI,1)])
            title(sprintf(['c = %.1f g/L, ' ...
                ' mean %i f, mean int. %.1f'], ...
                  c(i), time_mean, mean(A_ROI(:,:,i),'all')));
            colorbar
            %caxis([0 5])
        end
    end
    
    %% Delete bad regression points (error during calibration)
    % Only if necessary; here: not needed
    regint = 1:size(A,3); 
    
    A_reg = A_ROI(:,:,regint);
    c_reg = c(regint);   
    
    %% Averaging over complete Region of Interest
    A_mean_xy = squeeze(mean(A_reg,[1 2]));
    A_CI_mean = squeeze(mean(A_CI_ROI,[1 2]));
    A_mean_0  = mean(A_ROI0,'all');
    
    disp('Mean blank value over complete ROI')
    disp(A_mean_0)
    disp('conc vs. mean intensities ROI')
    disp([c_reg,A_mean_xy])
    
    disp('Regression over complete ROI')
    
    mdl     = fitlm(c_reg,A_mean_xy,'Intercept',false);
    b_ROI   = 0;
    m_ROI   = mdl.Coefficients.Estimate(1);
    R2_ROI  = mdl.Rsquared.Ordinary;
    
    % Calculate limits of detection and quantification for mean values
    [LOD,LOQ] = fct_LOD(c_reg,A_mean_xy);  
        
    % CI from dataset, not from regression
    A_minCI = A_mean_xy - A_CI_mean(2:end);
    A_maxCI = A_mean_xy + A_CI_mean(2:end);
    A_minCI(A_minCI<=0)=0.001;
    A_maxCI(A_maxCI<=0)=0.001;
    
    figure
    hold all
    plot(c_reg,A_mean_xy,'xr')
    plot(c_reg,m_ROI*c_reg,'b-')
    fill([c_reg(1:end); flipud(c_reg(1:end))], ...
        [A_minCI(1:end); flipud(A_maxCI(1:end))],'r', ...
            'FaceColor', 'r', 'EdgeColor', 'none', 'FaceAlpha', 0.5);
    xline(LOD,'k','LOD','LabelHorizontalAlignment','left');
    xline(LOQ,'k','LOQ','LabelHorizontalAlignment','right');
    hold off
    
    xlabel('Concentration MP [g/L]')
    ylabel('Illumination Intensity')
    title(sprintf(['ROI Averages: m = %1.3f, R2 = %2.4f, ' ...
        'LOD = %2.2f, LOQ = %2.2f'], ...
        m_ROI, R2_ROI, ...
        LOD, LOQ))
    hold off
    set(gca,'XScale','log','YScale','log')
    %ylim(10.^[-1,3])
    filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-mean_',exp_name,'_',analysis,'_mean');
    print('-r300',filename,'-dpng');
    
    %% Binning und linear regression binwise
    % Pre-allocation
    nvert = size(A_reg,1)/dpix_reg;  % Abstnde vertikal zwischen Analyse
    nhorz = size(A_reg,2)/dpix_reg;  % Abstnde horizontal
    
    b         = zeros(nvert,nhorz);
    m         = zeros(nvert,nhorz);
    R2        = zeros(nvert,nhorz);
    A_ROI0Bin = zeros(nvert,nhorz);
    A_CI_Bin  = zeros(nvert,nhorz,nr_cal);
    sprintf('Linear Regression over %i rows',nvert)
    
    for i=1:nvert
        if ~mod(i,10)
            fprintf('Current row is %d\n',i)
        end
        
        for j = 1:nhorz
            % Pixelbinning: Mean over time; and x and y direction
            if dpix_reg == 1 % just mean over time, no binning
                A_reg_xy = squeeze(A_reg(i,j,:)); 
            else
                vert_window = dpix_reg * (i-1) +1: dpix_reg * i; 
                hor_window  = dpix_reg * (j-1) +1: dpix_reg * j;
                A_reg_xy  = squeeze(mean(A_reg(vert_window,hor_window,:),[1 2]));
                
                % Bin and save mean values and CI for later use
                A_ROI0Bin(i,j) = mean(A_ROI0(  vert_window,hor_window),'all');
                A_CI_Bin(i,j,:)  = squeeze(mean(A_CI_ROI(vert_window,hor_window,:),[1 2]));
            end
             
            if CalcIntercept == true
                mdl = fitlm(c_reg,A_reg_xy,'Intercept',true);
                b(i,j) = mdl.Coefficients.Estimate(1);
                m(i,j) = mdl.Coefficients.Estimate(2);
                R2(i,j)= mdl.Rsquared.Ordinary;
            else 
                mdl = fitlm(c_reg,A_reg_xy,'Intercept',false);
                b(i,j) = 0;
                m(i,j) = mdl.Coefficients.Estimate(1);
                R2(i,j)= mdl.Rsquared.Ordinary;
            end                      
        end
    end
        
    %% Smooth calibration results (Gauss filter)
    % Calculate mean from ROI and normalize profile
    mean_m  = mean(m,'all');
    mean_R2 = mean(R2,'all');
    m_rel = m./mean_m;
    
    % Smoothing calibration slope with Gauss filtering
    mfilt   = imgaussfilt(m_rel, sigma_g);
    R2filt  = imgaussfilt(R2,    sigma_g);
    
    %% Plot Kalibrierung
    % (1) Plot m
    figure('name','Plot calibration slope m')
    hold all
    imagesc(mfilt)
    [~,h2] = contour(mfilt,[0.1:.2:1.7]);  
    [~,h3] = contour(mfilt,[0  :.2:1.6]);  
    hold off
    
    set(h2,'LineColor','k');
    set(h3,'LineColor','k', 'LineWidth',2, ...
           'ShowText', 'on', 'LabelSpacing',10000)
    axis equal
    axis([0 size(mfilt,2) 0 size(mfilt,1)])
    %caxis([0 1.6])
    
    cb = colorbar('eastoutside');
    cb.Title.String = '$m/\bar{m}$';
    cb.Title.Interpreter = 'latex'; 
    set(gca,'ColorScale','log','YDir','normal')
    %cb.Position = [0.92, 0.59, 0.02, 0.34];
    
    title(sprintf('$\\bar{m} = %0.3f$',mean_m), ...
     'Interpreter',"latex")

    filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-m_',analysis,'_',exp_name);
    print('-r300',filename,'-dpng');
    % (2) Plot R2
    figure('name','Plot calibration coefficient R2')
    hold all
    imagesc(R2filt)
    [~,h2] = contour(R2filt,[0.90:.02:1]);  
    hold off
    
    set(h2,'LineColor','k','LabelSpacing',10000,'ShowText','on', ...
           'ShowText','on','LineColor','k');
    set(gca,'YDir','normal'); 
    axis equal
    axis([0 size(R2filt,2) 0 size(R2filt,1)])
    
    caxis([0.90 1])
    cb = colorbar('eastoutside');
    cb.Title.String = '$R^2$';
    cb.Title.Interpreter = 'latex'; 
    
    title(sprintf('$\\bar{R^2} = %0.3f$',mean_R2), ...
     'Interpreter','latex')
    filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-R2_',analysis,'_',exp_name);
    print('-r300',filename,'-dpng');
    
    %% Check for fluctuating signal
    % Mean signal FOV will always decrease because water drains out of flume
    figure
    colororder(parula(size(A_fov,2)+1))
    hold all
    plot(A_fov_blank,'r','LineWidth',2,'DisplayName','Blank')
    for img_no = 1:size(A_fov,2)
        displayName = ['c = ', num2str(c(img_no),'%1.1f'), ' g/L,', ...
            'V_{Stl} = ' num2str(V_Stl(img_no),'%1.0f'), ' L'];
        plot(A_fov(:,img_no),'DisplayName',displayName)
    end
    
    hold off
    title('Time dependency signal (mean complete (!) fov)')
    xlabel('Frame No / time')
    ylabel('Mean signal intensity, raw')
    legend('Location','bestoutside')

    %% Save 
    close all
    clear A A_ROI A_ROI0 A_blank A_reg int_x int_y A_CI A_CI_ROI
    filename = strcat(pfad.FISCalRes,strrep(exp_folder,'/','-'),...
        analysis, num2str(dpix_reg),'pix.mat');
    save(filename); 
end

msgbox('Yeay')