% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Join individual FIS calibrations to calculate final relation
clear all; close all; clc
set(0,'DefaultFigureColormap',hot)

% Set experiment date and path
exp_date = '220901';
pfad = fct_pfad(exp_date);

% Set parameters
dpix_reg = 10;  % Spatial averaging over dpix^2, choose correct folder
R2Limit = 0.90; % For elliptical boundary of well-calibrated region

%% Get names of individual calibrations for final calibration relation
pfad.calresults = strcat(pfad.FIS_Matlab,'4_Cal-results/',num2str(dpix_reg),'pix/');
vorhandene_dateien = dir([pfad.calresults, '*.mat']);
dateinamen         = string({vorhandene_dateien.name});
datasets           = numel(dateinamen);
if datasets == 0
    error('Check folder names')
end

% Extract flume data
dataFlume  = dateinamen(contains(dateinamen,'flume'));

% Extract and sort cuboid data (first liquid, then sediment)
dataCuboid = dateinamen(contains(dateinamen,'cuboid'));
dataCuboid = [dataCuboid(~contains(dataCuboid,'sed')), ...
              dataCuboid(contains(dataCuboid,'sed'))];

% Extract calib name
dataCuboidText  = extractAfter(extractBefore(dataCuboid,'-cuboid'),'1_Calibration-');

% Resorted dataset
dataAll   = [dataCuboid, dataFlume];

% Load form image just for visualization of well-calibrated area
formfile = dir([pfad.FIS_Setup,'form_pos*']);
A_form = load([pfad.FIS_Setup,formfile.name],'A');
A_form = A_form.A;

clear dateinamen datasets vorhandene_dateien

%% Read in datasets
for datasetNr = 1 : numel(dataAll) 
    data = load(strcat(pfad.calresults,dataAll(datasetNr)), ...
         'ExpTime','time_mean', 'm_ROI', 'R2_ROI', 'mean_m', 'mean_R2',...
         'c_reg','A_mean_xy', 'dx_Aq2Cub', 'dy_Aq2Cub', 'ROI_Aq2Cub', ...
         'ROI_fov','y_aquarium','x_aquarium',...
         'm','R2', 'LOD','LOQ','mfilt','m_rel','R2filt');

    % Save point data
    t_exp(datasetNr)        = str2double(data.ExpTime);
    seriesLength(datasetNr) = data.time_mean;
    
    % Save regression result from ROI average 
    m_ROIavg(datasetNr)    = data.m_ROI;
    c_reg_i{datasetNr}     = data.c_reg;    
    A_reg_xy_i{datasetNr}  = data.m_ROI * data.c_reg;
    A_mean_xy_i{datasetNr} = data.A_mean_xy;
    R2_ROIavg(datasetNr)   = data.R2_ROI;  
   
    % Save average of pixelwise regression
    m_ROI(datasetNr)   = data.mean_m;
    R2_ROI(datasetNr)  = data.mean_R2;  
    
    % Save ROI translation Aquarium to Cube, with pixel binning
    dx_Aq2Cub  = ceil(data.dx_Aq2Cub/dpix_reg);    
    dy_Aq2Cub  = ceil(data.dy_Aq2Cub/dpix_reg);    
    ROI_Aq2Cub = ceil(data.ROI_Aq2Cub/dpix_reg) - 0.5*[1 1 0 0]; % Pixelcentered   
    
    % Save regression results
    m{datasetNr}  = data.m;
    R2{datasetNr} = data.R2;
    %
    mfilt{ datasetNr} = data.mfilt;
    R2filt{datasetNr} = data.R2filt;
    
    % Save LOD/LOQ's
    LOD(datasetNr) = data.LOD;
    LOQ(datasetNr) = data.LOQ;
end

%% Plotten Cuboids - Vergleich der Calibrations
figure
for datasetNr = 1 : numel(dataCuboid)-1     
        
    %%%%%%%%%%%%%%%% Subplot m %%%%%%%%%%%%%%%%
    subplot(2,numel(dataCuboid)-1,datasetNr)
    
    hold all
    imagesc(mfilt{datasetNr})   
    [~,h1] = contour(mfilt{datasetNr},[(.2:.2:.8), (1.2:.2:1.6)]);  
    [~,h2] = contour(mfilt{datasetNr},[0,1]);     
    rectangle('position',[0 0 size(mfilt{datasetNr},2) size(mfilt{datasetNr},1)], 'edgecolor', [0 0 0])
    hold off
    
    set(h1,'LineColor','k');
    set(h2,'LineColor','k', 'LineWidth',2, ...
           'ShowText', 'on', 'LabelSpacing',150)     
    set(gca,'YDir','normal','XTick',[], 'YTick',[]);
    %set(gca,'XColor','none', 'YColor','none')
    set(gca,'Position', get(gca,'Position') - [0.12 0.08 0 0]);
        
    axis equal
    axis([0 size(mfilt{datasetNr},2) 0 size(mfilt{datasetNr},1)])
    caxis([0.4 1.5])  
    caxis([0.001 1.5])
    set(gca,'ColorScale','log')
    
    % Add title and dataset name
    ti = title(sprintf('$\\bar{m} = %0.3f$',m_ROI(datasetNr)), ...
     'Interpreter',"latex");
    str = sprintf('%s,%1.1f s', dataCuboidText(datasetNr),t_exp(datasetNr));
    text(ti.Position(1),ti.Position(2)+20,str,'FontSize',15, ...
        'Units','data','HorizontalAlignment','center', ...
        'Interpreter','none')
    
    % Add Colorbar
    if datasetNr == numel(dataCuboid)-1
        m_cb = colorbar;
        m_cb.Title.String = '$m/\bar{m}$';
        m_cb.Title.Interpreter = 'latex'; 
        m_cb.Position = [0.86, 0.57, 0.02, 0.20];
        %m_cb.Ticks = [0.6 1 1.4];
        m_cb.Ticks = 10.^(-3:0);
    end
 
    % Save axpos for aquarium diagram
    axpos_m = get(gca,'Position');
   
    
    %%%%%%%%%%%%%%%% Subplot R2 %%%%%%%%%%%%%%%%
    subplot(2,numel(dataCuboid)-1,datasetNr + numel(dataCuboid)-1)
    
    hold all
    imagesc(R2filt{datasetNr})
    [~,h3] = contour(R2filt{datasetNr},[0.90:.02:1]);  
    rectangle('position',[0 0 size(R2filt{datasetNr},2) size(R2filt{datasetNr},1)], 'edgecolor', [0 0 0])
    hold off
    box on
    
    set(h3,'LineColor','k','LabelSpacing',10000, ...
        'ShowText','on','LineColor','k');    
    set(gca,'YDir','normal','XTick',[], 'YTick',[]);
    %set(gca,'XColor','none', 'YColor','none')    
    axis equal
    axis([0 size(R2filt{datasetNr},2) 0 size(R2filt{datasetNr},1)])    
    caxis([R2Limit 1])
    
    title(sprintf('$\\bar{R^2} = %0.3f$',R2_ROI(datasetNr)), ...
     'Interpreter',"latex")
    
    set(gca,'Position',get(gca,'Position')+[-0.12 0.00 0 0])
    
    % Add colorbar
    if datasetNr == numel(dataCuboid)-1
        R2_cb = colorbar;
        R2_cb.Title.String = '$R^2$';
        R2_cb.Title.Interpreter = 'latex'; 
        R2_cb.Position = [0.86, 0.18, 0.02, 0.20];
        R2_cb.TickLabels = compose('%0.2f',R2_cb.Ticks);
    end
    
    % Save axpos for aquarium
    axpos_R2 = get(gca,'Position');
end

set(gcf,'units','centimeters','position',[10,10,30,12])
set(findall(gcf,'-property','FontSize'),'FontSize',11)

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-comparison.png');
print('-r300',filename,'-dpng');

%% Plot ROI mean data and linear regression
colors = jet(length(c_reg_i)-1);
      
figure
hold all
for i = 1 : length(c_reg_i)-1
    p(i) = plot(c_reg_i{i},A_reg_xy_i{i},'-','Color',colors(i,:));
    plot(c_reg_i{i},A_mean_xy_i{i},'x','Color',colors(i,:));
end
hold off

% Add values of m and R2 table-like
str_m  = {'m', num2str( m_ROIavg(1:end-1)','%2.2f')};
str_R2 = {'R2',num2str(R2_ROIavg(1:end-1)','%2.3f')};
text(0.7,0.5,["",dataCuboidText],'Units','normalized', ...
    'VerticalAlignment','top','Interpreter','none');
text(0.95,0.5,str_m,'Units','normalized', ...
    'VerticalAlignment','top');
text(1.10,0.5,str_R2,'Units','normalized', ...
    'VerticalAlignment','top');


%title(sprintf('Linear regression of ROI-mean'))
xlabel('Concentration [g/L]')
ylabel('Signal Gain')
legtext = dataCuboidText;%{1:length(c_reg_i)-1}
%legtext = {'Flume, surface water','Tank, surface water','Flume, pore water'};
leg = legend(p(:),legtext,'Interpreter','none','Location','bestoutside');
leg.ItemTokenSize = [15,4];

set(gca,'YScale','log','XScale','log')
set(gca,'XMinorTick','on', 'YMinorTick','on')
set(gca,'YLim',10.^[-1 3], 'YTick',10.^(-1:3))
set(gca,'XLim',10.^[0 2.3], 'XTick',10.^(1:3))

x0_fig=10;
y0_fig=10;
w_fig=20; % 10
h_fig=14; % 6
set(gcf,'units','centimeters','position',[x0_fig,y0_fig,w_fig,h_fig])
set(findall(gcf,'-property','FontSize'),'FontSize',10)

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-comparison_mean');
print('-r300',filename,'-dpng');

%% Check ROI translation for aquarium analysis
idx_AqCube = find(contains(dataCuboid,'Aq'));

% 1) Calculate mean of all datapoints in aquarium - cuboid
m_results(1)  = mean( m{idx_AqCube},'all');
R2_results(1) = mean(R2{idx_AqCube},'all');

% 2) Calculate mean of ROI translation in aquarium - flume
m_results(2) = mean(m{end}(  dy_Aq2Cub(1):dy_Aq2Cub(2),dx_Aq2Cub(1):dx_Aq2Cub(2)),'all');
R2_results(2) = mean(R2{end}(dy_Aq2Cub(1):dy_Aq2Cub(2),dx_Aq2Cub(1):dx_Aq2Cub(2)),'all');

% 3) Read out saved averaged data from cam calibration itself
m_results(3) = m_ROI(idx_AqCube);
R2_results(3) = R2_ROI(idx_AqCube);

% Check if values are identical
if any(m_results-m_results(1)) 
    error('Calculated m-values do not match. Check ROI translation')
elseif any(R2_results-R2_results(1))
    error('Calculated R2-values do not match. Check ROI translation')
end

%% Aquarium - find elliptical boundary of well calibrated region
% Normalize by ROI average of cuboid
mfilt_aq  = mfilt{end}* m_ROI(end)/m_ROI(idx_AqCube);
R2filt_aq = R2filt{end};

% (1) Define well-illuminated region 
BinaryImage = R2filt_aq > R2Limit;    % create binary Image
se90 = strel('line',20,90);     % line element vertical
se0  = strel('line',20, 0);     % line element horizontal
BinaryImage = imerode(BinaryImage,[se90 se0]); % Erode a bit


% (2a) Find ellipsoid properties with regionprops, retrieve axes
s = regionprops(BinaryImage,{'Centroid','MajorAxisLength',...
                             'MinoraxisLength','Orientation'});
for i = 1 : length(s)
    a(i) = s(i).MajorAxisLength;
end

% (2b) Retrieve details for the biggest ellipse
[~,idx] = max(a);
a = s(idx).MajorAxisLength/2;
b = s(idx).MinorAxisLength/2;
Xc = s(idx).Centroid(1);
Yc = s(idx).Centroid(2);
phi = deg2rad(s(idx).Orientation); 


% (3) Create ellipsoid and display position
t = linspace(0,2*pi,100);
x_ellipse = Xc + a*cos(t)*cos(phi) + b*sin(t)*sin(phi);
y_ellipse = Yc - a*cos(t)*sin(phi) + b*sin(t)*cos(phi);

figure
hold all
imagesc(R2filt_aq)
plot(x_ellipse,y_ellipse,'LineWidth',5)
hold off
%
caxis([R2Limit 1])
colorbar
axis equal
axis([0 size(mfilt_aq,2) 0 size(mfilt_aq,1)])
title(sprintf('Finding R2 = %1.2f ellipsoid',R2Limit))


% (4) Find all pixels inside ellipsoid
[xgrid, ygrid] = meshgrid(1:size(mfilt_aq,2), 1:size(mfilt_aq,1)); % Define grids    
xgrid = xgrid - Xc;    % Subtract Centers
ygrid = ygrid - Yc;
inEllipse_ROI = ((xgrid*cos(phi)-ygrid*sin(phi)).^2/a^2 + (xgrid*sin(phi)+ygrid*cos(phi)).^2/b^2)<=1;


% (5) Show elliptical mask
figure
hold on
imagesc(inEllipse_ROI)
plot(x_ellipse,y_ellipse,'LineWidth',5)
hold off
set(gca,'YDir','normal')
axis equal
axis([0 size(mfilt_aq,2) 0 size(mfilt_aq,1)])
title('Retrieving Image Area inside of boundary ellipsoid')


% (6) Round elliptical boundary to integers, outer bnd = ROI
% (6a) Round boundary values to integer indices
x_boundary = round(x_ellipse,0);
y_boundary = round(y_ellipse,0);

% (6b) If boundary exceeds image, take outer ROI border
y_boundary(y_boundary>size(mfilt_aq,1)) = size(mfilt_aq,1);
x_boundary(x_boundary>size(mfilt_aq,2)) = size(mfilt_aq,2);
y_boundary(y_boundary<1) = 1;
x_boundary(x_boundary<1) = 1;


% (7) Additional info: slope value at border
m_edge  = mean(diag(mfilt_aq(y_boundary,x_boundary)));
R2_edge = mean(diag(R2filt_aq(y_boundary,x_boundary)));


% (8) Save Data for final relation and display
mfilt_aq(~inEllipse_ROI) = NaN;
R2filt_aq(~inEllipse_ROI) = NaN;

%
figure
imagesc(mfilt_aq)
colorbar
set(gca,'ColorScale','log','YDir','normal')
axis equal
axis([0 size(mfilt_aq,2) 0 size(mfilt_aq,1)])
title('Slope distribution')
%
figure
hold all
imagesc(mfilt_aq)
plot(x_boundary,y_boundary,'LineWidth',5)
hold off

colorbar
set(gca,'ColorScale','log','YDir','normal')
axis equal
axis([0 size(mfilt_aq,2) 0 size(mfilt_aq,1)])
title('Slope distribution with NaN outside of ellipse')

%% Graphic: Calibration large FOV (aquarium), showing elliptical boundaries (m and R2)
fig_aq = figure;

%%%%%%%%%%%%%%%% Subplot m %%%%%%%%%%%%%%%%
subplot(1,2,1)
hold all
imagesc(mfilt_aq)
[~,h1] = contour(mfilt_aq,[0.2: 0.4: 1.6]);  
[~,h2] = contour(mfilt_aq,[0.4: 0.4: 1.6]);  
rectangle('Position', ROI_Aq2Cub,'EdgeColor','c','LineWidth',2)
plot(x_boundary,y_boundary,'g','Linewidth',2)
hold off
    
set(h1,'LineColor','k');
set(h2,'LineColor','k', 'LineWidth',2, ...
       'ShowText', 'on', 'LabelSpacing',10000)
   
set(gca,'YDir','normal');     
set(gca,'XTick',[], 'YTick',[]);
set(gca,'XColor','none', 'YColor','none')
axpos = get(gca,'Position') + [0.0 0.10 0 0];
set(gca,'Position',axpos);
axis equal
axis([0 size(mfilt_aq,2) 0 size(mfilt_aq,1)])

caxis([0.001 1.5])
cb = colorbar('southoutside');
cb.Title.String = '$m/\bar{m}_{ROI}$';
cb.Title.Interpreter = 'latex'; 
cb.Position = [0.13 0.20 0.33 0.05];
cb.Ticks = 10.^(-3:0);
set(gca,'ColorScale','log')
box off

title(sprintf(['$\\bar{m}_{ROI} = %0.3f,\\bar{m}_{all} = %0.3f$'], ...
                     m_ROI(idx_AqCube), m_ROI(end)), 'Interpreter',"latex");
                 

%%%%%%%%%%%%%%%% Subplot R2 %%%%%%%%%%%%%%%%
subplot(1,2,2)
hold all
imagesc(R2filt_aq)
%[C2,h2] = contour(R2filt,[0.95:.01:1]);  
rectangle('Position', ROI_Aq2Cub,'EdgeColor','c','LineWidth',2);
plot(x_boundary,y_boundary,'g','Linewidth',2)
hold off
box off

% set(h2,'LineColor','k','LabelSpacing',10000,...
  %     'ShowText','on','LineColor','k');
set(gca,'YDir','normal'); 
set(gca,'XTick',[], 'YTick',[]);
set(gca,'XColor','none', 'YColor','none')
axpos = get(gca,'Position') + [0.0 0.10 0 0];
set(gca,'Position',axpos);
axis equal
axis([0 size(R2filt_aq,2) 0 size(R2filt_aq,1)])

caxis([R2Limit 1])
cb2 = colorbar('southoutside');
cb2.Title.String = '$R^2$';
cb2.Title.Interpreter = 'latex'; 
cb2.Position = [0.57 0.20 0.33 0.05];
box off

title(sprintf(['$\\bar{R^2}_{ROI} = %0.3f, ' ...
                '\\bar{R^2}_{all} = %0.3f$'], ...
                 R2_ROI(idx_AqCube), R2_ROI(end)), ...
                    'Interpreter',"latex");

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-Aquarium.png');
print('-r300',filename,'-dpng');

%% Final relation MP to light intensity in flume
%%%%%%%%%%%%%% Idea for relation %%%%%%%%%%%%
% (1) MP distribution is taken from normalized profile of aquarium 
%     --> large field of view, but wrong scaling (double glass, less depth)
% (2) Distribution is scaled with average slope in flume (liq or sed)

m_liq = m_ROI(contains(dataAll,'liq','IgnoreCase',true));
m_sed = m_ROI(contains(dataAll,'sed','IgnoreCase',true));
m_aq  = m_ROI(contains(dataAll,'aq', 'IgnoreCase',true));

% Distribution
m_MP_liq     = m_liq * mfilt_aq;
m_MP_sed     = m_sed * mfilt_aq;

% Inverse relation, from known concentration to known intensity
m_Int2Conc_liq = 1./m_MP_liq;
m_Int2Conc_sed = 1./m_MP_sed;


figure
subplot(1,2,1)
imagesc(m_Int2Conc_liq)
cb = colorbar;
cb.Ticks = 10.^(-1:1);
title('Final Relation Int2Conc, liq')
set(gca,'YDir','normal','ColorScale','log')
axis equal
axis([0 size(m_MP_liq,2) 0 size(m_MP_liq,1)])

subplot(1,2,2)
imagesc(m_Int2Conc_sed)
cb = colorbar;
cb.Ticks = 10.^(-1:4);
title('Final Relation Int2Conc, sed')
set(gca,'YDir','normal','ColorScale','log')
axis equal
axis([0 size(m_MP_liq,2) 0 size(m_MP_liq,1)])

set(gcf,'units','centimeters','position',[10,10,30,8])
set(findall(gcf,'-property','FontSize'),'FontSize',11)

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-final_relation');
print('-r300',filename,'-dpng');

%% Convert to complete "global" coordsys of camera
% To simplify later use, put calibrated region into whole FOV
% Flagging is NaN for bad signal (R2<R2Limit) and 0 for non-calibrated region
m_Cam_liq = zeros(data.ROI_fov(4)/dpix_reg,data.ROI_fov(3)/dpix_reg); 
m_Cam_sed = zeros(size(m_Cam_liq));
inEllipse = zeros(size(m_Cam_liq));

y_ROI_min = ceil(data.y_aquarium(1)/dpix_reg);
y_ROI_max = ceil(data.y_aquarium(2)/dpix_reg);
x_ROI_min = ceil(data.x_aquarium(1)/dpix_reg);
x_ROI_max = ceil(data.x_aquarium(2)/dpix_reg);

m_Cam_liq(y_ROI_min:y_ROI_max, x_ROI_min:x_ROI_max) = m_Int2Conc_liq;
m_Cam_sed(y_ROI_min:y_ROI_max, x_ROI_min:x_ROI_max) = m_Int2Conc_sed;
inEllipse(y_ROI_min:y_ROI_max, x_ROI_min:x_ROI_max) = inEllipse_ROI;

x_boundaryAll = dpix_reg * (x_boundary + x_ROI_min);
y_boundaryAll = dpix_reg * (y_boundary + y_ROI_min);

% Overlay Setup image (bedform) and well-calibrated region              
figure
hold all
imagesc(A_form)
setDiag
plot(x_boundaryAll,y_boundaryAll,'g-','LineWidth',2)
hold off
title('Setup image with well-calibrated region')

filename = strcat(pfad.graphics_FIS,exp_date,'-FIS-Cal-final_region');
print('-r300',filename,'-dpng');

%% Save
close all
clear data A A_ROI A_reg A_blank A_ROI0 R2 R2filt m mfilt m_rel A_form

filename = strcat(pfad.results,exp_date, '-FIS_Cal_Relation.mat');
save(filename);
disp(['Saved as ',filename])

msgbox('Finished','Yeay')