% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

clear all; close all; clc

% Define Calibration filenames
exp_date     = '220901'; % Date of Experiment = main folder/excel sheet!

% Signal Fluorometer: 1 = uranine, 2 = rhodamine b, 3 = tinopal
tracerNr = 1;     

% Define paths to datasets and labbook
pfad = fct_pfad(exp_date);
labbook = '../labbook.xlsx';

% Linear regression: false (y = mx), true: (y = mx + b)
CalcIntercept = false; 

%% Read labbook: Volumes of MP, times for calibration.
messung = readcell(labbook,"Sheet",exp_date);

% Search in 1st column, find data in 2nd column
ident = messung{strcmp(messung,'Ident'),2};
c_Stl = messung{strcmp(messung,'Stl conc'),2};  % conc MP-stock solution in mg/L
V_tot = messung{strcmp(messung,'Vol total'),2}; % Flask volume flask in mL
V_lin = messung{strcmp(messung,'Volumen linear'),2};  % End of linear range in calib
cal_date = num2str(messung{strcmp(messung,'Date calibration'),2});

% Fluorometers
% Read used devices
devices(1) = messung{strcmp(messung,'SN Fluo in'),2};
devices(2) = messung{strcmp(messung,'SN Fluo out'),2};

% Indices for Kalibration
i1  = find(strcmp(messung,'Kalibration start'));
i2  = find(strcmp(messung,'Kalibration end'));

if isempty(i1)
    error('Check Calibration indices')
end

% Read out Volumina and time values
V_Stl = cell2mat(messung(i1+2 : i2-1, 1)); % Zugabe Stammlsung in L
t0    = cell2mat(messung(i1+2 : i2-1, 2));
t1    = cell2mat(messung(i1+2 : i2-1, 3));

c     = c_Stl / V_tot * V_Stl; % g/L Standardlsung fr Kal
c_lin = c_Stl / V_tot * V_lin; % g/L End of linear range

% Errors if time is not sorted
time_error = find((t1-t0)<0)+i1;
if issorted(t0) == 0
    error('Begin Time array is not sorted, please check')
elseif issorted(t1) == 0
    error('End Time array is not sorted, please check')
elseif ~isempty(time_error)
    error('End Time before Begin Time in row %i', time_error)
end

% Umrechnen in Datetime
t0 = datetime(t0,'ConvertFrom','datenum');
t1 = datetime(t1,'ConvertFrom','datenum');
cal_datetime = datetime(cal_date,'Format','yyMMdd');
for i = 1 : length(t0)
    t0(i).Day    = cal_datetime.Day;
    t0(i).Month  = cal_datetime.Month;
    t0(i).Year   = cal_datetime.Year;
    
    t1(i).Day    = cal_datetime.Day;
    t1(i).Month  = cal_datetime.Month;
    t1(i).Year   = cal_datetime.Year; 
end

%% Read in all data
cal_datefile = cal_date; % Usually the same, but may be different

for j = devices
    device = sprintf('Fluo%i',j)
    readPath   = strcat(pfad.Fluo,cal_datefile,'-f',num2str(j),'.mv');
    try
        messwerte  = readmatrix(readPath,"FileType","text","OutputType","string");
    catch ME
        error('File not found. Is calibration stored in %s',cal_datefile)       
    end
    % infos in csv: (2) timestamp, 
    % (4) tracer1, (5) tracer2, (6) tracer3, (10) Temperature
    timestamps =   datetime(messwerte(:,2),'InputFormat', 'dd/MM/yy-HH:mm:ss');
    signal     = str2double(messwerte(:,tracerNr + 3)); 

    fluoData.(device).('signal_time') = timestamps;
    fluoData.(device).('signal_values') = signal;    
    
    clear signal timestamps messwerte
end

%% Analyse each calibration point
for j = devices
    device = sprintf('Fluo%i',j)
    timestamps = fluoData.(device).signal_time;
    signal     = fluoData.(device).signal_values;
    
    for i = 1 : length(t0)
        %%%% Find start and endpoints for each interval
        n0(i) = find(timestamps>=t0(i),1);
        n1(i) = find(timestamps>=t1(i),1);
        
        % Remove outliers from data
        [data, outliers] = rmoutliers(signal(n0(i):n1(i)));
        qty_data(i) = length(outliers);
        qty_data_invalid(i) = nnz(outliers);
        
        % Deviation from mean
        mean_value(i,1) = mean(data);
        dev_tim{i} = (data - mean_value(i,1)) / mean_value(i,1) * 100;
                
        %%%% 95 % Confidence Interval
        N = length(data);
        ySEM = std(data)/sqrt(N);   % Standard Error Of The Mean
        CI95 = tinv(0.975, N-1);    % 95% Probability Intervals
        yCI95(i,1) = ySEM * CI95;   % Confidence Intervals 
    end

    % Showing raw data for calibration analysis
    n_vec = [];
    for i = 1 : length(n0)
        n_vec = [n_vec, n0(i):n1(i)];
    end
    
    figure;
    plot(timestamps(n_vec),signal(n_vec),'x')
    xlabel('Measurement time')
    ylabel('Signal value')
    title(['Raw data: Calibration ',device])
    set(gca,'YScale','log')
    
    
    % Showing signal deviation from mean in calibration
    fig = figure; 
    plotcolor = jet(length(t0));
    hold all
    for i = 1:length(t0)-1
        plot(dev_tim{i},'-x','Color',plotcolor(i,:));
        legendStr{i} = sprintf('%2.1f,\t %i/%i',c(i),qty_data_invalid(i),qty_data(i));  
    end
    hold off
    
    xlabel('Measurement No')
    ylabel('Deviation in %')
    title(['signal deviation from mean - ', device]);
    lgd = legend(legendStr, 'Location','eastoutside');
    title(lgd,'c [g/L], Outliers');
    
    % Save to structure
    fluoData.(device).('yBlank') = mean_value(1);
    fluoData.(device).('yMean')  = mean_value(2:end) - mean_value(1); % Blank-subtracted
    fluoData.(device).('yCI')    = yCI95(2:end);     
    fluoData.(device).('yMaxCI') = fluoData.(device).yMean + fluoData.(device).yCI;
    fluoData.(device).('yMinCI') = fluoData.(device).yMean - fluoData.(device).yCI;
end

%% Linear Regression Analysis
c(c==0)=[];
cmin = min(c);
cmax = max(c); 
colors = {'r','b', 'g', 'c'};

figure('Units','centimeters','Position',[10,10,16,5])
hold all

for i = 1:length(devices)
    device = sprintf('Fluo%i',devices(i));

    % Three models: all values with intercept (0,0) and 
    % split into two halfs with (high/low) fixpoint (c(fix), s(fix))
    idx_fix = find(c==c_lin);
    
    x0 = c(1:end);
    x1 = c(1:idx_fix);
    x2 = c(idx_fix:end);
    
    y0 = fluoData.(device).yMean(1:end);
    y1 = fluoData.(device).yMean(1:idx_fix);
    y2 = fluoData.(device).yMean(idx_fix:end);
    
    c_fix = x0(idx_fix);
    y_fix = y0(idx_fix);
    
    % Calculate limits of detection and quantification
    [LOD,LOQ] = fct_LOD(x1,y1);
    
    c_predict = x0;%[cmin;(ceil(cmin):1:x0(end))'];
    
    switch CalcIntercept
        case true
            mdl1 = fitlm(x1,y1,'Intercept',true);
            b1   = mdl1.Coefficients.Estimate(1);
            m1   = mdl1.Coefficients.Estimate(2);
            R1   = mdl1.Rsquared.Ordinary;
        case false    
            mdl0 = fitlm(x0,y0,'Intercept',false);
            m0 = mdl0.Coefficients.Estimate(1);
            b0 = 0;
            R0 = mdl0.Rsquared.Ordinary;
            [~,yci0] = predict(mdl0, c_predict);
            
            
            mdl1 = fitlm(x1-c_fix,y1-y_fix,'Intercept',false);
            m1 = mdl1.Coefficients.Estimate(1);
            b1 = 0;
            R1 = mdl1.Rsquared.Ordinary;
            [~,yci1] = predict(mdl1, c_predict-c_fix);
            yci1 = yci1 + y_fix;
            yci1(idx_fix+1:end,:)=0;
            
            mdl2 = fitlm(x2-c_fix,y2-y_fix,'Intercept',false);
            m2 = mdl2.Coefficients.Estimate(1);
            b2 = 0;
            R2 = mdl2.Rsquared.Ordinary;
            [~,yci2] = predict(mdl2, c_predict-c_fix);
            yci2 = yci2 + y_fix;
            yci2(1:idx_fix,:)=0;
    end

    ylin0  = m0 * (x0        ) + b0;
    ylin1  = m1 * (x1 - c_fix) + b1 + y_fix;
    ylin2  = m2 * (x2 - c_fix) + b2 + y_fix;
    

    ylin12 = [ylin1; ylin2(2:end)];
    yci12  = yci1 + yci2;
    
    yci0  = yci0(:,2)  - ylin0;
    yci12 = yci12(:,2) - ylin12;
    
    fluoData.(device).('m0') = m0;
    fluoData.(device).('b0') = b0;
    fluoData.(device).('R0') = R0;
    fluoData.(device).('x0') = x0;
    fluoData.(device).('ylin0') = ylin0;
    fluoData.(device).('yCIReg0') = yci0;
    
    fluoData.(device).('cfix') = c_fix;
    fluoData.(device).('yfix') = y_fix;
    
    fluoData.(device).('m1')  = m1;
    fluoData.(device).('b1')  = -m1 * c_fix + b1 + y_fix;
    fluoData.(device).('R1')  = R1;
    fluoData.(device).('x1')  = x1;
    fluoData.(device).('ylin1')  = ylin1;
    fluoData.(device).('yCIReg1') = yci1;
    
    fluoData.(device).('m2')  = m2;
    fluoData.(device).('b2')  = -m2 * c_fix + b2 + y_fix;
    fluoData.(device).('R2')  = R2;
    fluoData.(device).('x2')  = x2;
    fluoData.(device).('ylin2')  = ylin2;
    fluoData.(device).('yCIReg2') = yci2;

    fluoData.(device).('ylin12')  = ylin12;
    fluoData.(device).('yCIReg12') = yci12;
    
    fluoData.(device).('LOD') = LOD;
    fluoData.(device).('LOQ') = LOQ;
  
    % Only draw positive CI -> if negative, completely ignored in log scale
    idx = find(fluoData.(device).yMinCI > 0);
    
    % Plot results    
    p(i) = plot(c,ylin12, '--',...
        'Color',colors{i},'LineWidth',1,'DisplayName',['Part ',device]);
    p0(i) = plot(c,ylin0, '-',...
        'Color',colors{i},'LineWidth',1,'DisplayName',['All  ',device]);
    plot(c,fluoData.(device).yMean, ...
        'Color',colors{i},'marker','x', 'LineStyle','none');
    
    fill([c_predict; flipud(c_predict)], ...
        [ylin12-yci12; flipud(ylin12+yci12)],colors{i}, ...
        'FaceColor', colors{i}, ...
        'EdgeColor', 'none', 'FaceAlpha', 0.5,'DisplayName','newCI');
        
    scatter(c_fix,y_fix,colors{i+2},'filled','o')
end
hold off

%title(sprintf(['Calibration Fluorometer with 95%%CI and regression, %s'],cal_date));
xlabel('Concentration [g/L]')
ylabel('Blank-subtr signal [mV]')
leg = legend(reshape([p0;p],1,[]),'Location','eastoutside');
leg.ItemTokenSize = [20,2];
ylim(10.^[-2,3])
yticks(10.^(-2:3))
set(gca,'XScale','log','YScale','log')
set(findall(gcf,'-property','FontSize'),'FontSize',11) % 11
fullfilename = strcat(pfad.graphics,exp_date,'-Fluo_Calibration.png');
print('-r300',fullfilename,'-dpng')

%% Save and End
close all
% Save in matlab dataformat
savefilename = strcat(pfad.results,exp_date,'-Fluo_Calibration.mat');
save(savefilename)

% Save calibration range in csv format
fullfilename = strcat(pfad.results,exp_date,'-Fluo_CalibrationRange.txt');
fileID = fopen(fullfilename,'w');
fprintf(fileID,'Exp-ID %6s - Calibration Range Fluorometers \n',exp_date);
fprintf(fileID,'c = concentration [g/L], s = signal [mV]\n');
fprintf(fileID,'----------------------------------------------\n');

for i = 1:length(devices)
    device = sprintf('Fluo%i',devices(i));
    export_xy = fct_export( ...
        fluoData.(device).x0, ...
        fluoData.(device).yMean, ...
        fluoData.(device).yCI, ...
        fluoData.(device).ylin0, ...
        fluoData.(device).yCIReg0, ...
        fluoData.(device).ylin12, ...
        fluoData.(device).yCIReg12 );
    fprintf(fileID,'\n%s\n',device);
    fprintf(fileID,['linear regression all data:\n' ...
        'm0 = %.4f,\tb0 = %.4f,\tR0 = %.4f\n\n'], ...
        fluoData.(device).m0, fluoData.(device).b0, fluoData.(device).R0);
    fprintf(fileID,['linear regression partial data (fix at c = %.4f g/L):\n' ...
        'm1 = %.4f,\tb1 = %.4f,\tR1 = %.4f\n', ...
        'm2 = %.4f,\tb2 = %.4f,\tR2 = %.4f\n\n'], c_fix,...
        fluoData.(device).m1, fluoData.(device).b1, fluoData.(device).R1, ...
        fluoData.(device).m2, fluoData.(device).b2, fluoData.(device).R2);
    fprintf(fileID,'c\ts_Mean\ts_CI95\ts_lin0\ts_lin0CI\ts_lin12\ts_lin12CI\n');
    fprintf(fileID,'%4.4f\t%4.4f\t%4.4f\t%4.4f\t%4.4f\t%4.4f\t%4.4f\n',export_xy);
    fprintf(fileID,'----------------------------------------------\n');
end
fclose(fileID);

msgbox('Finished')