function [c1Fit,S] = fit_ade(t,c,freeLowerBnd,limits)
% Fit advection-diffusion-equation to dataset
% Input: time, concentrations, limits
% If no limits given, take complete range

% Test dataset
%{
load('datasetADE_dt.mat','t_Cam','centersIn_len','n_vec')
t = t_Cam;
c = centersIn_len;
limits = [1,5000];
%}
if ~exist('freeLowerBnd','var')
    freeLowerBnd = false;
end

if ~exist('limits','var') || isempty(limits)
    limits = [1, size(t,1)];
end

t_lim = t(limits);

% Save original values
t0 = t;
c0 = c;

% Create new variables: time-shifted complete dataset
t1 = t;
c1 = c;

% Create new variables: time-shifted and cut dataset for fit
c2 = c(limits(1):limits(2));
t2 = t(limits(1):limits(2));

%% Rescaling
% (1) concentration fractions  
csum = sum(c2);
cr2 = c2/csum;
tau_s = sum(t2.*cr2); 

% (2a) Time-shift to start of analysis and rescaling by tau_s
dt = t2(1);    
[~,idx] = min(abs(t2));
%dt = t2(idx)
t1 = (t1-dt)/tau_s;
t2 = (t2-dt)/tau_s;

% (2b) Ensure all values >0 for fit -> eliminate first datapoint
t2(1)  = [];
c2(1)  = [];
cr2(1) = [];

%% Fitting 
if ~freeLowerBnd % Fixed boundaries, use anon fun (one coeff less to fit)   
  
    % (1) Set up fit function
    fun = @(a,tau,DP,t) ...
           (a* (1./tau ./sqrt( 4*pi*DP.*(t)./tau) ).*...
            exp(- ((1-(t)./tau).^2) ./ (4*DP*(t)./tau) ));
    ft = fittype( fun,'Independent','t','Dependent','cr');    
    %coeffnames(ft)% --> a; tau; DP

    % (2a) Set up options
    opts = fitoptions( 'Method', 'NonLinearLeastSquares' );
    opts.Robust = 'Bisquare';

    % (2b) Find first guess for coefficients
    tau0 = 1; % Since data was normalized to tau_s
    DP0 = 1;
    a0 = max(cr2)/(1/sqrt(4*pi)); % max of function, tau = 1, t'=0

    % (2c) Set bnds:  a; tau; DP (coeffnames(ft)
    opts.Lower      = [0 0 0]; % all coeffs positive 
    opts.StartPoint = [a0, tau0, DP0]; 

    % (3) Calculate fit and retrieve fitted values
    [fitresult, gof] = fit(t2,cr2, ft, opts );

    
else % Flexible start point, fun in separate mfile (NaN problem)
    
    % (1) Set up fit function
    ft = fittype( 'fit_ade_eq(t,tp,a,tau,DP)','Independent','t');
    %coeffnames(ft) %--> DP; a; tau; tp

    % (2a) Set up options
    opts = fitoptions( 'Method', 'NonLinearLeastSquares' );
    opts.Robust = 'Bisquare';

    % (2b) Find first guess for coefficients
    tau0 = 1; % Since data was normalized to tau_s
    DP0 = 1;
    a0 = max(cr2)/(1/sqrt(4*pi)); % max of function, tau = 1, t'=0
    tp0 = 0;

    % (2c) Set bnds:   DP; a; tau; tp (coeffnames(ft))
    opts.Lower      = 10^-4.*[1 1 1 1]; % all coeffs positive 
    opts.StartPoint = [DP0,a0, tau0,tp0]; 

    % (3) Calculate fit and retrieve fitted values
    [fitresult, gof] = fit(t2,cr2, ft, opts );
    
    % (4) Cut start to calculated point (cut off 0s)
    [~,idx] = min(abs(fitresult.tp-t2));
    t2 = t2(idx:end);
    cr2 = cr2(idx:end);
    c2 = c2(idx:end);
end

%% Postprocessing
% (1) Retrieve fitted values
cr1Fit = feval(fitresult,t1);
cr2Fit = feval(fitresult,t2);

% (2) Rescale concentrations, set to 0 if nan (at negative time)
c2Fit = csum * cr2Fit;
c1Fit = csum * cr1Fit;
c1Fit(isnan(c1Fit)) = 0;

% (3) Rescale time
t1 = dt + tau_s*t1;
t2 = dt + tau_s*t2;
if freeLowerBnd
    t_limNEW = dt + tau_s*fitresult.tp;
end

%% Plot result
fig = figure;
hold all
p(1) = plot(t0,c0,'x','LineStyle','none','Color',[1 .5 0],'DisplayName','Data all');
p(2) = plot(t2,c2,'rx','DisplayName','Data fit');
p(3) = plot(t2,c2Fit,'b','DisplayName','ADE fit');
p(4) = plot(t1,c1Fit,'b--','DisplayName','ADE fit+epol');
p(5) = plot(t1,c0-c1Fit,'Color',[0 .75 0],'DisplayName','ADE-adjusted data');
%
if freeLowerBnd
         xline(t_lim(1),'k','LineWidth',0.75); 
    xl = xline(t_limNEW,'m','LineWidth',2);
else
    xl = xline(t_lim(1),'m','LineWidth',2); 
end
xl.Label = {'Fit Bound In',num2str(xl.Value,'%2.1f')};
xl.LabelOrientation = 'horizontal';
xl.LabelHorizontalAlignment = 'left';

xl = xline(t_lim(2),'m','LineWidth',2);
xl.Label = {'Fit Bound Out',num2str(xl.Value,'%2.1f')};
xl.LabelOrientation = 'horizontal';
xl.LabelHorizontalAlignment = 'right';

%hold off

xlabel('Time [s]')
ylabel('Concentration [g/L]')
%xlim([0, 1000])
ylim([0,  max(c0)])
title(sprintf(['ADE-Fit: R^2 = %0.2f, ', ...
               'a = %2.2e, tau = %2.2e, DP = %2.2e'], ...
                gof.rsquare,fitresult.a, fitresult.tau, fitresult.DP))
legend(p,'Location','best')

% Return data
S.tau_s = tau_s;
S.fig = fig;
S.fitresult = fitresult;
S.gof = gof;
S.limits = limits;


end