% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Analyse and join PIV for different illumination positions
clear all; close all; clc

% Choose experiment date (YYMMDD), set path accordingly
exp_date = '220901'
pfad = fct_pfad(exp_date);

% Choose PIV positions that shall be analysed
folders_all = ["1_upstream", "2_centerstream", "3_downstream"] 
folders_analyse = folders_all%(3)

% Define interpolation for PIV: fine, constant grid
ds = 0.0005; % 0.5 mm
dx = ds;
dy = ds;

for k = 1:length(folders_analyse)  
    % Select current dataset
    piv_chosen = char(folders_analyse(k))
    pfad_dataset = strcat(pfad.PIV, piv_chosen,'/');
    
    vorhandene_ordner  = dir(pfad_dataset)
    foldernames        = string({vorhandene_ordner.name})
    foldernames(contains(foldernames,[".","scale","checker"])) = []
    qty_subdatasets = length(foldernames)
    
    % Load coordsys anchors for transformation to global coordsys
    try
        labbook = readcell('../labbook.xlsx','Sheet',exp_date);
    catch ME
        error('No labbook data found. Check for existing sheet')
    end
    
    x_gl = labbook{strcmp(labbook, piv_chosen),2}/1000 % mm-> m
    y_gl = labbook{strcmp(labbook, piv_chosen),3}/1000 % mm-> m
    
    
    % Colors for boundary detection visualization
    cmap_bnd = flipud(0.5.*[ ...
        0 0 2; 0 0 1; ...
        0 2 0; 0 1 0; ...
        2 0 0; 1 0 0; ...
        0 2 2; 2 0 2; ...
        2 2 0; 1 1 2]);
    
%% Create backgrounds, if necessary

    for i = 1 : 3
        pfad_i = strcat(pfad.PIV, char(folders_all(i)),'/');
        fct_createBGmultiple(pfad_i,1,'min')
    end
    
%% Analyse Datasets

    % Reading in scale image, only for background visualization
    fileScale = dir([pfad_dataset,'*scaleimg*']);
    PIV_imscale = flipud(imread([pfad_dataset,fileScale.name]));
    
    % Reading in mask image from PIV calculation to restrict analysis region
    fileMask = dir(strcat(pfad.PIV,'*mask*.tif'));
    mask_calc = imbinarize(flipud(imread([pfad.PIV,fileMask.name])));
    
    for i = 1 : qty_subdatasets
        pfad_subordner = char(strcat(pfad_dataset,foldernames(i),'/'))
        fileBg    = dir([pfad_subordner, '*.tif' ]); % Background
        filePrm   = dir([pfad_subordner, '*.par' ]); % Parameters
        fileAvg   = dir([pfad_subordner, '*avg*' ]); % mean PIV-results
        fileReyn  = dir([pfad_subordner, '*reyn*']); % mean PIV + Reynolds stresses
        fileImg   = dir([pfad_subordner, '*.bmp' ]); % Individual images
        
        fileData  = dir([pfad_subordner, '*.nc' ]);  % pairwise PIV-results
        fileInd   = strrep(string({fileData.name}'),'.nc','.dat');
        fileQty   = length(fileImg)
        
        %%%%%  Read in parameters and background
        % Read parameters: save length and time scale (scalar and unit)
        % Scale Length: scale_L Pixel / scale_L_unit
        PIV_prm      = readcell([pfad_subordner,filePrm.name],'FileType','text');
        scale_L      = cell2mat(PIV_prm(strcmp(PIV_prm,'View0_PIV_Conv_LengthConversion'),2))
        scale_L_unit = cell2mat(PIV_prm(strcmp(PIV_prm,'View0_PIV_Conv_LengthConversionUnits'),2))
        scale_t      = cell2mat(PIV_prm(strcmp(PIV_prm,'View0_PIV_Conv_PulseDelay'),2))
        scale_t_unit = cell2mat(PIV_prm(strcmp(PIV_prm,'View0_PIV_Conv_PulseDelayUnits'),2))
        scale_v_unit = cell2mat(PIV_prm(strcmp(PIV_prm,'View0_PIV_Conv_VelocityUnits'),2))
        
        % Background: Convert to double for later use, test if identical, show
        PIV_bg16bit = flipud(imread([pfad_subordner,fileBg.name]));
        PIV_bg = cast(PIV_bg16bit,'double');
        if (mean(PIV_bg,'all')-mean(PIV_bg16bit,'all')) ~=0
            error('Check conversion uint16 to double')
        end
        
        % Read in first image
        PIV_img = flipud(imread([pfad_subordner,fileImg(1).name]));
        
        % Display
        figure
        imagesc(PIV_bg)
        setDiag
        title(sprintf('Background, average over %i frames',fileQty))
        
        figure
        imagesc(mask_calc)
        setDiag
        title('Mask for PIV calculation in ILA program')
       
        figure
        imagesc(PIV_img)
        setDiag
        title('Sample Img (First)')
        
        
        % Read in PIV Data
        %{
        (v1) Averages: 
        1-6: "x", "y", "z", "u", "v", "w", 
        7-10: "velmag", "flag", "count", "valid", 
        11-14: "dudx", "dudy", "dvdx", "dvdy"    
        
        (v2) Reynolds: 
        1-5:   "x", "y", "z", "U_bar", "V_bar", 
        6-9:   "uu", "vv", "uv", "TKE", 
        10-11: "count", "valid"
    
        (v3) Individual measurements (for transient study): 
        1-7:   "x", "y", "z", "u", "v", "w", "velmag"
        8-12:  "flag", "count", "valid", "snr", "corr" 
        13-18: "du/dx", "dv/dy", "dv/dx", "du/dy"
        17-19: "vort_z", "shear_z", "nstrain"
        
        
        Additional info
        flag (status): 0=Disabled,  1=NoResult,  2=Outlier, 
                       3=ValidData, 4=OtherPeak, 5=Interpolated
        dataCount: number of valid data at node
        dataValid: dataCount, normalized by qtySamples (0...1)
        %}
        
        
        % Choosing Reynolds file (for TKE)
        filename = strcat(pfad_subordner,fileReyn.name);
        %filename = strcat(pfad_subordner,fileAvg.name);
        %filename = strcat(pfad_subordner,fileInd(101));
        
        % Read out dataset; locs [mm] and vels [mm/s], convert mm into m
        % Data is saved in vector form
        messung  = readmatrix(filename,'Delimiter',' ');
        x_all    =   messung(:,1)/1000;
        y_all    =   messung(:,2)/1000;
        u_all    = - messung(:,4)/1000; % Positive downstream
        v_all    =   messung(:,5)/1000;
        TKE_all  =   messung(:,9)/1000 / 1000; % velocity^2
        
          
        %{
        % Average Data
        status = messung(:,8);
        dataCount = messung(:,9);
        dataValid = messung(:,10);
        %}
                
        %Reynolds Data
        dataCount = messung(:,10);
        dataValid = messung(:,11);
        qtySamples = round(nanmean(dataCount./dataValid,'all'),0)
        
        
        % Find grid, reshape data into matrix form; flip lr for increasing
        % order for interpolation
        size_x = find(y_all==y_all(1),1,'last')
        size_y = size(y_all,1) / size_x
        x         = reshape(x_all,     [size_x, size_y])';
        y         = reshape(y_all,     [size_x, size_y])';
        u         = reshape(u_all,     [size_x, size_y])';
        v         = reshape(v_all,     [size_x, size_y])';
        TKE       = reshape(TKE_all,   [size_x, size_y])';
        %status    = reshape(status,    [size_x  size_y])';
        dataCount = reshape(dataCount, [size_x, size_y])';
        dataValid = reshape(dataValid, [size_x, size_y])';
        
        clear x_all y_all u_all v_all
        
        % Extract vectors for x and y
        x_vec = x(1,:)';
        y_vec = y(:,1);
        
        % Extract increment per measurement point
        del_x = x_vec(2) - x_vec(1);
        del_y = y_vec(2) - y_vec(1);
        
        % Retrieve non-valid flags
        % [bad_row, bad_col] = find(status<3);
        
        
        % Analyse for data validity
        % 1) Image
        figure
        imagesc(x_vec,y_vec,dataValid)
        title('Valid Data in the FOV')
        colorbar
        caxis([0 1])
        set(gca,'YDir','normal')
        %
        % 2) Histogram
        figure
        hold all
        h = histogram(dataValid);
        [~,histoMinBin] = min(h.Values);
        dataValidMin = h.BinEdges(1+histoMinBin)
        xline(dataValidMin,'r','Cut-off @ histo min','LineWidth',2);
        hold off
        title('Histogram: Valid Data rate per gridpoint')
        %
        
        % 3) Flags
        %{
        figure
        imagesc(x_vec,y_vec,status)
        colorbar
        colormap(jet(6))
        caxis([-0.5 5.5])
        set(gca,'YDir','normal')
        title('Flags: 0 Disabled, 1 No Result, 2 Outlier, 3 Valid, 4 OtherPeak, 5 Interp')
        %}
        
        % Set masks to identify data with valid flag and high data Count
        mask_valid   = dataValid > dataValidMin;
        %mask_flags   = status >=0;
        mask_total   = mask_valid;% & mask_flags;
        
        % Extract longest continous boundary -> largest area with good data
        [B,L] = bwboundaries(mask_total,'noholes'); % B is vector, L is B-indexcoded matr
        [~, maxindex] = max(cellfun(@length,B));
        mask_conti = (L == maxindex);
        
        
        figure
        subplot(1,2,1)
        imagesc(mask_total)
        title(sprintf('Mask: data quantity > %.2f and flagging',dataValidMin))
        set(gca,'YDir','normal')
        colormap(gray(2))
        colorbar
    
        subplot(1,2,2)
        imagesc(mask_conti)
        title('Longest continuous area in mask')
        set(gca,'YDir','normal')
        colormap(gray(2))
        colorbar
        
        % If outside mask, set to nan
        u(~mask_conti)   = nan;
        v(~mask_conti)   = nan;
        TKE(~mask_conti) = nan;
        
        % Set nice colormaps (change at +/-0)
        [cmap_u,min_u,max_u]       = fct_colormap(u,[],'min');
        [cmap_v,min_v,max_v]       = fct_colormap(v,[],'min');
        [cmap_TKE,min_TKE,max_TKE] = fct_colormap(TKE,[],'mean');
        
        
        %%%%% Output Graphics: measured datapoints, raw data with masks
        % Horizontal Velocity
        figure 
        subplot(2,2,1)
        imagesc(x_vec,y_vec,u)
        %
        colorbar
        colormap(gca,cmap_u);
        caxis([min_u, max_u])
        set(gca,'YDir','normal')
        title('Measured u[m/s]')
        xlabel('Distance [m]')
        ylabel('Distance [m]')
        axis equal
            
        % Vertical Velocity
        subplot(2,2,2)
        imagesc(x_vec,y_vec,v)
        %
        colorbar
        colormap(gca,cmap_v);
        caxis([min_v, max_v])
        set(gca,'YDir','normal')
        title('Measured v [m/s]')
        xlabel('Distance [m]')
        ylabel('Distance [m]')
        axis equal
        
        % Total kinetic energy
        subplot(2,2,3)
        imagesc(x_vec,y_vec,TKE)
        %
        colorbar
        %colormap(gca,cmap_TKE);
        %caxis(10.^[-4, 0]);
        set(gca,'YDir','normal','ColorScale','log')
        title('Measured TKE [m/s]')
        xlabel('Distance [m]')
        ylabel('Distance [m]')
        axis equal
        %%%%
        %%%% End Graphic
               
        if ~exist('u_i','var')
            u_i = zeros(size(u,1), size(u,2), qty_subdatasets);
        end
        % Save velocity profiles for merging
        u_i(:,:,i) = u;
        v_i(:,:,i) = v;
        TKE_i(:,:,i) = TKE;
        PIV_bg_i(:,:,i) = PIV_bg;
    end

%% Join PIV data from different illumination positions

    % Mean values u/v/TKE: if nan, ignore and take from other dataset
    u_mean   = nanmean(u_i,3);
    v_mean   = nanmean(v_i,3);
    TKE_mean = nanmean(TKE_i,3);              
    
    % Complete background as sum of three individuals; ignore outside mask
    % Intensity cut-off from histo to ignore LED position
    PIV_bg_sum = sum(PIV_bg_i,3);
    PIV_bg_sum = PIV_bg_sum.*mask_calc; 
    
    % Graphics sum of backgrounds  
    figure
    imagesc(PIV_bg_sum)
    setDiag
    set(gca,'ColorScale','log')
    title('Total Background = sum of individual backgrounds')
    
    clear PIV_bg_i u_i v_i TKE_i
%% Interpolate velocities and TKE

    % Interpolate matrices to constant (fine) grid
    x_interp = dx*round(x_vec(1)/dx) :dx: dx*round(x_vec(end)/dx);
    y_interp = dy*round(y_vec(1)/dy) :dy: dy*round(y_vec(end)/dy);
        
    % Generate Interpolant from measurement
    [X,Y] = ndgrid(x_vec,y_vec);
    gridInt_u_mean   = griddedInterpolant(X,Y,u_mean','linear');
    gridInt_v_mean   = griddedInterpolant(X,Y,v_mean','linear');
    gridInt_TKE_mean = griddedInterpolant(X,Y,TKE_mean','linear');
    
    % Generate interpolated data from interpolant
    [X,Y] = ndgrid( x_interp, y_interp );
    u_mean_interp   = gridInt_u_mean(X,Y)';
    v_mean_interp   = gridInt_v_mean(X,Y)';
    TKE_mean_interp = gridInt_TKE_mean(X,Y)';
        

    [cmap_u,min_u,max_u]       = fct_colormap(u_mean_interp,[],'min');
    [cmap_v,min_v,max_v]       = fct_colormap(v_mean_interp,[],'min');
    [cmap_TKE,min_TKE,max_TKE] = fct_colormap(TKE_mean_interp,[],'mean');
        
    
    %%%%% Graphics: Interpolated velocities and TKE
    % Interpolated u
    figure
    subplot(1,3,1)
    hold all
    imagesc(x_interp,y_interp,u_mean_interp)
    hold off
    %
    colorbar;
    colormap(gca,cmap_u)
    caxis([min_u, max_u])
    set(gca,'DataAspectRatio',[1,1,1])
    set(gca,'YDir','normal')
    title('Interpolated u [m/s]')
    xlabel('Distance [m]')
    ylabel('Distance [m]')
    
    % Interpolated v
    subplot(1,3,2)
    imagesc(x_interp,y_interp,v_mean_interp)
    colorbar;
    colormap(gca,cmap_v)
    caxis([min_v, max_v])
    set(gca,'DataAspectRatio',[1,1,1])
    set(gca,'YDir','normal')
    title('Interpolated v [m/s]')
    xlabel('Distance [m]')
    ylabel('Distance [m]')
    
    % Interpolated TKE
    subplot(1,3,3)
    imagesc(x_interp,y_interp,TKE_mean_interp)
    colorbar;
    colormap(gca,cmap_TKE)
    caxis(10.^[-4, 0]);
    set(gca,'DataAspectRatio',[1,1,1])
    set(gca,'ColorScale','log')
    set(gca,'YDir','normal')
    title('Interpolated TKE [m/s]')
    xlabel('Distance [m]')
    ylabel('Distance [m]')

%% Extract Interfaces air-water and water-sediment

    % All values not in water are set to 0
    PIV_bg_int = PIV_bg_sum; % Save background for edges detection
    
    % If interfaces are not correctly identified, change following prms
    threshold_mult = 1; % Multiplication for edge analysis
    strel_radius   = 5; % length of structuring line -> for dilating edges
    histogram_mult = 0.75; % Intensity threshold of background img (suggest: [0-0.9]
    
    % Show histogram of image: Threshold cut-off
    figure
    hold all
    h = histogram(PIV_bg_int(PIV_bg_int>0)); % Ignore 0's for histogram
    histoCenters = h.BinEdges(1:end-1)+0*h.BinWidth;
    [pks, locs] = findpeaks(h.Values,histoCenters,'MinPeakProminence',100)
    locs(locs<100)=0; % Ignore peaks around dark pixels (0 intensity)
    dataValidMin = histogram_mult*min(locs);
    %
    xlinetext = sprintf('Cut-off @ %0.2f*first peak histo',histogram_mult);
    xline(dataValidMin,'r',xlinetext,'LineWidth',2);
    hold off
    title('Histogram: Light intensities in background image')
    PIV_bg_int(PIV_bg_int<dataValidMin) = 0;
    
    figure
    imagesc(PIV_bg_int)
    setDiag
    set(gca,'ColorScale','log')
    
    figure
    set(gcf,'Name','Please check if interfaces are correct: 1 = surface, 2 = sediment')
    
    %%%%%%%%%%%%%%%% Subplot 1 %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Binarize by analysis for sharp edges, dilate a bit
    subplot(2,2,1) 
    % Analyse for sharp edges
    [~,threshold] = edge(PIV_bg_int,'sobel');
    BWs = edge(PIV_bg_int,'sobel',threshold * threshold_mult);
    
    % Dilate a bit
    se90 = strel('line',strel_radius,90);
    se0  = strel('line',strel_radius,0);
    BWsdil = imdilate(BWs,[se90 se0]);
    
    imshow(BWsdil)
    title('Dilated (binary) Gradient Mask')
    %
    ax = gca;
    ax.YDir = 'normal';
    axis equal
    axis([0 2048 0 2048])
    
    
    %%%%%%%%%%%%%%%% Subplot 2 %%%%%%%%%%%%%%%%%%%%%%%%%%
    % Fill holes
    subplot(2,2,2)
    BWdfill = imfill(BWsdil,'holes');
    imshow(BWdfill)
    title('Binary Image with Filled Holes')
    %
    ax = gca;
    ax.YDir = 'normal';
    axis equal
    axis([0 2048 0 2048])
    
    
    % Find boundary perimeter
    % (1) Find all boundaries, L is boundary loc with index of B
    [B,L] = bwboundaries(BWdfill,'noholes'); % B is vector, L is B-indexcoded matrix
    size_B = size(B,1);
    bnd_length = zeros(size_B,1);
    for j = 1 : size_B
        bnd_length(j) = size(B{j},1);
    end
    
    % (2) Take longest boundaries and extract interfaces
    bnd_lengthS = sort(bnd_length,'descend');
    bnd_10 = min(10, length(bnd_lengthS)); % Show up to 10 longest bnds
    interfaces = find(ismember(bnd_length,bnd_lengthS(1:bnd_10)))
    
    interfaces_name = string(1:bnd_10)
    xpix = 1:size(PIV_bg_sum,2);
    ypix = zeros(1,length(xpix));
    
    for i = 1 : length(interfaces)
    
        % (4) Extract highest point of continuous boundary
        for j = xpix
            y_bnd    = find(L(:,j)==interfaces(i), 1, 'last');
            if isempty(y_bnd)
                y_bnd = nan; % If no value at all, set to nan
            end
            ypix(j)=y_bnd;
        end
    
    
        % (5) Exclude nans from boundary locations       
        ypix_li  = find(~isnan(ypix),1,'first');
        ypix_re  = find(~isnan(ypix),1,'last');
        bnd_nan  = [xpix; ypix]';
        bnd_data = bnd_nan(ypix_li:ypix_re,:);
    
    
        % (6) Extrapolate borders if necessary (if nan is set at borders)
        %{
        no_fit = 10; % Number of points used for interpolation
        
        %  Left side
        xepol_li = (1:ypix_li-1);
        xipol_li = (ypix_li : ypix_li + no_fit);
        %
        fit_li   = polyfit(xipol_li, ypix(xipol_li),1);
        yepol_li = round(polyval(fit_li,xepol_li),0);
    
        % Right side
        xint_re  = (ypix_re - no_fit +1 : ypix_re);
        xepol_re = (ypix_re + 1 :2048);
        % 
        fit_re   = polyfit(xint_re, ypix(xint_re),1);
        yepol_re = round(polyval(fit_re,xepol_re),0);
        
        sprintf('Extrapolating %d (%d) points in %s interface',...
                 length(xepol_li)+length(xepol_re), length(xpix), interfaces_name(i))
        bnd_epol(:,:,i) = cat(1,[xepol_li',yepol_li'],bnd_data,[xepol_re',yepol_re']);
        %}
        bnd_epol{i} = bnd_data; % Pix units for imagesc
    end
    
    
    
    %%%%%%%%%%%%%%%% Subplot 3 %%%%%%%%%%%%%%%%%%%%%%%%%%
    subplot(2,2,3)
    hold all
    imagesc(PIV_bg_sum)
    colormap("gray")
    for i = 1 : bnd_10
        plot(bnd_epol{i}(:,1), bnd_epol{i}(:,2), ...
            'Color',cmap_bnd(i,:),'LineWidth',2, ...
            'DisplayName',interfaces_name(i))
    end
    hold off
    
    set(gca,'YDir','normal')
    axis equal
    axis([0 2048 0 2048])
    legend('Location','southeast')
    title('Backgrounds with interfaces')
     
    
    %%%%%%%%%%%%%%%% Subplot 4 %%%%%%%%%%%%%%%%%%%%%%%%%%
    subplot(2,2,4)
    hold all
    imagesc(PIV_imscale)
    for i = 1 : bnd_10
        plot(bnd_epol{i}(:,1), bnd_epol{i}(:,2),'Color',cmap_bnd(i,:),'LineWidth',2,'DisplayName',interfaces_name(i))
    end
    hold off
    %
    colormap('gray')
    set(gca,'YDir','normal')
    axis equal
    axis([0 2048 0 2048])
    title('Scale image with interfaces')

    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Output graphic to check if interfaces correctly assigned    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    repeatassignment = true; % Repeat if not correctly assigned
    
    while repeatassignment
        clear bnd_int1 bnd_int2
        
        figure('units','normalized','outerposition',[0 0 1 1])
        hold all
        imagesc(PIV_bg_sum)
        for i = 1 : bnd_10
            plot(bnd_epol{i}(:,1), bnd_epol{i}(:,2),'Color',cmap_bnd(i,:),'LineWidth',2,'DisplayName',interfaces_name(i))
        end
        hold off
        
        ax = gca;
        ax.YDir = 'normal';
        axis([0 2048 0 2048])
        colormap("gray")
        legend('Location','southeast')
        title('Backgrounds with interfaces')
        set(gcf,'Visible','on')
    
        answer1 = questdlg({'Interfaces correctly assigned?', ...
                           'Int 1 is sed->water, Int 2 is water->air'}, ...
        	'Interfaces','Continue','Reassign Interfaces','Exit: Bad interfaces','Reassign Interfaces');
        % Handle response
        switch answer1
            case 'Continue'
                disp('Continuing')
                bnd_int1 = bnd_epol{1};
                bnd_int2 = bnd_epol{2};
                close(gcf)
            case 'Exit: Bad interfaces'
                return
            case 'Reassign Interfaces'       
                answer_int1 = inputdlg('Enter space-separated interfaces for sed->water:',...
                     'Sediment-Water-Interface', [1 50])
                interfaces_1 = str2num(answer_int1{1})
                
                answer_int2 = inputdlg('Enter space-separated interfaces for water->air:',...
                     'Water-Air-Interface', [1 50])
                interfaces_2 = str2num(answer_int2{1})
                close(gcf)
                % Concatenate and sort
                bnd_int1_notunique = sortrows(vertcat(bnd_epol{interfaces_1}))
                bnd_int2_notunique = sortrows(vertcat(bnd_epol{interfaces_2}))
                
                
                % If x-duplicates, take median
                [bnd_int1(:,1),~,idx1] = unique(bnd_int1_notunique(:,1),'stable');
                [bnd_int2(:,1),~,idx2] = unique(bnd_int2_notunique(:,1),'stable');
                
                bnd_int1(:,2) =accumarray(idx1,bnd_int1_notunique(:,2),[],@median); 
                bnd_int2(:,2) =accumarray(idx2,bnd_int2_notunique(:,2),[],@median); 
        end
        
        % Output of boundaries
        figure
        hold all
        imagesc(PIV_imscale)
        plot(bnd_int2(:,1), bnd_int2(:,2),'g-','LineWidth',2,'DisplayName','Water->Air')
        plot(bnd_int1(:,1), bnd_int1(:,2),'r-','LineWidth',2,'DisplayName','Sed->Water')
        hold off
    
        colormap("gray")
        set(gca,'YDir','normal')
        axis equal
        axis([0 2048 0 2048])
        title('Scale image with defined interfaces')
        legend
        set(gcf,'Visible','on')
        
        answer2 = questdlg({'Interfaces correctly set?', ...
                   'red is sed->water, green is water->air'}, ...
                   'Interfaces','Continue','Reassign Interfaces','Exit','Continue');
        % Handle response
        switch answer2
            case 'Continue'
                disp('Continuing')
                repeatassignment = false;
                close(gcf)
            case 'Reassign Interfaces'
                repeatassignment = true;
                close(gcf)
            case 'Exit'
                close(gcf)
                return
        end
    end
        
    % Konversion Pixel / (Pix/mm) * 1m/1000mm for merging
    bnd_int1m = bnd_int1 /scale_L / 1000;
    bnd_int2m = bnd_int2 /scale_L / 1000;
    
    % Interpolate interfaces to query points from velocities
    y_int1 = interp1(bnd_int1m(:,1),bnd_int1m(:,2),x_interp,'linear');
    y_int2 = interp1(bnd_int2m(:,1),bnd_int2m(:,2),x_interp,'linear');
    
    clear bnd_epol L
    
%% Modify vel-data: Set to NaN if outside boundaries

    u_mean_interp_bnd = u_mean_interp;
    v_mean_interp_bnd = v_mean_interp;
    TKE_mean_interp_bnd = TKE_mean_interp;
    
    for i = 1 : length(x_interp)   
        y_delete = (y_interp < y_int1(i)) | (y_interp > y_int2(i));
        u_mean_interp_bnd(y_delete,i)   = nan;
        v_mean_interp_bnd(y_delete,i)   = nan;
        TKE_mean_interp_bnd(y_delete,i) = nan;
        
        if isnan(y_int1(i)) || isnan(y_int2(i))
            u_mean_interp_bnd(:,i)   = nan;
            v_mean_interp_bnd(:,i)   = nan;
            TKE_mean_interp_bnd(:,i) = nan;
        end
    end
    
    % Original vs cropped: u
    figure
    subplot(1,2,1)
    hold all
    imagesc(x_interp,y_interp,u_mean_interp)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_u)
    caxis([-0.3 0.3])
    title('Original u')
    
    subplot(1,2,2)
    hold all
    imagesc(x_interp,y_interp,u_mean_interp_bnd)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_u)
    caxis([-0.3 0.3])
    title('Cropped u')
    
    
    % Original vs cropped: v
    figure
    subplot(1,2,1)
    hold all
    imagesc(x_interp,y_interp,v_mean_interp)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_v)
    caxis([-0.1 0.1])
    title('Original v')
    
    subplot(1,2,2)
    hold all
    imagesc(x_interp,y_interp,v_mean_interp_bnd)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_v)
    caxis([-0.1 0.1])
    title('Cropped v')
    
    % Original vs cropped: TKE
    figure
    subplot(1,2,1)
    hold all
    imagesc(x_interp,y_interp,TKE_mean_interp)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_TKE)
    caxis(10.^[-4, 0]);
    set(gca,'ColorScale','log')
    title('Original TKE')
    
    subplot(1,2,2)
    hold all
    imagesc(x_interp,y_interp,TKE_mean_interp_bnd)
    plot(x_interp,y_int1,'r',x_interp,y_int2,'r','LineWidth',2)
    hold off
    %
    colorbar
    colormap(gca,cmap_TKE)
    caxis(10.^[-4, 0]);
    set(gca,'ColorScale','log')
    title('Cropped TKE')
%% Graphic: Velocity and TKE distribution

    [cmap_u,min_u,max_u]       = fct_colormap(u_mean_interp_bnd,[],'min');
    [cmap_v,min_v,max_v]       = fct_colormap(v_mean_interp_bnd,[],'min');
    [cmap_TKE,min_TKE,max_TKE] = fct_colormap(TKE_mean_interp_bnd,[],'mean');
 
    % Velocity u
    figure 
    hold all
    imagesc(x_interp,y_interp,u_mean_interp_bnd)
    plot(x_interp,y_int1,'k',x_interp,y_int2,'k','LineWidth',2)
    hold off
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_u)
    caxis([min_u, max_u])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('Interpol. hor. velocity u [m/s]')
    xlabel('Hor. distance [m]')
    ylabel('Vert. distance [m]')
    
    
    % Velocity v
    figure
    hold all
    imagesc(x_interp,y_interp,v_mean_interp_bnd)
    plot(x_interp,y_int1,'k',x_interp,y_int2,'k','LineWidth',2)
    hold off
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_v)
    caxis([min_v, max_v])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('Interpol. vert. velocity u [m/s]')
    xlabel('Hor. distance [m]')
    ylabel('Vert. distance [m]')
    
    
    % Kinetic Energy TKE
    figure
    hold all
    imagesc(x_interp,y_interp,TKE_mean_interp_bnd)
    plot(x_interp,y_int1,'k',x_interp,y_int2,'k','LineWidth',2)
    hold off    
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_TKE)
    caxis([min_TKE, max_TKE])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('Interpol. TKE u [m^2/s^2]')
    xlabel('Hor. distance [m]')
    ylabel('Vert. distance [m]')
    
%% Convert to global coordinates

    % Flipping left to right to have ascending order in x_coordinates.
    % Therefore flipping in all data BUT the rising y_coords (grid)
    
    x_global = fliplr(x_gl - x_interp);
    
    y_global = -y_gl + y_interp;
    y_int1gl = fliplr(-y_gl + y_int1);
    y_int2gl = fliplr(-y_gl + y_int2);
    
    u_global = fliplr(u_mean_interp_bnd);
    v_global = fliplr(v_mean_interp_bnd);
    TKE_global = fliplr(TKE_mean_interp_bnd);
    

    [cmap_u,min_u,max_u]       = fct_colormap(u_global,[],'min');
    [cmap_v,min_v,max_v]       = fct_colormap(v_global,[],'min');
    [cmap_TKE,min_TKE,max_TKE] = fct_colormap(TKE_global,[],'mean');
       
    
    % Figure: u in global coordinates
    figure
    hold all
    imagesc(x_global,y_global,u_global)
    plot(x_global,y_int1gl,'k','LineWidth',2)
    plot(x_global,y_int2gl,'k','LineWidth',2)
    hold off
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_u)
    caxis([min_u, max_u])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('Horizontal velocities in global KOS')
    
    
    % Figure: v in global coordinates
    figure
    hold all
    imagesc(x_global,y_global,v_global)
    plot(x_global,y_int1gl,'k','LineWidth',2)
    plot(x_global,y_int2gl,'k','LineWidth',2)
    hold off    
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_v)
    caxis([min_v, max_v])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('Vertical velocities in global KOS')
   
    
    % Figure: TKE in global coordinates
    figure
    hold all
    imagesc(x_global,y_global,TKE_global)
    plot(x_global,y_int1gl,'k','LineWidth',2)
    plot(x_global,y_int2gl,'k','LineWidth',2)
    hold off
    %
    colorbar('Position',[0.80 0.3 0.05 0.4]);
    colormap(gca,cmap_TKE)
    caxis([min_TKE, max_TKE])
    %
    set(gcf, 'Units', 'centimeters', 'Position', [0,0,6,6]);
    set(gca,'Position',[0.25 0.25 0.5 0.5])
    set(gca,'YDir','normal','Layer','top')  
    set(gca,'DataAspectRatio',[1,1,1])    
    axis equal
    %
    title('TKE in global KOS')
    
%% Clear and save

    clear PIV_bg PIV_bg16bit X Y PIV_bg_i
    close all
    save([pfad_dataset,'pivresults.mat'])
end

msgbox('Script finished successfully')
% EOF