% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Join PIV from different cam positions
clear all; close all; clc

% Choose experiment date (YYMMDD), set path accordingly
exp_date = '220901'
pfad = fct_pfad(exp_date);

% Interpolation to global grid
ds = 10^-3; % m

%% Reading in datasets
piv.up     = load([pfad.PIV, '1_upstream/pivresults.mat']);
piv.center = load([pfad.PIV, '2_centerstream/pivresults.mat']);
piv.down   = load([pfad.PIV, '3_downstream/pivresults.mat']);

location = fieldnames(piv);
nr_data = length(location);
fprintf('Merging PIV results from %i different camera positions\n',nr_data)

% Extracting results and saving in easier form
for i = 1 : nr_data
    x{i}      = piv.(location{i}).x_global;
    
    y{i}      = piv.(location{i}).y_global;
    y_int1{i} = piv.(location{i}).y_int1gl;
    y_int2{i} = piv.(location{i}).y_int2gl;
   
    u{i}      = piv.(location{i}).u_global;
    v{i}      = piv.(location{i}).v_global;
    TKE{i}    = piv.(location{i}).TKE_global;
end

%% Interpolate to global grid and merge individual measurements
% Defining range for interpolation to global grid
xmin = min(cellfun(@min,x));
xmax = max(cellfun(@max,x));
ymin = min(cellfun(@min,y));
ymax = max(cellfun(@max,y));

x_interp = xmin:ds:xmax;
y_interp = ymin:ds:ymax;

% Interpolation of velocities and interface to global grid 
% (nan for extrapolation)
for i = 1 : nr_data
    
    % 2d-Interpol for velocities
    % Create interpolant from old grid and values
    [X,Y] = ndgrid(x{i},y{i});
    gridInt_u_mean = griddedInterpolant(X,Y,u{i}','linear','none');
    gridInt_v_mean = griddedInterpolant(X,Y,v{i}','linear','none');
    gridInt_TKE_mean = griddedInterpolant(X,Y,TKE{i}','linear','none');
    
    % Apply interpolant to new grid
    [X,Y] = ndgrid( x_interp, y_interp );
    u_i(:,:,i)   = gridInt_u_mean(X,Y)';
    v_i(:,:,i)   = gridInt_v_mean(X,Y)';
    TKE_i(:,:,i) = gridInt_TKE_mean(X,Y)';
            
    % 1d-Interpol for interfaces
    y_int1_i(:,:,i) = interp1(x{i}, y_int1{i},x_interp);
    y_int2_i(:,:,i) = interp1(x{i}, y_int2{i},x_interp);
end


% Averaging of velocities and TKE
u_all      = nanmean(u_i,3);
v_all      = nanmean(v_i,3);
TKE_all    = nanmean(TKE_i,3);

u_mag = sqrt(u_all.^2+v_all.^2); % Velocity magnitude

% Some metrics: Mean velocity, mean quick, mean slow, ratio
u_mean = nanmean(u_all,'all')
v_mean = nanmean(v_all,'all');

u_meanQuick = nanmean(u_all(u_all>u_mean),'all')
u_meanSlow = nanmean(u_all(u_all<u_mean),'all')
u_meanRatio = u_meanSlow / u_meanQuick


% Merge interfaces (sed-water and water-air) and display
y_int1_min   = nanmin( y_int1_i,[],3); % Lower interface
y_int1_mean  = nanmean(y_int1_i,3);
y_int1_max   = nanmax( y_int1_i,[],3);

y_int2_min   = nanmin( y_int2_i,[],3); % Upper interface
y_int2_mean  = nanmean(y_int2_i,3);
y_int2_max   = nanmax( y_int2_i,[],3);

figure
hold all
plot(y_int1_max, 'g','DisplayName','max')
plot(y_int1_mean,'b','DisplayName','mean')
plot(y_int1_min, 'r','DisplayName','min')
%
plot(y_int2_max, 'g','DisplayName','max')
plot(y_int2_mean,'b','DisplayName','mean')
plot(y_int2_min, 'r','DisplayName','min')
hold off
%
legend('NumColumns',2,'Location','best')
title('Superposition of individual interfaces')

%% Different plots with quiver superposed
data_i = cat(3, ...
    u_all, v_all, ...
    u_all-u_mean, v_all-v_mean, ...
    u_mag);
data_ctitle = [
    "v_x [m/s]", "v_z [m/s]",  ...
    "\Deltav_x [m/s]", "\Deltav_z [m/s]",  ...
    "abs(u,v) [m/s]"];

data_step = [0.05, 0.01, ...
             0.05, 0.01, ...
             0.05];
data_int = [-0.05, 0.2; -0.02, 0.02; 
            -0.15, 0.1; -0.02, 0.02;
             0,    0.2];

data_type = extractBefore(data_ctitle,' ');
data_unit = extractAfter( data_ctitle,' ');
    
xlbl = 'Longitudinal position [m]';
ylbl = 'Vertical position [m]';

% Quiver: locations for vector arrows
dy = 8;
dx = 30;
x0 = 30;
y0 = 4;

% Sample matrices according to quiver locations
Xq = (X(x0:dx:end, y0:dy:end))';
Yq = (Y(x0:dx:end, y0:dy:end))';
uq = u_all(y0:dy:end, x0:dx:end);
vq = v_all(y0:dy:end, x0:dx:end);
wq = u_mag(y0:dy:end, x0:dx:end);

% Create graphics for each distribution
for i = 1:3
    data = data_i(:,:,i); 

    [data_cmap,data_min,data_max] = fct_colormap(data,data_step(i),'min',data_int(i,:));
    data_mean = nanmean(data,'all');
    
    f = 0.008; % Length of quiver arrows (unscaled!)
    displayRatio = [2,1,1];
    
    % Output figure distribution
    fig = figure;
    hold all
    imagesc(x_interp,y_interp,data)   
    q = quiver(Xq, Yq, uq./wq*f*displayRatio(1), vq./wq*f*displayRatio(2));
    set(q,'Color','k','LineWidth',0.75,'AutoScale','off','ShowArrowHead','on','MaxHeadSize',1e10)
    plot(x_interp,y_int1_mean,'k','LineWidth',2) % sed-water
    plot(x_interp,y_int2_mean,'k','LineWidth',2) % Water-air
    hold off
    %
    c = colorbar;
    c.Limits = [data_min, data_max];
    c.Ticks = data_min:data_step(i):data_max;
    caxis(c.Limits);
    colormap(data_cmap);
    c.Title.String = data_ctitle(i);
    c.Title.Position = [10,120,0];    
    c.Position = [0.82, 0.285, 0.04, 0.58];
    set(gca,'Position',[0.20, 0.20, 0.60, 0.75])
    %
    xlim([3.5, 3.94])
    ylim([0.05, 0.25])
    xticks(3.4:0.1:4)
    yticks(0:0.05:0.25)
    xtickformat('%.2f')
    ytickformat('%.2f')
    xlabel(xlbl)
    ylabel(ylbl)
    %
    %title(sprintf('Mean: %s = %0.3f %s',data_type(i),data_mean, data_unit))
    %
    set(gca,'XDir','reverse','YDir','normal','Layer','top')
    set(gca,'DataAspectRatio',displayRatio)
    set(gcf,'units','centimeters','position',[10,10,7.5,7])
    set(findall(gcf,'-property','FontSize'),'FontSize',11) 
    set(findall(gcf,'-property','fontname'),'FontName','helvetica')
    %
    fullfilename = strcat(pfad.graphics,exp_date,'_PIV_',strrep(data_type(i),'\',''),'.png');
    print('-r300',fullfilename,'-dpng')  
end

%% Export
export_x   = reshape(X,       [],1);
export_y   = reshape(Y,       [],1);
export_u   = reshape(u_all',  [],1);
export_v   = reshape(v_all',  [],1);
export_TKE = reshape(TKE_all',[],1);


export_xyuvTKE = [export_x'; export_y'; export_u'; export_v'; export_TKE'];
export_xy1y2 = [x_interp; y_int1_mean; y_int2_mean];

fileID = fopen([pfad.results,exp_date,'-piv_results.txt'],'w');
fprintf(fileID,'%% Experiment %s\n Results of: PIV\n',exp_date);
fprintf(fileID,['%% x = horizontal positioning, y = vertical positioning\n' ...
    '%% u = horizontal velocity, v = vertical velocity\n' ...
    '%% TKE = total kinetic energy = 0.5*sum(u''u'',v''v'')\n']);
fprintf(fileID,'%5s\t%5s\t%7s\t%7s\t%11s\n','x [m]','y [m]', 'u [m/s]', 'v [m/s]', 'TKE [m/s]');
fprintf(fileID,'%5.3f\t%5.4f\t%5.4f\t%5.4f\t%5.4f\n',export_xyuvTKE);
fclose(fileID);

fileID = fopen([pfad.results,exp_date,'-piv_results_boundaries.txt'],'w');
fprintf(fileID,'%% Experiment %s\n',exp_date);
fprintf(fileID,'%% Results of: PIV - identified boundaries\n');
fprintf(fileID,['%% x = horizontal positioning, y = vertical positioning\n' ...
    '%% int1 = sediment-water-interface, int2 = water-air-interface\n']);
fprintf(fileID,'%s\t%s\t%s\n','x [m]','y_int1 [m]', 'y_int2 [m]');
fprintf(fileID,'%5.3f\t%5.4f\t%5.4f\n',export_xy1y2);
fclose(fileID);

% Save as matlab file
close all
clear piv TKE u v data_i export_xyuvTKE gridInt*
fullfilename = strcat(pfad.results,exp_date,'-pivresults_all.mat');
save(fullfilename)
%EOF