% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Laserprofiler: Analyse bedform profile after experiment by merging individual measurements
clear all; close all; clc

exp_date = '220901'
pfad = fct_pfad(exp_date);

x_limits = [2400, 4900]; % mm for interpolating to uniform grid
z_limits = [100,160];    % mm maximum elevation for bedforms
dx_interp = 1;           % mm grid spacing for interpolating
ROI_xLim = [2650,4750];  % Metrics: ignore first and last bedform (erroneous)

% Find individual laserscan files
vorhandene_dateien = dir([pfad.profiler_total,'*.txt']);
if isempty(vorhandene_dateien)
    error('Data not found. Please check folder structure')
end
dateinamen = string({vorhandene_dateien.name})';
plate_end  = str2double(extractBefore(dateinamen,'.txt')); % Reference location [cm]

% Some colors for nice plots
colors  = lines(length(dateinamen));    % Define anchors
colors2 = jet(  length(dateinamen));    % All other plots

% Read in Labbook for number of profiles per file
labbook       = '../labbook.xlsx';
sheets = sheetnames(labbook);
metadata = readcell(labbook,"Sheet",exp_date);
nr_profiles = metadata{strcmp(metadata,'nr_profiles'),2};

%% Show raw data: horizontal range 
% Range of measurement in longitudinal direction (X Range) for complete
% dataset, consisting of 10 measurements each
figure
hold all
for i = 1 : length(dateinamen)
    rawdata  = readmatrix(strcat(pfad.profiler_total,dateinamen(i)));
    x_loc{i} = rawdata(:,1);
    z_loc{i} = rawdata(:,3);
    len = repmat((1:size(rawdata,1) / nr_profiles)',nr_profiles,1);
    plot(len,x_loc{i},'x', 'Color', colors2(i,:),'DisplayName',dateinamen(i))    
end
hold off
legend
xlabel('Value Nr.')
ylabel('x distance')
title('Horizontal Range of measurement')

%% Improve raw data, histogram
% Excluding missing data ((0,0) tupels = no data detected)
fprintf('Deleting (0,0) tupels')
for i = 1 : length(dateinamen)
    delete_x = find(x_loc{i}==0);
    x_loc{i}(delete_x) = [];
    z_loc{i}(delete_x) = [];
    fprintf('Deleting %i (%i) 0''s at %s\n', ...
        length(delete_x),length(x_loc{i}),dateinamen(i));    
end

% Sorting datasets (if various measurements are in one file, normally: 10)
for i = 1 : length(dateinamen)
    [x_locSorted{i},sortIdx] = sort(x_loc{i});
    z_locSorted{i} = z_loc{i}(sortIdx);
    
    % Take median z values at duplicated x-locations
    [x_locUnique{i},~,idx] = unique(x_locSorted{i},'stable');   % Find unique x values
    z_locUnique{i} = accumarray(idx,z_locSorted{i},[],@median); % Take median z value
end

% Analyse spatial resolution
% (a) Defining histogram edges
histo_edges = (0:0.1:2).';
histo_centers = ( (histo_edges(2)-histo_edges(1))/2 :...
                   histo_edges(2)-histo_edges(1): ...
                   histo_edges(end) ).';
%
% (b) Calculting spatial resolution and histogram
for i = 1 : length(dateinamen)
    delta_x{i}       = - (x_loc{i}      (2:end)-x_loc{i}      (1:end-1) ) ;
    delta_xUnique{i} =    x_locUnique{i}(2:end)-x_locUnique{i}(1:end-1);
    
    h{i}        = histcounts(delta_x{i}       ,histo_edges);
    hUnique{i}  = histcounts(delta_xUnique{i} ,histo_edges);
end
%
% (c) Metrics: mean value
hSum        = (sum(cat(1,h{:})      ,1)).'; % For each bin, sum up all histocounts
hSumUnique  = (sum(cat(1,hUnique{:}),1)).';
hMean       = histo_centers'*hSum      /sum(hSum); % Center of gravity / mean
hUniqueMean = histo_centers'*hSumUnique/sum(hSumUnique);


% Figures histograms longitudinal resolution 
% (a): overlay individual scans
figure
hold all
for i = 1 : length(dateinamen)
    bar(histo_centers,h{i},'EdgeColor',colors2(i,:),...
        'DisplayName',dateinamen(i),'EdgeAlpha',1,'FaceAlpha',0)   
end 
hold off
legend 
xlabel('\Deltax [mm]')
ylabel('Bin Count')
title(sprintf('Histogram of longitudinal resolution, non-sorted, mean \\Deltax = %2.2f mm',hMean))
%
% (b): Sorted and Unified dataset
figure
hold all
for i = 1 : length(dateinamen)
    bar(histo_centers,hUnique{i},'EdgeColor',colors2(i,:),...
        'DisplayName',dateinamen(i),'EdgeAlpha',1,'FaceAlpha',0)   
end
hold off
legend 
xlabel('\Deltax [mm]')
ylabel('Bin Count')
title(sprintf('Histogram of longitudinal resolution, sorted and unified, mean \\Deltax = %2.2f mm',hUniqueMean))

%% Coordsys transformation: from local to global
% Figures: raw data for manual setting of anchors
for i = 1: length(dateinamen) 
    figure
    plot(x_locSorted{i},z_locSorted{i}, ...
        'x','Color', colors(i,:), 'DisplayName',dateinamen(i))
    xlabel('x-Distance [mm]')
    ylabel('height z [mm]')
    legend('Location','bestoutside')
    title(sprintf('Profile in local coordsys from Laser downwards, expdate %s',exp_date))
end

% Anchors are saved in reference file. Read in or create new one
reference_file = [pfad.profiler_total,'loc_reference.xlsx'];
if ~isfile(reference_file)
    % Create new file with given x_global anchors for manual filling
    ref_xgl = plate_end;
    ref_xloc = zeros(length(plate_end),1);
    ref_zloc = zeros(length(plate_end),1);
    
    T = table(ref_xgl,ref_xloc,ref_zloc);
    writetable(T,reference_file)            
end
        
answer = questdlg('Anchors for conversion of coordinatesystems already set?', ...
	'','Continue','Define anchors','Continue');
switch answer
    case 'Continue'
        disp('Continuing')
    case 'Define anchors'
        msgbox(sprintf(['Define anchors for conversion from local to global coordsys\n' ...
            'using the profile images recently created\n\n', ...
            ['In loc_reference file define for each \n' ...
            'measurement location x_global the\n' ...
            'corresponding maximum positive x-location and ' ...
            '\ncorresponding z_location for plate'], ...
            '']))  
        winopen(reference_file)
        return
end

% Read in anchors: Max. positive x-loc and corresponding z-loc of plates
references = readmatrix(reference_file);
ref_xgl  = references(:,1);
ref_xloc = references(:,2);
ref_zloc = references(:,3);

%{
Trafo explication
Measurement direction: 
local x positive downstream  = positive global x
local z vertically downwards = negative global z

% Reference plate location 
% horizontal: ref_global [cm] from flume start (global x=0)
% vertical: flume height - plate depth = 300-75 = 225 mm over bottom (global z=0)
%}

for i = 1 :length(dateinamen)
    x_gl{i} = + (x_locSorted{i} - ref_xloc(i)) + 10*ref_xgl(i);
    z_gl{i} = - (z_locSorted{i} - ref_zloc(i)) + 225;
end

% analyse only measurements downstream of plate, cut off the rest
x_glcut = x_gl;
z_glcut = z_gl;
for i = 1 : length(dateinamen)
    delete_x = find(x_glcut{i}<10*plate_end(i));
    x_glcut{i}(delete_x) = [];
    z_glcut{i}(delete_x) = [];
end

% Output figure: Show individual measurements in global KOS
figure
hold all
for i = 1 : length(dateinamen)
    plot(x_glcut{i},z_glcut{i},'x','Color', colors2(i,:), 'DisplayName',dateinamen(i))
end
hold off

xlabel('x-Distance [mm]')
ylabel('height z [mm]')
legend('Location','bestoutside')
title('Profile in global coordsys from flume bottom upwards')
xlim(x_limits)
ylim(z_limits)
print('-r300',strcat(pfad.graphics,exp_date,'-profiler_joining'),'-dpng')

%% Enhance data quality
% Join datasets
x_profile = cat(1, x_glcut{:});
z_profile = cat(1, z_glcut{:});

% Remove obvious errors by deleting heights outside expected range
delete_z = vertcat(find(z_profile<z_limits(1)), find(z_profile>z_limits(2)));
x_profile(delete_z) = [];
z_profile(delete_z) = [];

% Sort dataset (requirement to analyse for unique values)
[x_profile, sortvector] = sort(x_profile);
z_profile = z_profile(sortvector);

% Take median z values at duplicated x-locations
[x_profile_unique,~,idx] = unique(x_profile,'stable');   % Find unique x values
z_profile_unique = accumarray(idx,z_profile,[],@median); % Take median z value

% Interpolate to uniform grid
x_interp = (x_limits(1):dx_interp:x_limits(2))';
gridInt_profile = griddedInterpolant(x_profile_unique,z_profile_unique,'linear')
z_interp = gridInt_profile(x_interp);

% Smooth profile using moveing mean over 10 mm
z_interp_mmean = movmean(z_interp,10);


% Graphical output final result laserscan
figure
hold all
scatter(x_profile, z_profile, 'rx','DisplayName','Raw Data')
scatter(x_interp,  z_interp,  'bx','DisplayName','Unique datapoints (median)')
plot(x_interp, z_interp_mmean,'g-','LineWidth',2,'DisplayName','Moving mean')
plot([3500,4000,4000,3500,3500],[100,100,160,160,100],'m','LineWidth',2,'DisplayName','FIS FOV')
hold off

legend('Location','northeast')
xlabel('Long. distance from flume inlet [mm]')
ylabel('Sediment elevation [mm]')
ylim([100,180])
title(sprintf('Result of laserscan merging %i individual datasets',length(x_gl)))

%% Analysing for bedform metrics
% (0) Analysing in "ROI": ignoring first and last bedform (high dynamics)
ROI_bedforms = (x_interp > ROI_xLim(1)) & (x_interp <= ROI_xLim(2));

% (1) Detecting crests and troughs
[crestVal, crestLoc]   = findpeaks( z_interp_mmean(ROI_bedforms),x_interp(ROI_bedforms),'MinPeakProminence',20);
[troughVal, troughLoc] = findpeaks(-z_interp_mmean(ROI_bedforms),x_interp(ROI_bedforms),'MinPeakProminence',20);
troughVal = -troughVal; % Because of -1 in findpeaks

% (2) Define waveforms from crest to crest = starts and ends at crest
%     --> delete detected troughs if outside this interval
if  troughLoc(1) < crestLoc(1)
    troughLoc(1) = [];
    troughVal(1) = [];
end
if  troughLoc(end) > crestLoc(end)
    troughLoc(end) = [];
    troughVal(end) = [];
end

% (3) Calculating bedform metrics
% (3a) Defining individual bedforms
ripple_len = crestLoc(2:end)-crestLoc(1:end-1);
ripple_nr = (1:length(ripple_len))';
%
crestLoc1  = crestLoc(1:end-1);
crestVal1  = crestVal(1:end-1);
crestLoc2  = crestLoc(2:end);
crestVal2  = crestVal(2:end);

% (3b) Distances and heights
ripple_trough2crest = crestLoc2 - troughLoc;
ripple_t2cPerctg = ripple_trough2crest./ripple_len*100;
ripple_amp = mean([crestVal1,crestVal2],2)-troughVal;

% (3c) Gather data in table, round for export
T = table(ripple_nr, ...
      crestLoc1,troughLoc,crestLoc2, ...
      crestVal1,troughVal,crestVal2, ...
      ripple_len, ripple_t2cPerctg, ripple_amp);
%
T.Variables = round(T.Variables);


% (4) Graphical output
% Figure bedform crests and troughs
fig_title = sprintf('Experiment %s, Sediment elevation from profiler',exp_date);

figure('Units','centimeters','Position',[10 10 8 8],'Name',fig_title)
hold all
scatter(x_interp,z_interp,'kx','MarkerEdgeAlpha',0.2,'DisplayName','Data points')
plot(x_interp,z_interp_mmean,'k-','DisplayName','Interpol Surface','LineWidth',2)
plot(crestLoc,crestVal,  'rx','LineWidth',2,'DisplayName','Crests')
plot(troughLoc,troughVal,'bo','LineWidth',2,'DisplayName','Troughs')
hold off

leg = legend('Location','northeast');
leg.Position = [0.5,0.8,0.3,0.2];
leg.NumColumns = 2;
leg.ItemTokenSize = [10,2];

xlim([2400,5000])
ylim(z_limits)
xlabel('Horizontal Position [mm]')
ylabel('Sediment elevantion [mm]')

set(gca,'DataAspectRatio',[50,1,1])
set(findall(gcf,'-property','FontSize'),'FontSize',11)

filename = strcat(pfad.graphics,exp_date,'-profiler_bedforms.png');
print('-r300',filename,'-dpng')

%% Save for comparison
close all
savefilename = [pfad.results,exp_date,'-profiler.mat'];
save(savefilename)

%% Export sediment elevation
% Exporting x-grid and sediment elevations (median + movmean (10)
export_profile = [x_interp'; z_interp'; z_interp_mmean'];
write_profile = strcat(pfad.results,exp_date,'-profiler_profile.dat');
write_bedform = strcat(pfad.results,exp_date,'-profiler_bedforms.dat');

fileID = fopen(write_profile,'w');
fprintf(fileID,'Exp-ID %6s, Sediment height positioning\n',exp_date);
fprintf(fileID,'Uniform x-grid, z-values are median of duplicate measurements\n');
fprintf(fileID,'%6s,%6s,%6s\n','x [mm] ','z [mm] ','z_mmean [mm] ');
fprintf(fileID,'%4.0f,%4.1f,%4.1f\n',export_profile);
fclose(fileID);

% Ripple bedforms: Use table, and add header
writetable(T,write_bedform,'Delimiter','tab')
temp  = fileread(write_bedform);
fileID = fopen(write_bedform,'w');
fprintf(fileID,'Exp-ID %6s, Ripple bedforms from profiler\n',exp_date);
fprintf(fileID,'Uniform x-grid, z-values are interpolated from median of duplicate measurements\n\n');
fwrite(fileID,temp);
fclose(fileID);

msgbox('Finished')
%EOF