% Jan-Pascal Boos, University of Bayreuth, Germany, 2023
% Available online: DOI: 10.5281/zenodo.10083568

%% Laserprofiler: Analyse bedform elevation during experiment
clear all; close all; clc

exp_date = '220901'
pfad = fct_pfad(exp_date);

dx_interp = 1; % mm grid spacing for interpolating
z_limits = [100,160];    % mm maximum elevation for bedforms

%% Read metadata
% Find all profiler datasets
vorhandene_dateien = dir([pfad.profiler_time,'*.txt']);
dateinamen         = string({vorhandene_dateien.name})';

% Read  metadata from labbook
labbook       = '../labbook.xlsx'; % Metadata file

sheets = sheetnames(labbook);
metadata = readcell(labbook,"Sheet",exp_date);
nr_profiles = metadata{strcmp(metadata,'nr_profiles'),2};
dt_PC = seconds(metadata{strcmp(metadata,'t_Lap = t_PC +'),2}); % s Time diff of PC to Laptop

exp_datetime = datetime(exp_date,'Format','yyMMdd');
tZ = metadata{strcmp(metadata,'Input Experiment'),2};
tZ = datetime(tZ+datenum(exp_datetime),'ConvertFrom','datenum');

tE = metadata{strcmp(metadata,'Ende Experiment'),2};
tE = datetime(tE+datenum(exp_datetime),'ConvertFrom','datenum');
dt_Experiment = minutes(tE-tZ);


% Extract x-anchor and time from profiles: format: 123_yyMMdd-HHmmss.txt
% (1) x-anchor
plate_end = str2double(extractBefore(dateinamen,'_'));
if ~isempty(find(plate_end-plate_end(1), 1))
    error('Format of datanames is not consistent: x_loc changes, please check')
else
    plate_end = plate_end(1);
end

% (2) Time info
% (2a) Extract from filename
timeFormat = 'yyMMdd-HHmmss';
%
t_Profile = extractBefore(extractAfter( dateinamen,'_'),'.');
t_Profile = datetime(t_Profile,'InputFormat', timeFormat);
t_Profile = minutes( t_Profile + dt_PC - tZ);
%
% (2b) Cut dataset to one point every minute
t_min = 0; %fix(min(t_Profile))
t_max = dt_Experiment; %fix(max(t_Profile))
%
t_Profile_min = t_min:1:t_max;
[~,idx_minute] = min(abs(t_Profile-t_Profile_min));
%
t_Profile  =  t_Profile(idx_minute);
dateinamen = dateinamen(idx_minute);


% Some colors for nice plots
colors  = jet(length(dateinamen));

%% Show raw data: horizontal range 
% Range of measurement in longitudinal direction (X Range) for complete
% dataset, consisting of 10 measurements each
figure
hold all
for i = 1 : length(dateinamen)
    rawdata  = readmatrix(strcat(pfad.profiler_time,dateinamen(i)));
    x_loc{i} = rawdata(:,1);
    z_loc{i} = rawdata(:,3);
    len = repmat((1:size(rawdata,1) / nr_profiles)',nr_profiles,1);
    plot(len,x_loc{i},'x', 'Color', colors(i,:),'DisplayName',dateinamen(i))    
end
hold off
legend('Interpreter','none','NumColumns',3)
xlabel('Value Nr.')
ylabel('x distance')
title('Horizontal Range of measurement')


% Deleting 0 values
fprintf('Deleting (0,0) tupels')
for i = 1 : length(dateinamen)
    delete_x = find(x_loc{i}==0);
    fprintf('Deleting %i (%i) 0''s at %s\n',length(delete_x),length(x_loc{i}),dateinamen(i));
    x_loc{i}(delete_x) = [];
    z_loc{i}(delete_x) = [];
end

%% Display raw data, dialog for setting anchors
% Figures: raw data for manual setting of anchors (from local to global coordsys)
fig = figure('Visible','on');
plot(x_loc{1},z_loc{1}, ...
    'x','Color', 'r', 'DisplayName',dateinamen(i))
xlabel('x-Distance [mm]')
ylabel('height z [mm]')
legend('Location','bestoutside','Interpreter','none')
title(sprintf('Profile in local coordsys from Laser downwards, expdate %s',exp_date))
zoom on

msgtext = sprintf([...
    'Define anchors for conversion from local to global coordsys \n' ...
    'using the profile images recently created                   \n\n',...
    'In loc_reference file define for each                       \n' ...
    'measurement location x_global the                           \n' ...
    'corresponding maximum positive x-location and               \n' ...
    'corresponding z_location for plate (without decimals)']);

reference_file = [pfad.profiler_time,'loc_reference_time.xlsx'];

if ~isfile(reference_file)
    % Create new file with given x_global anchors
    ref_xgl = plate_end;
    ref_xloc = zeros(length(plate_end),1);
    ref_zloc = zeros(length(plate_end),1);
    
    T = table(ref_xgl,ref_xloc,ref_zloc);
    writetable(T,reference_file)

    msgbox(msgtext)
    winopen(reference_file)
    return   
end
        
answer = questdlg('Anchors for conversion of coordinatesystems already set?', ...
	'','Continue','Define anchors','Continue');
switch answer
    case 'Continue'
        disp('Continuing')
        close(gcf)
    case 'Define anchors' % Should be automized, for now: manual setting
        msgbox(msgtext)   
        winopen(reference_file)
        return
end

%% Coordsys transformation: from local to global
% Read in anchors Max. positive x-loc and related z-loc of plates
references = readmatrix(reference_file);
ref_xgl  = references(:,1);
ref_xloc = references(:,2);
ref_zloc = references(:,3);

%{
Trafo explication
Measurement direction: 
local x positive downstream  = positive global x
local z vertically downwards = negative global z

% Reference plate location 
% horizontal: ref_global [cm] from flume start (global x=0)
% vertical: flume height - plate depth = 300-75 = 225 mm over bottom (global z=0)
%}

for i = 1 :length(dateinamen)
    x_gl{i} = + (x_loc{i} - ref_xloc) + 10*ref_xgl;
    z_gl{i} = - (z_loc{i} - ref_zloc) + 225;
end

% analyse only measurements downstream of plate, cut off the rest
x_glcut = x_gl;
z_glcut = z_gl;
for i = 1 : length(dateinamen)
    delete_x = find(x_glcut{i}<10*plate_end);
    x_glcut{i}(delete_x) = [];
    z_glcut{i}(delete_x) = [];
end

x_limits = round([min(cellfun(@min,x_glcut)), max(cellfun(@max,x_glcut))]);
x_interp = (x_limits(1):dx_interp:x_limits(2))';

% Output figure, joining individual measurements in global KOS
figure
hold all
for i = 1 : length(dateinamen)
    plot(x_glcut{i},z_glcut{i},'x','Color', colors(i,:), ...
        'DisplayName',dateinamen(i))
end
hold off

xlabel('x-Distance [mm]')
ylabel('height z [mm]')
legend('Location','best','Interpreter','none','NumColumns',3)
title('Profile in global coordsys from flume bottom upwards')
xlim(x_limits)
set(gca,'XDir','reverse')

%% Enhance data quality
z_interp = zeros(length(x_interp),length(dateinamen));
z_interp_mmean = zeros(length(x_interp),length(dateinamen));
for i = 1 : length(dateinamen)
    % Join datasets
    x_profile = cat(1, x_glcut{i});
    z_profile = cat(1, z_glcut{i});
    
    % Remove "noise" by deleting heights outside expected range
    delete_z = vertcat(find(z_profile<z_limits(1)), find(z_profile>z_limits(2)));
    x_profile(delete_z) = [];
    z_profile(delete_z) = [];
    
    % Sort dataset (requirement to analyse for unique values)
    [x_profile, sortvector] = sort(x_profile);
    z_profile = z_profile(sortvector);
    
    % Take median z values at duplicated x-locations
    [x_profile_unique,~,idx] = unique(x_profile,'stable');   % Find unique x values
    z_profile_unique = accumarray(idx,z_profile,[],@median); % Take median z value
    
    % Interpolate to uniform grid
    gridInt_profile = griddedInterpolant(x_profile_unique,z_profile_unique,'linear');
    z_interp(:,i) = gridInt_profile(x_interp);
    
    % Smooth profile with moving mean over 10 mm
    z_interp_mmean(:,i) = movmean(z_interp(:,i),10);
end

% Bedform elevation, temporal mean over all timepoints
z_interp_mmean_all = mean(z_interp_mmean,2);

%% Graphical output laserscan during experiment
figure
hold all
for i = 1 : length(dateinamen)
    plot(x_interp, z_interp_mmean(:,i), 'Color',colors(i,:), ...
        'LineWidth',1,'DisplayName',num2str(t_Profile(i),'%.0f'))  
end
plot(x_interp,z_interp_mmean_all,'k','LineWidth',2,'DisplayName','mean')
hold off

leg = legend('Location','bestoutside','NumColumns',2);
leg.Title.String = 't [min]';
leg.ItemTokenSize = [10,4];
xlabel('Long. distance from flume inlet [mm]');
ylabel('Sediment elevation [mm]');
title(sprintf('Result of laserscan at %i individual timepoints',length(x_gl)));
ylim([100,160])

print('-r300',strcat(pfad.graphics,exp_date,'-profiler_timemean'),'-dpng')

%% Export sediment elevation over time to csv files
% Exporting x-grid and corresponding median values for mean profile
export_profile = fct_export(x_interp,z_interp_mmean_all);
fileName = strcat(pfad.results,exp_date,'-profiler_timemean.dat');

fileID = fopen(fileName,'w');
fprintf(fileID,'%% Exp-ID %6s, Sediment height positioning, mean over experiment\n',exp_date);
fprintf(fileID,'%% Uniform x-grid, z-values are median of duplicate measurements\n');
fprintf(fileID,'%% %6s\t%6s\n','x [mm]','z [mm]');
fprintf(fileID,'%4.0f\t%4.1f\n',export_profile);
fclose(fileID);

%% Time comparison: during experiment - afterwards
profile_mean = readmatrix(strcat(pfad.results,exp_date,"-profiler_timemean.dat"));
profile_all  = readmatrix(strcat(pfad.results,exp_date,"-profiler_profile.dat"));

xmin = min(profile_mean(:,1));
xmax = max(profile_mean(:,1));

figure
hold all
plot(profile_all(:,1), profile_all(:,2),'r','DisplayName', 'After experiment')
plot(profile_mean(:,1),profile_mean(:,2),'b','DisplayName','During experiment')
hold off
xlim([xmin, xmax])
xlim([2600,3200])
ylim([100,150])
xlbl = xlabel('Longitudinal position [mm]');
ylbl = ylabel('Vertical position [mm]');
xlbl.Position(2) = 90; 
ylbl.Position(1) = 3260;
leg = legend('Position',[0.66,0.8,0.2,0.10]);
leg.ItemTokenSize = [10,4];
%title('Comparison sediment elevation during and after experiment')
set(gca,'XDir','reverse')
set(gca,'DataAspectRatio',[5 1 1])

set(gcf,'units','centimeters','position',[0,0,13,8])
set(findall(gcf,'-property','FontSize'),'FontSize',11)
fullfilename = strcat(pfad.graphics,exp_date,'-profiler_comparison.png');
print('-r300',fullfilename,'-dpng')

close all
msgbox('Finished')
%EOF